# -*- coding: utf-8 -*-
#	Copyright 2005 Spike^ekipS <spikeekips@gmail.com>
#
#	   This program is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; either version 2 of the License, or
#	(at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import sqlobject
import database
import config
import channel_entry_enclosures

QUERY_TRIGGER_CREATE = \
"""
CREATE TRIGGER updateEntry_%(id_channel)s AFTER INSERT ON `channel_entry_%(id_channel)s`
  FOR EACH ROW BEGIN
	 INSERT INTO entry_pool SET channel_id = NEW.channel_id, entry_id = NEW.id, time_updated = NEW.time_updated, time_added = now();
  END;
"""

QUERY_TRIGGER_DROP = \
"""
DROP TRIGGER updateEntry_%(id_channel)s
"""

class ChannelEntry (sqlobject.SQLObject) :
	_connection = config.database["entry"]

	channelID		= sqlobject.IntCol(notNone=True)
	author			= sqlobject.UnicodeCol(default="", notNone=False)
	uid				= sqlobject.UnicodeCol(default="", notNone=False)
	link				= sqlobject.UnicodeCol(default="", notNone=True)
	links				= sqlobject.UnicodeCol(default="", notNone=False)
	publisher		= sqlobject.UnicodeCol(default="", notNone=False)
	subtitle			= sqlobject.UnicodeCol(default="", notNone=False)
	summary			= sqlobject.UnicodeCol(default="", notNone=False)
	timeUpdated		= sqlobject.DateTimeCol(default=None, notNone=False)
	title				= sqlobject.UnicodeCol(default="", notNone=False)
	tags				= sqlobject.UnicodeCol(default="", notNone=False)

	index0			= sqlobject.DatabaseIndex({'column': "uid", 'length': 200})
	index1			= sqlobject.DatabaseIndex({'column': "link", 'length': 200})
	index2			= sqlobject.DatabaseIndex({'column': "timeUpdated"})
	index3			= sqlobject.DatabaseIndex({'column': "title", "length": 200})

	isRead = False
	isSkip = False
	isStar = False

	enclosures = None
	audios = None

	def getEnclosuresAudable (self) :
		__audios = list()
		if self.enclosures is None :
			self.getEnclosures()

		if len(self.enclosures) < 1 :
			return __audios
		else :
			for i in self.enclosures :
				if i.isAudable() :
					__audios.append(i)

		return __audios

	def getEnclosures (self) :
		__e = channel_entry_enclosures.ChannelEntryEnclosures
		self.enclosures = list( \
			__e.select( \
				sqlobject.AND( \
					__e.q.channelID == self.channelID, \
					__e.q.entryID == self.id \
				) \
			) \
		)

		if len(self.enclosures) > 0 :
			return True

		return False

	def getIdChannel (self) :
		try :
			return int(self._table.replace("channel_entry_", ""))
		except :
			return None

	getIdChannel = classmethod(getIdChannel)

	def getConnection (self) :
		if not hasattr(self, "_conn") :
			__conn = database.get_connection(config.database["entry"])
		else :
			__conn = getattr(self, "_conn")

		return __conn

	getConnection = classmethod(getConnection)

	def createTableNew ( \
			self, ifNotExists=False, createJoinTables=True,
			createIndexes=True, connection=None) :

		if not connection.tableExists(self.sqlmeta.table) :
			self.createTable(ifNotExists, createJoinTables, createIndexes, connection)
			self.createTrigger()

	createTableNew = classmethod(createTableNew)

	def createTrigger (self) :
		# create trigger.
		__conn = self.getConnection()

		__id_channel = self.getIdChannel()
		try :
			__conn.queryAll(QUERY_TRIGGER_DROP % {"id_channel" : __id_channel})
		except :
			pass

		try :
			__conn.queryAll(QUERY_TRIGGER_CREATE % {"id_channel" : __id_channel})
		except :
			return False

		return True

	createTrigger = classmethod(createTrigger)

SKEL_TABLENAME_ENTRIES = "channel_entry_%s"
def get_entry (id_channel) :
	__model_entry = get_entry_model(SKEL_TABLENAME_ENTRIES % id_channel)

	__conn = database.get_connection(config.database["entry"])
	__model_entry.createTableNew( \
		ifNotExists=True, \
		createIndexes=True, \
		connection= __conn \
	)

	return __model_entry

def get_entry_model (tablename) :
	return database.get_model(database.get_table("channel_entry"), tablename)


"""
Description
-----------


ChangeLog
---------


Usage
-----


"""

__author__ =  "Spike^ekipS <spikeekips@gmail.com>"
__version__=  "0.1"
__nonsense__ = ""
