# -*- coding: utf-8 -*-
#	Copyright 2005 Spike^ekipS <spikeekips@gmail.com>
#
#	   This program is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; either version 2 of the License, or
#	(at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import sqlobject
import database

from model.channel_entry import get_entry as get_model_channel_entry
from model.memo_user import get_model as get_model_memo_user

import config

SKEL_TABLENAME = "channel_entry_%s"
URI = config.database["user"]

class ChannelEntryUser (sqlobject.SQLObject) :
	_connection = URI

	channelID		= sqlobject.IntCol(notNone=True)
	entryID			= sqlobject.IntCol(notNone=True)
	timeUpdated		= sqlobject.DateTimeCol(default=None, notNone=False)
	isRead			= sqlobject.BoolCol(default=False, notNone=False)
	isStar			= sqlobject.BoolCol(default=False, notNone=False)
	isSkip			= sqlobject.BoolCol(default=False, notNone=False)

	index0			= sqlobject.DatabaseIndex({'column': "timeUpdated"})
	index1			= sqlobject.DatabaseIndex({'column': "channelID"})
	index2			= sqlobject.DatabaseIndex({'column': "entryID"})
	index3			= sqlobject.DatabaseIndex({'column': "isRead"})
	index3			= sqlobject.DatabaseIndex({'column': "isStar"})
	index3			= sqlobject.DatabaseIndex({'column': "isSkip"})

	def getUserID (self) :
		try :
			return int(self.sqlmeta.table.replace("channel_entry_", ""))
		except :
			return None

	def getEntry_new (self, id_channel, id_entry) :
		__id_user = self.getUserID()

		# get channel_entry_user
		__model_memo_user = get_model_memo_user(__id_user)

		__entry_user = self.getEntry(id_channel, id_entry)
		__entry = get_model_channel_entry(id_channel).get(id_entry)

		# findout is audable?
		__entry.audios = __entry.getEnclosuresAudable()
		__has_memo = \
			__model_memo_user.hasMemo( \
				**{"entryID" : id_entry, "channelID" : id_channel} \
			)

		return (__entry_user, __entry, __has_memo, )

	def getEntry (self, id_channel, id_entry) :
		try :
			return list( \
				self.select( \
					sqlobject.AND( \
						self.q.channelID == id_channel, \
						self.q.entryID == id_entry, \
					) \
				).limit(1) \
			)[0]
		except :
			return None

	def setSkip (self, id_channel, id_entry, isSkip=True) :
		# check whether the row, which has id_channel & id_entry exists or not
		__o = self.select( \
			sqlobject.AND( \
				self.q.channelID == id_channel, \
				self.q.entryID == id_entry, \
			) \
		).limit(1)

		if __o.count() > 0 :
			__o = list(__o)[0]
			if __o.isSkip != isSkip :
				__o.isSkip = isSkip

			return __o
		else :
			# get entry info
			__entry = get_model_channel_entry(id_channel).get(id_entry)
			if not __entry :
				return False;

			__kwargs = { \
				"channelID" : id_channel, \
				"entryID" : id_entry, \
				"timeUpdated" : __entry.timeUpdated, \
				"isSkip" : isSkip, \
			}

			return self(**__kwargs)


	def setStar (self, id_channel, id_entry, isStar=True) :
		# check whether the row, which has id_channel & id_entry exists or not
		__o = self.select( \
			sqlobject.AND( \
				self.q.channelID == id_channel, \
				self.q.entryID == id_entry, \
			) \
		).limit(1)

		if __o.count() > 0 :
			__o = list(__o)[0]
			if __o.isStar != isStar :
				__o.isStar = isStar

			return __o
		else :
			# get entry info
			__entry = get_model_channel_entry(id_channel).get(id_entry)
			if not __entry :
				return False;

			__kwargs = { \
				"channelID" : id_channel, \
				"entryID" : id_entry, \
				"timeUpdated" : __entry.timeUpdated, \
				"isStar" : isStar, \
				"isRead" : True, \
			}

			return self(**__kwargs)

	def setRead (self, id_channel, id_entry, isRead=True) :
		# check whether the row, which has id_channel & id_entry exists or not
		__o = self.select( \
			sqlobject.AND( \
				self.q.channelID == id_channel, \
				self.q.entryID == id_entry, \
			) \
		).limit(1)

		if __o.count() > 0 :
			__o = list(__o)[0]
			if __o.isRead != isRead :
				__o.isRead = isRead

			return __o
		else :
			# get entry info
			__entry = get_model_channel_entry(id_channel).get(id_entry)
			if not __entry :
				return False;

			__kwargs = { \
				"channelID" : id_channel, \
				"entryID" : id_entry, \
				"timeUpdated" : __entry.timeUpdated, \
				"isRead" : isRead, \
			}

			return self(**__kwargs)

	getUserID 				= classmethod(getUserID)
	getEntry_new = classmethod(getEntry_new)
	getEntry = classmethod(getEntry)
	setSkip = classmethod(setSkip)
	setRead = classmethod(setRead)
	setStar = classmethod(setStar)

def get_model (id_user) :
	__model = database.get_model( \
		database.get_table("channel_entry_user", "ChannelEntryUser"), \
		SKEL_TABLENAME % id_user \
	)

	__conn = database.get_connection(URI)
	__model.createTable( \
		ifNotExists=True, \
		createIndexes=True, \
		connection= __conn \
	)

	return __model


"""
Description
-----------


ChangeLog
---------


Usage
-----


"""

__author__ =  "Spike^ekipS <spikeekips@gmail.com>"
__version__=  "0.1"
__nonsense__ = ""
