# -*- coding: utf-8 -*-
#    Copyright 2005 Spike^ekipS <spikeekips@gmail.com>
#
#       This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import sqlobject
import database, config

from model.label_user import get_model as get_model_label_user

SKEL_TABLENAME = "channel_label_%s"
URI = config.database["user"]

class ChannelLabelUser (sqlobject.SQLObject) :
	_connection = URI

	channelID		= sqlobject.ForeignKey("Channel")
	labelID			= sqlobject.ForeignKey("LabelUser")
	seq				= sqlobject.IntCol(default=0)

	index0			= sqlobject.DatabaseIndex({'column': "channelID"})
	index1			= sqlobject.DatabaseIndex({'column': "labelID"})

	name = None

	def getLabels (self, id_channel) :
		__id_user = self.getUserID()

		__l = list( \
			self.select( \
				self.q.channelID == id_channel, \
				orderBy=self.q.seq, \
				reversed=False \
			) \
		)

		__label_user = get_model_label_user(__id_user)
		for i in __l :
			i.name = __label_user.get(i.labelID).name

		return __l

	getLabels = classmethod(getLabels)

	def getUserID (self) :
		try :
			return int(self.sqlmeta.table.replace("channel_label_", ""))
		except :
			return None

	def removeLabel (self, id_channel, id_label) :
		# At first, insert label_user table.
		try :
			__channel_label = list( \
				self.select( \
					sqlobject.AND( \
						self.q.labelID == id_label, \
						self.q.channelID == id_channel \
					) \
				).limit(1) \
			)[0]
		except IndexError : # create new
			return False
		else :
			__seq = __channel_label.seq
			__channel_label.destroySelf()

			# reset 'seq', is it needed ?
			__channel_labels_0 = list( \
				self.select( \
					sqlobject.AND( \
						self.q.channelID == id_channel, \
						self.q.seq > __seq \
					), \
					orderBy=self.q.seq, \
					reversed=False \
				) \
			)

			for i in __channel_labels_0 :
				i.seq -= 1

			return True

	def setLabel (self, id_channel, name_label) :
		__id_user = self.getUserID()

		__name_label = str(name_label).strip()
		# At first, insert label_user table.
		__model_label_user = get_model_label_user(__id_user)
		try :
			__label = list( \
				__model_label_user.select( \
					__model_label_user.q.name == __name_label \
				).limit(1) \
			)[0]
		except IndexError : # create new
			__kwargs = { \
				"name" : __name_label, \
			}
			__label = __model_label_user(**__kwargs)

		try :
			__channel_label = list( \
				self.select( \
					sqlobject.AND( \
						self.q.channelID == id_channel, \
						self.q.labelID == __label.id \
					), \
				) \
			)[0]
		except :
			pass
		else :
			return __channel_label

		# insert data into channel_label
		try :
			__channel_label_prev = list( \
				self.select( \
					self.q.channelID == id_channel, \
					orderBy=self.q.seq, \
					reversed=True \
				) \
			)[0]
		except IndexError, e :
			__seq = 0
		else :
			# insert by sequence
			__seq = __channel_label_prev.seq + 1

		__kwargs = { \
			"channelID" : id_channel, \
			"labelID" : __label.id, \
			"seq" : __seq, \
		}

		return self(**__kwargs)

	def setSeq (self, id_channel, id_label, seq=0) :
		try :
			__channel_label = list( \
				self.select( \
					sqlobject.AND( \
						self.q.channelID == id_channel, \
						self.q.labelID == id_label \
					) \
				) \
			)[0]
		except IndexError, e : # no exist.
			return False
		else :
			if __channel_label.seq == seq :
				return __channel_label
			else :
				# re-arrange sequences from seq to last one.
				__num = self.select( \
						self.q.channelID == id_channel, \
				).count()
				if seq > __num : seq = __num - 1
				if seq < 0 : seq = 0

				if __channel_label.seq > seq :
					__tok = seq + 1

					__clause_seq = sqlobject.AND( \
						self.q.seq >= seq, \
						self.q.seq < __channel_label.seq, \
					)
				else :
					__tok = __channel_label.seq

					__clause_seq = sqlobject.AND( \
						self.q.seq >= __channel_label.seq, \
						self.q.seq <= seq, \
					)

				__channel_labels = list( \
					self.select( \
						sqlobject.AND( \
							self.q.channelID == id_channel, \
							self.q.labelID != id_label, \
							__clause_seq, \
						), \
						orderBy=self.q.seq, \
						reversed=False \
					) \
				)

				for i in __channel_labels :
					i.seq = __tok
					__tok += 1

				__channel_label.seq = seq

				return __channel_label

	removeLabel = classmethod(removeLabel)
	setLabel = classmethod(setLabel)
	setSeq = classmethod(setSeq)
	getUserID = classmethod(getUserID)

def get_model (id_user) :
	__model = database.get_model( \
		database.get_table("channel_label_user", "ChannelLabelUser"), \
		SKEL_TABLENAME % id_user \
	)

	__conn = database.get_connection(URI)
	__model.createTable( \
		ifNotExists=True, \
		createIndexes=True, \
		connection= __conn \
	)

	return __model


"""
Description
-----------


ChangeLog
---------


Usage
-----


"""

__author__ =  "Spike^ekipS <spikeekips@gmail.com>"
__version__=  "0.1"
__nonsense__ = ""
