# -*- coding: utf-8 -*-
#    Copyright 2005 Spike^ekipS <spikeekips@gmail.com>
#
#       This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import sqlobject
import database, config
from model.channel import Channel as model_channel
from model.channel_label_user import get_model as get_model_channel_label_user
from document import channel_func as document_channel_func

SKEL_TABLENAME = "channel_%s"

URI = config.database["user"]

class ChannelUserNull :
	main = None
	labels = list()
	image = None

class ChannelUser (sqlobject.SQLObject) :
	_connection = URI

	channelID		= sqlobject.ForeignKey("Channel")
	title				= sqlobject.UnicodeCol(default="", notNone=False)
	timeAdded		= sqlobject.DateTimeCol(notNone=True)
	showFrontpage	= sqlobject.BoolCol(default=True, notNone=True)
	removed			= sqlobject.BoolCol(default=False, notNone=False)

	index0		= sqlobject.DatabaseIndex({'column': "channelID"})
	index1		= sqlobject.DatabaseIndex({'column': "title", "length" : 200})
	index2		= sqlobject.DatabaseIndex({'column': "timeAdded"})
	index3		= sqlobject.DatabaseIndex({'column': "removed"})

	main = None
	labels = list()

	def getUserID (self) :
		try :
			return int(self.sqlmeta.table.replace("channel_", ""))
		except :
			return None

	"""
	def getChannel (self, id_channel) :
		try :
			__channel = list( \
					self.select( \
						self.q.channelID == id_channel \
				).limit(1) \
			)[0]
		except IndexError :
			return None
		else :
			return __channel
	"""

	def setTitle (self, id_channel, title) :
		# check whether the row, which has id_channel & id_entry exists or not
		__channel = self.getChannel(id_channel)
		if not __channel :
			return False
		else :
			if __channel.title != title.strip() :
				__channel.title = title.strip()

			return __channel

	def setRemoved (self, id_channel, remove=True) :
		# check whether the row, which has id_channel & id_entry exists or not
		__channel = self.getChannel(id_channel)
		if not __channel :
			return False
		else :
			if __channel.removed != remove :
				__channel.removed = remove

			return __channel

	def setShowFrontpage (self, id_channel, showFrontpage=True) :
		# check whether the row, which has id_channel & id_entry exists or not
		__channel = self.getChannel(id_channel)
		if not __channel :
			return False
		else :
			if __channel.showFrontpage != showFrontpage :
				__channel.showFrontpage = showFrontpage

			return __channel

	def isExists (self, id_channel) :
		try :
			__channel = list( \
				self.select( \
					self.q.channelID == id_channel \
				).limit(1) \
			)[0]
		except :
			return False
		else :
			if not __channel :
				return False
			else :
				return __channel

	def getChannelIDs (self) :
		__o = list(self.select())
		__ids = list()
		for i in __o :
			__ids.append(i.channelID)

		return __ids

	def getChannelList (self, clause=None, **kw) :
		__limit = kw.get("limit", None)
		__offset = kw.get("offset", 0)

		if kw.has_key("limit") : del kw["limit"]
		if kw.has_key("offset") : del kw["offset"]

		__r = self.select(clause, **kw)
		if __limit :
			__l = list(__r)[__offset:__offset + __limit]
		else :
			__l = list(__r)[__offset:]

		__new = list()
		for i in __l :
			__o = self.getChannel(i.channelID)
			if not __o :
				continue

			__new.append(__o)

		return __new

	def getChannel (self, id_channel) :
		try :
			__channel_main = model_channel.get(id_channel)
		except :
			return None

		__id_user = self.getUserID()

		try :
			__channel = list( \
				self.select( \
					self.q.channelID == id_channel \
				).limit(1) \
			)[0]
		except :
			__channel = ChannelUserNull()

		# get channel data.
		__channel.main = __channel_main

		# get label data
		__channel_label_user = get_model_channel_label_user(__id_user)
		__channel.labels = __channel_label_user.getLabels(id_channel)

		# get label image
		__channel.image = document_channel_func.get_image(__channel.main)

		return __channel

	getUserID 				= classmethod(getUserID)
	getChannel				= classmethod(getChannel)
	getChannelList			= classmethod(getChannelList)
	getChannelIDs			= classmethod(getChannelIDs)
	isExists					= classmethod(isExists)
	setShowFrontpage		= classmethod(setShowFrontpage)
	setRemoved				= classmethod(setRemoved)
	setTitle				= classmethod(setTitle)

def get_model (id_user) :
	__model = database.get_model( \
		database.get_table("channel_user", "ChannelUser"), \
		SKEL_TABLENAME % id_user \
	)

	__conn = database.get_connection(URI)
	__model.createTable( \
		ifNotExists=True, \
		createIndexes=True, \
		connection= __conn \
	)

	return __model

if __name__ == "__main__" :
	print ChannelUser.createTable()


"""
Description
-----------


ChangeLog
---------


Usage
-----


"""

__author__ =  "Spike^ekipS <spikeekips@gmail.com>"
__version__=  "0.1"
__nonsense__ = ""
