# -*- coding: utf-8 -*-
#    Copyright 2005 Spike^ekipS <spikeekips@gmail.com>
#
#       This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import sqlobject
import database
import config

from model.channel import Channel as model_channel
from model.channel_user import get_model as get_model_channel_user
from model.channel_entry import get_entry
from model.channel_entry_user import get_model as get_model_channel_entry_user
from model.memo_user import get_model as get_model_memo_user

class EntryPool (sqlobject.SQLObject) :
	_connection = config.database["entry"]

	channelID		= sqlobject.IntCol(notNone=True)
	entryID			= sqlobject.IntCol(notNone=True)
	timeUpdated		= sqlobject.DateTimeCol(default=None, notNone=False)
	timeAdded		= sqlobject.DateTimeCol(default=None, notNone=False)

	index0		= sqlobject.DatabaseIndex({'column': "channelID"})
	index1		= sqlobject.DatabaseIndex({'column': "entryID"})
	index2		= sqlobject.DatabaseIndex({'column': "timeUpdated"})
	index3		= sqlobject.DatabaseIndex({'column': "timeAdded"})

	entry = None
	entry_user = None
	channel = None
	channel_user = None
	hasMemo = None

	def getOne (self, id_channel, id_entry) :
		try :
			return list( \
				self.select(
					sqlobject.AND( \
						self.q.channelID == id_channel, \
						self.q.entryID == id_entry, \
					) \
				).limit(1)
			)[0]
		except :
			return None

	def getChannels (self, id_user) :
		__model_channel_user = get_model_channel_user(id_user)
		return __model_channel_user.getChannel(self.channelID)
		#return (self.getChannel(), self.getChannelUser(id_user), )

	def getChannelUser (self, id_user) :
		try :
			__channel = list( \
				model_channel_user.select( \
					sqlobject.AND( \
						model_channel_user.q.userID == id_user, \
						model_channel_user.q.channelID == self.channelID, \
					) \
				) \
			)[0]
		except :
			return None

	def getChannel (self) :
		return model_channel.get(self.channelID)

	def getEntry (self, id_user) :
		# get channel_entry_user
		__model_memo_user = get_model_memo_user(id_user)
		__model_channel_entry_user = get_model_channel_entry_user(id_user)

		__entry_user = __model_channel_entry_user.getEntry(self.channelID, self.entryID)
		try :
			__entry = get_entry(self.channelID).get(self.entryID)
		except :
			return (None, None, None, )

		# findout is audable?
		__entry.audios = __entry.getEnclosuresAudable()
		__has_memo = \
			__model_memo_user.hasMemo( \
				**{"entryID" : self.entryID, "channelID" : self.channelID} \
			)
		return (__entry_user, __entry, __has_memo, )

	getOne = classmethod(getOne)


"""
Description
mysql> desc entry_pool;
+--------------+----------+------+-----+---------+----------------+
| Field        | Type     | Null | Key | Default | Extra          |
+--------------+----------+------+-----+---------+----------------+
| id           | int(11)  | NO   | PRI | NULL    | auto_increment |
| channel_id   | int(11)  | NO   |     |         |                |
| entry_id     | int(11)  | NO   |     |         |                |
| time_updated | datetime | YES  |     | NULL    |                |
+--------------+----------+------+-----+---------+----------------+


ChangeLog
---------


Usage
-----


"""

__author__ =  "Spike^ekipS <spikeekips@gmail.com>"
__version__=  "0.1"
__nonsense__ = ""
