# -*- coding: utf-8 -*-
#    Copyright 2005 Spike^ekipS <spikeekips@gmail.com>
#
#       This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import sqlobject, sqlobject.sqlbuilder
import database, config
from util import datetime__
SKEL_TABLENAME = "memo_%s"

from model.memo_property_user import get_model as get_model_memo_property_user
from model.memo_history_user import get_model as get_model_memo_history_user

URI = config.database["user"]

class MemoUser (sqlobject.SQLObject) :
	_connection = URI

	title			= sqlobject.UnicodeCol(notNone=False)
	content		= sqlobject.UnicodeCol(notNone=False)
	timeAdded	= sqlobject.DateTimeCol(default=None, notNone=False)

	index0		= sqlobject.DatabaseIndex({'column': "title", "length" : 200})

	def getUserID (self) :
		try :
			return int(self.sqlmeta.table.replace("memo_", ""))
		except :
			return None

	def removeMemo (self, id_memo) :
		__memo = self.get(id_memo)

		__id_user = self.getUserID()
		__memo_property = get_model_memo_property_user(__id_user)

		__list = list( \
				__memo_property.select( \
					__memo_property.q.memoID == __memo.id \
				) \
			)

		for i in __list :
			i.destroySelf()

		__memo.destroySelf()

		return True

	def insert (self, title, content, properties=dict()) :
		__kwargs = { \
			"title" : title, \
			"content" : content, \
			"timeAdded" : datetime__.now(), \
		}

		__memo = self(**__kwargs)

		# set property
		__id_user = self.getUserID()
		__memo_property = get_model_memo_property_user(__id_user)
		__memo_property.setProperties(__memo.id, properties)

		return __memo

	def hasMemo (self, **properties) :
		if not properties :
			return None

		__id_user = self.getUserID()
		__memo_property = get_model_memo_property_user(__id_user)

		__condition = None
		for __k, __v in properties.items() :
			__c = sqlobject.AND( \
				__memo_property.q.type == __k,
				__memo_property.q.value == __v,
			)

			if not __condition :
				__condition = __c
				continue
			else :
				__condition = sqlobject.AND( \
					__memo_property.q.type == __k,
					__memo_property.q.value == __v,
					sqlobject.sqlbuilder.IN(
						__memo_property.q.memoID,
						sqlobject.sqlbuilder.Select( \
							__memo_property.q.memoID, \
							where = __condition \
						) \
					) \
				)

		__num = __memo_property.select( \
			sqlobject.AND(
				__condition, \
				self.q.id == __memo_property.q.memoID, \
			) \
		).count()

		return __num > 0

	def getList (self) :
		__memos = list( \
			self.select( \
				orderBy=self.q.timeAdded, \
				reversed=True, \
			) \
		)

		__id_user = self.getUserID()
		__memo_property = get_model_memo_property_user(__id_user)

		for i in __memos :
			__p = list( \
				__memo_property.select( \
					__memo_property.q.memoID == i.id \
				) \
			)

			# transform to dict().
			__dict = dict()
			[__dict.update({pp.type: pp.value}) for pp in __p]

			i.property = __dict

		return __memos

	def getMemos (self, **properties) :
		if not properties :
			return None

		__id_user = self.getUserID()
		__memo_property = get_model_memo_property_user(__id_user)

		__condition = None
		for __k, __v in properties.items() :
			__c = sqlobject.AND( \
				__memo_property.q.type == __k,
				__memo_property.q.value == __v,
			)

			if not __condition :
				__condition = __c
				continue
			else :
				__condition = sqlobject.AND( \
					__memo_property.q.type == __k,
					__memo_property.q.value == __v,
					sqlobject.sqlbuilder.IN(
						__memo_property.q.memoID,
						sqlobject.sqlbuilder.Select( \
							__memo_property.q.memoID, \
							where = __condition \
						) \
					) \
				)

		__memos = list()
		__result = __memo_property.select( \
			sqlobject.AND(
				__condition, \
				self.q.id == __memo_property.q.memoID, \
			), \
			orderBy=self.q.timeAdded, \
			reversed=True \
		)
		for i in list(__result) :
			i.memo = self.get(i.memoID)
			__memos.append(i)

		return __memos

	def getMemosWithHistory (self, **properties) :
		if not properties :
			return None

		__id_user = self.getUserID()
		__memo_property = get_model_memo_property_user(__id_user)
		__memo_history = get_model_memo_history_user(__id_user)

		__condition = None
		for __k, __v in properties.items() :
			__c = sqlobject.AND( \
				__memo_property.q.type == __k,
				__memo_property.q.value == __v,
			)

			if not __condition :
				__condition = __c
				continue
			else :
				__condition = sqlobject.AND( \
					__memo_property.q.type == __k,
					__memo_property.q.value == __v,
					sqlobject.sqlbuilder.IN(
						__memo_property.q.memoID,
						sqlobject.sqlbuilder.Select( \
							__memo_property.q.memoID, \
							where = __condition \
						) \
					) \
				)

		__memos = list()
		__result = __memo_property.select( \
			sqlobject.AND(
				__condition, \
				self.q.id == __memo_property.q.memoID, \
			), \
			orderBy=self.q.timeAdded, \
			reversed=True \
		)
		for i in list(__result) :
			i.memo = self.get(i.memoID)
			i.history = list( \
				__memo_history.select( \
					__memo_history.q.memoID == i.memoID, \
					orderBy=__memo_history.q.timeChanged, \
					reversed=True \
				) \
			)
			__memos.append(i)

		return __memos

	insert					= classmethod(insert)
	getUserID 				= classmethod(getUserID)
	getList 				= classmethod(getList)
	getMemos 				= classmethod(getMemos)
	getMemosWithHistory 	= classmethod(getMemosWithHistory)
	hasMemo 					= classmethod(hasMemo)
	removeMemo 				= classmethod(removeMemo)

def get_model (id_user) :
	__model = database.get_model( \
		database.get_table("memo_user", "MemoUser"), \
		SKEL_TABLENAME % id_user \
	)

	__conn = database.get_connection(URI)
	__model.createTable( \
		ifNotExists=True, \
		createIndexes=True, \
		connection= __conn \
	)

	return __model

"""
Description
-----------


ChangeLog
---------


Usage
-----


"""

__author__ =  "Spike^ekipS <spikeekips@gmail.com>"
__version__=  "0.1"
__nonsense__ = ""
