# -*- coding: utf-8 -*-
#	Copyright (C) 2005 Spike^ekipS <spikeekips@gmail.com>
#
#	http://vee.kldp.net
#
#	   This program is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; either version 2 of the License, or
#	(at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from optparse import OptionParser
from twisted.conch.client import options

import i18n
_ = i18n.gettext()

class SSHClient (options.ConchOptions) :

	synopsis = \
"""Usage:   sshclient.py [options] host [command]
"""

	optParameters = [ \
		['escape', 'e', '~'], \
		['localforward', 'L', None, 'listen-port:host:port   Forward local port to remote address'], \
		['remoteforward', 'R', None, 'listen-port:host:port   Forward remote port to local address'], \
		['uid', 'U', None, 'uid for VEE'], \
	]

	optFlags = [ \
		['null', 'n', 'Redirect input from /dev/null.'], \
		['fork', 'f', 'Fork to background after authentication.'], \
		['tty', 't', 'Tty; allocate a tty even if command is given.'], \
		['notty', 'T', 'Do not allocate a tty.'], \
		['noshell', 'N', 'Do not execute a shell or command.'], \
		['subsystem', 's', 'Invoke command (mandatory) as SSH2 subsystem.'], \
	]

	#zsh_altArgDescr = {"foo":"use this description for foo instead"}
	#zsh_multiUse = ["foo", "bar"]
	#zsh_mutuallyExclusive = [("foo", "bar"), ("bar", "baz")]
	#zsh_actions = {"foo":'_files -g "*.foo"', "bar":"(one two three)"}
	zsh_actionDescr = { \
		"localforward":"listen-port:host:port", \
		"remoteforward":"listen-port:host:port" \
	}
	zsh_extras = ["*:command: "]

	localForwards = []
	remoteForwards = []

	def opt_escape(self, esc):
		"Set escape character; ``none'' = disable"
		if esc == 'none':
			self['escape'] = None
		elif esc[0] == '^' and len(esc) == 2:
			self['escape'] = chr(ord(esc[1])-64)
		elif len(esc) == 1:
			self['escape'] = esc
		else:
			sys.exit("Bad escape character '%s'." % esc)

	def opt_localforward(self, f):
		"Forward local port to remote address (lport:host:port)"
		localPort, remoteHost, remotePort = f.split(':') # doesn't do v6 yet
		localPort = int(localPort)
		remotePort = int(remotePort)
		self.localForwards.append((localPort, (remoteHost, remotePort)))

	def opt_remoteforward(self, f):
		"""Forward remote port to local address (rport:host:port)"""
		remotePort, connHost, connPort = f.split(':') # doesn't do v6 yet
		remotePort = int(remotePort)
		connPort = int(connPort)
		self.remoteForwards.append((remotePort, (connHost, connPort)))

	def parseArgs(self, host="", *command):
		if host.find("@") == -1 :
			self['host'] = host
		else :
			try :
				(__user, __host, ) = host.split("@")
			except ValueError :
				__s = host.split("@")
				(__user, __host, ) = (__s[:1], __s[1:],)

			self['host'] = __host
			self['user'] = __user

		self['command'] = ' '.join(command)

	def opt_user (self, user="") :
		if not user.strip() :
			user = os.getenv("USER")

		self["user"] = user.strip()

def VEE () :

	parser = OptionParser(version="%prog 1.0")

	parser.usage = "%prog [options] filename [filename ...]"

	parser.add_option( \
		"-V", "--start-with-vim", \
		action="store_true", \
		dest="ARG_START_WITH_VIM", default=False, \
		help=_("Start with vim."), \
	)

	parser.add_option( \
		"-C", "--start-with-terminal", \
		action="store_true", \
		dest="ARG_START_WITH_TERMINAL", default=False, \
		help=_("Start with terminal."), \
	)

	parser.add_option( \
		"-M", "--show-menubar", \
		action="store_const", \
		dest="ARG_SHOW_MENUBAR", const=True, \
		default=True, \
		help=_("Show menubar."), \
	)

	parser.add_option( \
		"-m", "--hide-menubar", \
		action="store_const", \
		dest="ARG_SHOW_MENUBAR", const=False, \
		help=_("Hide menubar."), \
	)

	parser.add_option( \
		"-U", "--show-statusbar", \
		action="store_const", \
		dest="ARG_SHOW_STATUSBAR", const=True, default=True, \
		help=_("Show statusbar."), \
	)

	parser.add_option( \
		"-u", "--hide-statusbar", \
		action="store_const", \
		dest="ARG_SHOW_STATUSBAR", const=False, \
		help=_("Hide statusbar."), \
	)

	parser.add_option( \
		"-T", "--show-toolbar", \
		action="store_const", \
		dest="ARG_SHOW_TOOLBAR", const=True, default=True, \
		help=_("Show toolbar."), \
	)

	parser.add_option( \
		"-t", "--hide-toolbar", \
		action="store_const", \
		dest="ARG_SHOW_TOOLBAR", const=False, \
		help=_("Hide toolbar."), \
	)

	parser.add_option( \
		"-B", "--show-bookmark", \
		action="store_const", \
		dest="ARG_SHOW_BOOKMARK", const=True, \
		default=False, \
		help=_("Show bookmark."), \
	)

	parser.add_option( \
		"-b", "--hide-bookmark", \
		action="store_const", \
		dest="ARG_SHOW_BOOKMARK", const=False, \
		help=_("Hide bookmark."), \
	)

	parser.add_option( \
		"-s", "--install-schema", \
		action="store_const", \
		dest="ARG_INSTALL_SCHEMA", const=True, default=False, \
		help=_("Install GConf schema."), \
	)

	parser.add_option( \
		"-F", "--fullscreen", \
		action="store_const", \
		dest="ARG_FULLSCREEN_WINDOW", const=True, default=False, \
		help=_("Start the window to be fullscreened."), \
	)

	parser.add_option( \
		"-X", "--maximaze", \
		action="store_const", \
		dest="ARG_MAXIMIZE_WINDOW", const=True, default=False, \
		help=_("Start the default window to be maximized."), \
	)

	parser.add_option( \
		"--geometry", \
		dest="ARG_GEOMETRY", default="", \
		help=_("Set the initial geometry. ex) 500x400+10+10"), \
	)

	return parser

def VEET () :

	parser = OptionParser(version="%prog 1.0")

	parser.usage = _("%prog [options] filename [filename ...]")

	parser.add_option( \
		"-v", "--as-vim", \
		action="store_true", \
		dest="ARG_AS_VIM", default=True, \
		help=_("Start with vim. (Default)"), \
	)

	parser.add_option( \
		"-g", "--as-gvim", \
		action="store_true", \
		dest="ARG_AS_GVIM", default=False, \
		help=_("Start with gvim."), \
	)

	return parser

"""
Description
-----------


ChangeLog
---------


Usage
-----


"""

__author__ =  "Spike^ekipS <spikeekips@gmail.com>"
__version__=  "0.1"
__nonsense__ = ""

__file__ = "Options.py"


