# -*- coding: utf-8 -*-
#    Copyright (C) 2005 Spike^ekipS <spikeekips@gmail.com>
#
#    http://vee.kldp.net
#
#       This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import os, gtk, time, UID, gobject
import Conf__, Conf, Menu, i18n, func, Window, Handler

_ = i18n.gettext()

class __TREEVIEW__ :

	"""
	The basic treeview class.

	Properties
	===========
		enable-search
		expander-column
		fixed-height-mode
		hadjustment
		headers-clickable
		headers-visible
		hover-expand
		hover-selection
		model
		reorderable
		rules-hint
		search-column
		vadjustment

	PROPERTIES = { \
		"enable-search" : True, \
		"expander-column" : , \
		"fixed-height-mode" : True, \
		"hadjustment" : True, \
		"headers-clickable" : True, \
		"headers-visible" : True, \
		"hover-expand" : True, \
		"hover-selection" : True, \
		"model" : True, \
		"reorderable" : False, \
		"rules-hint" : True, \
		"search-column" : -1, \
		"vadjustment" : , \
	}

	"""

	XML = None
	WIDGET_TREEVIEW = None
	WIDGET_TREESTORE = None
	MODEL = None

	"""
	GTK selection constants are,
		gtk.SELECTION_NONE	No selection allowed.
		gtk.SELECTION_SINGLE	A single selection allowed by clicking.
		gtk.SELECTION_BROWSE	A single selection allowed by browsing with the pointer.
		gtk.SELECTION_MULTIPLE	Multiple items can be selected at once.
		gtk.SELECTION_EXTENDED	Deprecated.
	"""
	SELECTION_MODE = gtk.SELECTION_SINGLE
	SELECTION_CURRENT = None

	PATH_CURRENT = (0,)

	def __init__ (self, xml, treeview_name, properties=dict()) :

		self.XML = xml
		self.WIDGET_TREEVIEW = self.XML.get_widget(treeview_name)

		# set property
		if len(properties) > 0 :
			for i, j in properties.items() :
				self.WIDGET_TREEVIEW.set_property(i, j)

		self.create_frame()
		self.insert_row()

		__selection = self.WIDGET_TREEVIEW.get_selection()
		__selection.set_mode(self.SELECTION_MODE)
		__selection.connect("changed", self.on_change_selection)

		# connect handler
		self.WIDGET_TREEVIEW.connect("row-activated", self.on_row_activated)
		self.WIDGET_TREEVIEW.connect("row-expanded", self.on_row_expanded)
		self.WIDGET_TREEVIEW.connect("row-collapsed", self.on_row_collapsed)

	def create_frame (self) : pass
	def insert_row (self) : pass
	def add_row (self, *args) : pass

	###############################################################
	# misc methods
	def append (self, rows, parent=None) :
		return self.WIDGET_TREESTORE.append(parent, rows)

	def prepend (self, rows, parent=None) :
		self.WIDGET_TREESTORE.prepend(parent, rows)

	def set_cursor (self, path) :
		self.WIDGET_TREEVIEW.set_cursor(path)

	def move (self, from_path, to_path) :
		if from_path == to_path : return

		__iter_from = self.WIDGET_TREESTORE.get_iter(from_path)
		if not __iter_from : return
		__iter_to = self.WIDGET_TREESTORE.get_iter(to_path)
		if not __iter_to : return

		self.WIDGET_TREESTORE.move_after(__iter_from, __iter_to)

	def remove (self, **kwargs) :
		"""
		One of path, iter must be given.

		@return : bool()
		"""

		treeiter = None
		if kwargs.has_key("treeiter") : treeiter = kwargs["treeiter"]
		if kwargs.has_key("path") :
			treeiter = self.WIDGET_TREESTORE.get_iter(kwargs["path"])

		if not treeiter : return False

		self.WIDGET_TREESTORE.remove(treeiter)
		self.WIDGET_TREEVIEW.set_cursor((1,))

	def get_path (self, iter) :
		return self.WIDGET_TREESTORE.get_path(iter)

	def get_iter (self, path) :
		__treeiter = self.WIDGET_TREESTORE.get_iter(path)

		return __treeiter

	def get_value (self, path, column_number) :
		__treeiter = self.WIDGET_TREESTORE.get_iter(path)
		__value = self.WIDGET_TREESTORE.get_value(__treeiter, column_number)

		return __value

	def set_value (self, path, column_number, value) :
		__iter = self.WIDGET_TREESTORE.get_iter(path)

		if not __iter : return False

		self.WIDGET_TREESTORE.set_value( \
			__iter, \
			column_number, \
			value \
		)

	def get_selected (self) :
		(__treestore, __treeiter) = \
			self.WIDGET_TREEVIEW.get_selection().get_selected()

		return (__treestore, __treeiter, )

	def get_value_selected (self, column_number) :
		(__treestore, __treeiter, ) = self.get_selected()
		if not __treeiter : return

		return self.WIDGET_TREESTORE.get_value(__treeiter, column_number)

	def get_current_path (self, path) :
		return self.PATH_CURRENT

	def set_current_path (self, path) :
		self.PATH_CURRENT = path

	################################################################
	# HANDLER
	def on_change_selection (self, selection) :
		self.SELECTION_CURRENT = selection

	def on_row_expanded \
			(self, treeview, path, view_column, user_param1=None) :
		pass

	def on_row_collapsed \
			(self, treeview, path, view_column, user_param1=None) :
		pass

	def on_row_activated \
			(self, treeview, path, view_column, user_param1=None) :
		pass

class ConnectionListSSH (__TREEVIEW__) :

	CONNECTION_LIST = None

	def __init__ (self, xml, **kwargs) :

		PROPERTIES = { \
			"enable-search" : True, \
			"fixed-height-mode" : False, \
			"headers-clickable" : True, \
			"headers-visible" : True, \
			"hover-expand" : False, \
			"hover-selection" : False, \
			"reorderable" : True, \
			"rules-hint" : True, \
		}

		if not kwargs.has_key("connection_list") :
			raise RuntimeError, "connection_list must be given."
		else :
			self.CONNECTION_LIST = kwargs["connection_list"]

		__TREEVIEW__.__init__( \
			self, xml, "treeview_connection_ssh", PROPERTIES)

		#self.set_cursor((0,))

	def create_frame (self) :

		# create a CellRenderers to render the data
		__cell0 = gtk.CellRendererText()
		__cell0.set_property("xalign", 0.0)

		__cell1 = gtk.CellRendererText()
		__cell1.set_property("width", 100)
		__cell2 = gtk.CellRendererText()

		# set property.
		self.tvcolumn0 = gtk.TreeViewColumn(_("Name"))
		self.tvcolumn1 = gtk.TreeViewColumn(_("Server Name or IP"))
		self.tvcolumn2 = gtk.TreeViewColumn(_("UID"))

		self.WIDGET_TREESTORE = gtk.TreeStore(str, str, str)
		self.WIDGET_TREEVIEW.set_model(self.WIDGET_TREESTORE)

		# add columns to treeview
		self.WIDGET_TREEVIEW.append_column(self.tvcolumn0)
		self.WIDGET_TREEVIEW.append_column(self.tvcolumn1)

		# add the cells to the columns - 2 in the first
		self.tvcolumn0.pack_start(__cell0, False)
		self.tvcolumn1.pack_start(__cell1, False)
		self.tvcolumn2.pack_start(__cell2, False)

		self.tvcolumn0.set_attributes( \
			__cell0, markup=Conf.TREEVIEW_CONNECTION_SSH_PANEL_COLUMN_NAME)
		self.tvcolumn1.set_attributes( \
			__cell1, text=Conf.TREEVIEW_CONNECTION_SSH_PANEL_COLUMN_IP)
		self.tvcolumn2.set_attributes( \
			__cell2, text=Conf.TREEVIEW_CONNECTION_SSH_PANEL_COLUMN_UID)

		self.tvcolumn0.set_resizable(True)
		self.tvcolumn1.set_resizable(True)
		self.tvcolumn0.set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)
		self.tvcolumn1.set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)

		self.tvcolumn0.set_reorderable(True)
		self.tvcolumn1.set_reorderable(True)

	def insert_row (self) :
		self.WIDGET_TREESTORE.clear()

		__data = self.CONNECTION_LIST.get_list()

		for __name, __value in __data.items() :
			self.append( \
				[ \
					"<span weight=\"bold\" size=\"large\">%s</span>" % __name, \
					__value["host"], \
					__value["__uid__"], \
				] \
			)

	def update_row (self, path, uid) :
	
		__name = self.CONNECTION_LIST.get_value_by_key(uid, "name")
		__ip = self.CONNECTION_LIST.get_value_by_key(uid, "host")

		self.set_value( \
			path, 0, \
			"<span weight=\"bold\" size=\"large\">%s</span>" % __name \
		)
		self.set_value(path, 1, __ip)

class Bookmarks (__TREEVIEW__) :

	CALLBACK_FUNCTIONS = dict()

	ICON_CATEGORY = None
	ICON_TERMINAL = None
	ICON_VIM = None
	ICON_GVIM = None

	def __init__ (self, xml, **kwargs) :

		if not kwargs.has_key("state") :
			raise RuntimeError, "state must be give."
		else :
			self.STATE = kwargs["state"]
			self.BOOKMARKS = self.STATE.get_value("bookmark")

		if not kwargs.has_key("callback_functions") :
			raise RuntimeError, "callback_functions argument must be give."
		else :
			self.CALLBACK_FUNCTIONS = kwargs["callback_functions"]

		PROPERTIES = { \
			"enable-search" : False, \
			"fixed-height-mode" : False, \
			"headers-clickable" : False, \
			"headers-visible" : False, \
			"hover-expand" : False, \
			"hover-selection" : False, \
			"reorderable" : False, \
			"rules-hint" : False, \
		}

		self.__create_icons()

		__TREEVIEW__.__init__( \
			self, xml, "treeview_bookmark", PROPERTIES)

		# activate popup menu
		self.WIDGET_TREEVIEW.connect( \
			"button-release-event", \
			Menu.on_show_popup_bookmarks, \
			self.STATE, \
			self.CALLBACK_FUNCTIONS \
		)

		###############################################################
		# ACTIVATE dnd.
		self.WIDGET_TREEVIEW.enable_model_drag_dest(Conf.DND_TARGET_TABLE, gtk.gdk.ACTION_DEFAULT)

		self.WIDGET_TREEVIEW.connect( \
			"drag_data_received", self.on_drag_received_add_item)

	def create_frame (self) :

		self.cell0 = gtk.CellRendererPixbuf()
		self.cell1 = gtk.CellRendererText()
		self.cell2 = gtk.CellRendererText()

		self.cell1.connect("edited", self.on_rename_edited)

		self.tvcolumn0 = gtk.TreeViewColumn()
		self.tvcolumn1 = gtk.TreeViewColumn()
		self.tvcolumn0.set_expand(True)

		self.tvcolumn1.set_sort_column_id(Conf.TREEVIEW_BOOKMARK_PANEL_COLUMN_LABEL)

		self.WIDGET_TREESTORE = gtk.TreeStore(gtk.gdk.Pixbuf, str, str)
		self.WIDGET_TREEVIEW.set_model(self.WIDGET_TREESTORE)

		self.WIDGET_TREEVIEW.append_column(self.tvcolumn0)

		self.tvcolumn0.pack_start(self.cell0, False)
		self.tvcolumn0.pack_start(self.cell1, True)
		self.tvcolumn1.pack_start(self.cell2, True)

		self.tvcolumn0.set_attributes(self.cell0, pixbuf=0)
		self.tvcolumn0.set_attributes(self.cell1, markup=1)
		self.tvcolumn1.set_attributes(self.cell2, text=2)

		# make treeview searchable
		self.WIDGET_TREEVIEW.set_search_column(Conf.TREEVIEW_BOOKMARK_PANEL_COLUMN_LABEL)

	def insert_row (self) :
		self.WIDGET_TREESTORE.clear()

		__root = self.BOOKMARKS.get_root()
		self.__insert_item(None, __root)

		self.WIDGET_TREESTORE[(0,)][Conf.TREEVIEW_BOOKMARK_PANEL_COLUMN_LABEL] = \
			_("My Bookmark")

		self.WIDGET_TREEVIEW.expand_row((0,), False)

	def __insert_item (self, parent, uid_list) :
		for i in uid_list :
			__data = self.BOOKMARKS.get_value(i, "asDict")

			if not __data["type"] : continuewidget_TREESTORE
			if not __data["label"] : __label = __data["value"]

			__iter_parent = \
				self.add_row(parent, i, __data["type"], __data["label"])

			__child_list = self.BOOKMARKS.get_child(i)
			if len(__child_list) > 0 :
				self.__insert_item(__iter_parent, __child_list)

	def add_row (self, parent, uid, type, label) :

		if type == "category" :
			icon = self.ICON_CATEGORY
		elif type == "gvim" :
			icon = self.ICON_GVIM
		elif type == "vim" :
			icon = self.ICON_VIM
		elif type == "terminal" :
			icon = self.ICON_TERMINAL
		elif type == "external_command" :
			icon = self.ICON_EXTERNAL_COMMAND
		else :
			icon = None

		child = self.WIDGET_TREESTORE.append(parent, [icon, label, uid])

		return child

	def on_drag_received_add_item \
			(self, widget, context, x, y, selection, targetType, time) :

		__file_path = func.get_filename_from_uri(selection.data)
		__path = self.WIDGET.get_dest_row_at_pos(x, y)

		Menu.BookmarkItemType(self.BOOKMARKS, self.WIDGET, __path[0], __file_path)

	def on_row_expanded \
			(self, treeview, iter, path, user_param1=None) :
		return

	def on_row_collapsed \
			(self, treeview, iter, path, user_param1=None) :
		return

	def on_row_activated \
			(self, treeview, path, view_column, user_param1=None) :

		uid = self.get_value(path, Conf.TREEVIEW_BOOKMARK_PANEL_COLUMN_UID)
		__data = self.BOOKMARKS.get_value(uid, "asDict")

		if __data["type"] == "category" :
			if self.WIDGET_TREEVIEW.row_expanded(path) :
				self.WIDGET_TREEVIEW.collapse_row(path)
			else :
				self.WIDGET_TREEVIEW.expand_row(path, False)

			return

		try :
			__setting = self.BOOKMARKS.get_setting(uid)
		except IndexError :
			__setting = None

		# run command.
		__func = self.CALLBACK_FUNCTIONS["addPage"]

		if __data["type"] == "terminal" :
			__func(__data["type"], directory=__data["value"], setting=__setting)
		elif __data["type"] == "external_command" :
			__func(__data["type"], labelname=__data["label"], \
				external_command=__data["value"], setting=__setting)
		elif __data["type"] in ("vim", "gvim") :
			__func(__data["type"], filename=__data["value"], setting=__setting)

		self.STATE.get_value("panel").hide()

		return

	def on_rename_edited (self, cell, path, new_label) :
		__path = tuple([ int(i) for i in path.split(":")])

		cell.set_property("editable", False)
		self.rename_by_path(__path, new_label)

		# refresh menu items in menubar.
		self.CALLBACK_FUNCTIONS["insert_bookmark"]()

		return

	def rename_by_path (self, path, s) :
		__uid = self.get_value(path, Conf.TREEVIEW_BOOKMARK_PANEL_COLUMN_UID)
		self.BOOKMARKS.set_value(__uid, "label", s)

		self.WIDGET_TREESTORE[path][Conf.TREEVIEW_BOOKMARK_PANEL_COLUMN_LABEL] = s

		return gtk.TRUE

	####################################################################
	# misc
	def __create_icons (self) :

		self.ICON_CATEGORY = func.create_icon(os.path.join(Conf__.DIR_PIXMAPS, "category.png"))
		self.ICON_EXTERNAL_COMMAND = func.create_icon(os.path.join(Conf__.DIR_PIXMAPS, "external-command.png"))
		self.ICON_TERMINAL = func.create_icon(os.path.join(Conf__.DIR_PIXMAPS, "terminal.png"))
		self.ICON_VIM = func.create_icon(os.path.join(Conf__.DIR_PIXMAPS, "vim.png"))
		self.ICON_GVIM = func.create_icon(os.path.join(Conf__.DIR_PIXMAPS, "gvim.png"))

"""
Description
-----------


ChangeLog
---------


Usage
-----


"""

__author__ =  "Spike^ekipS <spikeekips@gmail.com>"
__version__=  "0.1"
__nonsense__ = ""

__file__ = "Panel.py"


