# -*- coding: utf-8 -*-
#	Copyright 2005 Spike^ekipS <spikeekips@gmail.com>
#
#	   This program is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; either version 2 of the License, or
#	(at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

# Copyright (c) 2001-2004 Twisted Matrix Laboratories.
# See LICENSE for details.

from twisted.conch.client import agent, connect, default, options
from twisted.conch.error import ConchError
from twisted.conch.ssh import connection, common, transport
from twisted.conch.ssh.common import NS, getNS, MP
from twisted.conch.ssh import session, forwarding, channel
from twisted.internet import reactor, stdio, defer, task, abstract, fdesc
from twisted.python import log

import Conf, GConf, ItemConnection

import os, sys, getpass, struct, tty, fcntl, signal, errno

def _stopReactor() :
	try :
		reactor.stop()
	except : pass

class SSHUserAuthClient (default.SSHUserAuthClient) :

	OPTIONS = dict()

	def __init__(self, user, options, *args):
		self.OPTIONS = options

		default.SSHUserAuthClient.__init__(self, user, options, *args)

	def getPassword (self, prompt=None) :

		if self.__autoLogin() :
			return defer.succeed(self.OPTIONS["password"])
		else :
			if not prompt:
				prompt = "%s@%s's password: " % (self.user, self.transport.transport.getPeer().host)
			try :
				p = self._getPassword(prompt)
				return defer.succeed(p)
			except ConchError:
				return defer.fail()

	def getGenericAnswers(self, name, instruction, prompts):
		responses = []

		if self.__autoLogin() :
			return defer.succeed([self.OPTIONS["password"], ])
		else :
			return default.SSHUserAuthClient.getGenericAnswers( \
						self, name, instruction, prompts)

	def ssh_USERAUTH_FAILURE(self, packet):

		if self.__autoLogin() :
			self.transport.sendDisconnect( \
				transport.DISCONNECT_NO_MORE_AUTH_METHODS_AVAILABLE, \
				'no more authentication methods available' \
			)

		return default.SSHUserAuthClient.ssh_USERAUTH_FAILURE(self, packet)

	def __autoLogin (self) :
		if self.OPTIONS.has_key("password") and self.OPTIONS["password"].strip() :
			return True
		else :
			return False

class StandardIOWriterVTE (stdio.StandardIOWriter) :

	def __init__ (self, **objects) :
		abstract.FileDescriptor.__init__(self)

		if objects and objects["terminal"] and hasattr(objects["terminal"], "write") :
			self.fileno = objects["terminal"].fileno
		else :
			self.fileno = sys.__stdout__.fileno

		fdesc.setNonBlocking(self.fileno())

class StandardIOVTE (stdio.StandardIO) :

	def __init__ (self, protocol, **objects) :
		stdio.StandardIO.__init__(self, protocol)
		self.writer = StandardIOWriterVTE(**objects)

class ClientOptions(options.ConchOptions) :

	__SKEL__ = { \
			"host" : str(),					   "agent" : 0, \
			"user" : str(),						"known-hosts" : None, \
			"port" : None,							"notty" : 0, \
			"command" : str(),					"reconnect" : 0, \
			"log" : 0,								"escape" : "~", \
			"fork" : 0,								"remoteforward" : None, \
			"subsystem" : 0,						"null" : 0, \
			"tty" : 0,								"identity" : None, \
			"noagent" : 0,							"nocache" : 0, \
			"macs" : None,							"nox11" : 0, \
			"option" : None,							"version" : 0, \
			"host-key-algorithms" : None,   		"noshell" : 0, \
			"user-authentications" : None,  		"connection-usage" : None, \
			"ciphers" : None,			   			"localforward" : None, \
			"compress" : 0, \
		}

	#zsh_altArgDescr = {"foo":"use this description for foo instead"}
	#zsh_multiUse = ["foo", "bar"]
	#zsh_mutuallyExclusive = [("foo", "bar"), ("bar", "baz")]
	#zsh_actions = {"foo":'_files -g "*.foo"', "bar":"(one two three)"}
	zsh_actionDescr = { \
		"localforward":"listen-port:host:port", \
	   "remoteforward":"listen-port:host:port" \
	}
	zsh_extras = ["*:command: ", ]

	localForwards = []
	remoteForwards = []

	def __init__ (self) :
		options.ConchOptions.__init__(self)
		self.update(self.__SKEL__)

	def opt_escape(self, esc) :
		"Set escape character; ``none'' = disable"
		if esc == 'none' :
			self['escape'] = None
		elif esc[0] == '^' and len(esc) == 2 :
			self['escape'] = chr(ord(esc[1])-64)
		elif len(esc) == 1 :
			self['escape'] = esc
		else :
			sys.exit("Bad escape character '%s'." % esc)

	def opt_localforward(self, f) :
		"Forward local port to remote address (lport:host:port)"
		localPort, remoteHost, remotePort = f.split(':') # doesn't do v6 yet
		localPort = int(localPort)
		remotePort = int(remotePort)
		self.localForwards.append((localPort, (remoteHost, remotePort)))

	def opt_remoteforward(self, f) :
		"""Forward remote port to local address (rport:host:port)"""
		remotePort, connHost, connPort = f.split(':') # doesn't do v6 yet
		remotePort = int(remotePort)
		connPort = int(connPort)
		self.remoteForwards.append((remotePort, (connHost, connPort)))

	def parseArgs(self, host, *command) :
		self['host'] = host
		self['command'] = ' '.join(command)

class _KeepAlive :

	def __init__(self, conn) :
		self.conn = conn
		self.globalTimeout = None
		self.lc = task.LoopingCall(self.sendGlobal)
		self.lc.start(300)

	def sendGlobal(self) :
		d = self.conn.sendGlobalRequest("conch-keep-alive@twistedmatrix.com",
				"", wantReply = 1)
		d.addBoth(self._cbGlobal)
		self.globalTimeout = reactor.callLater(30, self._ebGlobal)

	def _cbGlobal(self, res) :
		if self.globalTimeout :
			self.globalTimeout.cancel()
			self.globalTimeout = None

	def _ebGlobal(self) :
		if self.globalTimeout :
			self.globalTimeout = None
			self.conn.transport.loseConnection()

class SSHConnection(connection.SSHConnection) :
	OPTIONS = dict()
	OBJECTS = None

	def set_parameters (self, options, objects=None) :
		self.OPTIONS = options
		if objects : self.OBJECTS = objects

	def beforeShutdown(self) :
		remoteForwards = self.OPTIONS.remoteForwards
		for remotePort, hostport in remoteForwards :
			log.msg('cancelling %s:%s' % (remotePort, hostport))
			conn.cancelRemoteForwarding(remotePort)

	def onConnect(self) :
	#	if keyAgent and self.OPTIONS['agent'] :
	#		cc = protocol.ClientCreator(reactor, SSHAgentForwardingLocal, conn)
	#		cc.connectUNIX(os.environ['SSH_AUTH_SOCK'])

		if hasattr(conn.transport, 'sendIgnore') :
			_KeepAlive(conn)

		if self.OPTIONS.localForwards :
			for localPort, hostport in self.OPTIONS.localForwards :
				reactor.listenTCP(localPort,
							forwarding.SSHListenForwardingFactory(conn,
								hostport,
								SSHListenClientForwardingChannel))

		if self.OPTIONS.remoteForwards :
			for remotePort, hostport in self.OPTIONS.remoteForwards :
				log.msg('asking for remote forwarding for %s:%s' %
						(remotePort, hostport))
				conn.requestRemoteForwarding(remotePort, hostport)
			reactor.addSystemEventTrigger('before', 'shutdown', self.beforeShutdown)

		if not self.OPTIONS['noshell'] or self.OPTIONS['agent'] :
			SS = SSHSession()
			SS.set_parameters(self.OPTIONS, self.OBJECTS)
			conn.openChannel(SS)

		if self.OPTIONS['fork'] :
			if os.fork() :
				os._exit(0)
			os.setsid()
			for i in range(3) :
				try :
					os.close(i)
				except OSError, e :
					if e.errno != errno.EBADF :
						raise

	def serviceStarted(self) :
		global conn
		conn = self
		self.remoteForwards = {}
		if not isinstance(self, connection.SSHConnection) :
			# make these fall through
			del self.__class__.requestRemoteForwarding
			del self.__class__.cancelRemoteForwarding

		self.onConnect()

	def stopConnection(self) :
		if not self.OPTIONS["reconnect"] :
			reactor.callLater(0.1, _stopReactor)

	def serviceStopped(self) :
		self.stopConnection()

	def requestRemoteForwarding(self, remotePort, hostport) :
		data = forwarding.packGlobal_tcpip_forward(('0.0.0.0', remotePort))
		d = self.sendGlobalRequest('tcpip-forward', data,
								   wantReply=1)
		log.msg('requesting remote forwarding %s:%s' %(remotePort, hostport))
		d.addCallback(self._cbRemoteForwarding, remotePort, hostport)
		d.addErrback(self._ebRemoteForwarding, remotePort, hostport)

	def _cbRemoteForwarding(self, result, remotePort, hostport) :
		log.msg('accepted remote forwarding %s:%s' % (remotePort, hostport))
		self.remoteForwards[remotePort] = hostport
		log.msg(repr(self.remoteForwards))

	def _ebRemoteForwarding(self, f, remotePort, hostport) :
		log.msg('remote forwarding %s:%s failed' % (remotePort, hostport))
		log.msg(f)

	def cancelRemoteForwarding(self, remotePort) :
		data = forwarding.packGlobal_tcpip_forward(('0.0.0.0', remotePort))
		self.sendGlobalRequest('cancel-tcpip-forward', data)
		log.msg('cancelling remote forwarding %s' % remotePort)
		try :
			del self.remoteForwards[remotePort]
		except :
			pass
		log.msg(repr(self.remoteForwards))

	def channel_forwarded_tcpip(self, windowSize, maxPacket, data) :
		log.msg('%s %s' % ('FTCP', repr(data)))
		remoteHP, origHP = forwarding.unpackOpen_forwarded_tcpip(data)
		log.msg(self.remoteForwards)
		log.msg(remoteHP)
		if self.remoteForwards.has_key(remoteHP[1]) :
			connectHP = self.remoteForwards[remoteHP[1]]
			log.msg('connect forwarding %s' % (connectHP,))
			return SSHConnectForwardingChannel(connectHP,
											remoteWindow = windowSize,
											remoteMaxPacket = maxPacket,
											conn = self)
		else :
			raise ConchError(connection.OPEN_CONNECT_FAILED, "don't know about that port")

#	def channel_auth_agent_openssh_com(self, windowSize, maxPacket, data) :
#		if self.OPTIONS['agent'] and keyAgent :
#			return agent.SSHAgentForwardingChannel(remoteWindow = windowSize,
#											 remoteMaxPacket = maxPacket,
#											 conn = self)
#		else :
#			return connection.OPEN_CONNECT_FAILED, "don't have an agent"

	def channelClosed(self, channel) :
		log.msg('connection closing %s' % channel)
		log.msg(self.channels)
		if len(self.channels) == 1 and not (self.OPTIONS['noshell'] and not self.OPTIONS['nocache']) : # just us left
			log.msg('stopping connection')
			self.stopConnection()
		else :
			# because of the unix thing
			self.__class__.__bases__[0].channelClosed(self, channel)

class SSHSession(channel.SSHChannel) :

	name = 'session'
	OPTIONS = dict()
	OBJECTS = None

	_inRawMode = False
	_savedRawMode = None

	def set_parameters (self, kwargs, objects=None) :
		self.OPTIONS = kwargs
		if objects : self.OBJECTS = objects

	def channelOpen(self, foo) :
		log.msg('session %s open' % self.id)

		if self.OPTIONS['agent'] :
			d = self.conn.sendRequest(self, 'auth-agent-req@openssh.com', '', wantReply=1)
			d.addBoth(lambda x:log.msg(x))
		if self.OPTIONS['noshell']: return
		if (self.OPTIONS['command'] and self.OPTIONS['tty']) or not self.OPTIONS['notty'] :
			self._enterRawMode()

		c = session.SSHSessionClient()
		if self.OPTIONS['escape'] and not self.OPTIONS['notty'] :
			self.escapeMode = 1
			c.dataReceived = self.handleInput
		else:
			c.dataReceived = self.write

		c.connectionLost = lambda x=None,s=self:s.sendEOF()

		#self.stdio = stdio.StandardIO(c)
		if self.OBJECTS :
			self.stdio = StandardIOVTE(c, **self.OBJECTS)
		else :
			self.stdio = StandardIOVTE(c)

		fd = 0
		if self.OPTIONS['subsystem'] :
			self.conn.sendRequest(self, 'subsystem', \
				common.NS(self.OPTIONS['command']))
		elif self.OPTIONS['command'] :
			if self.OPTIONS['tty'] :
				term = os.environ['TERM']
				winsz = fcntl.ioctl(fd, tty.TIOCGWINSZ, '12345678')
				winSize = struct.unpack('4H', winsz)
				ptyReqData = session.packRequest_pty_req(term, winSize, '')
				self.conn.sendRequest(self, 'pty-req', ptyReqData)
				signal.signal(signal.SIGWINCH, self._windowResized)

			self.conn.sendRequest(self, 'exec', common.NS(self.OPTIONS['command']))
		else :
			if not self.OPTIONS['notty'] :
				term = os.environ['TERM']
				winsz = fcntl.ioctl(fd, tty.TIOCGWINSZ, '12345678')
				winSize = struct.unpack('4H', winsz)
				ptyReqData = session.packRequest_pty_req(term, winSize, '')
				self.conn.sendRequest(self, 'pty-req', ptyReqData)
				signal.signal(signal.SIGWINCH, self._windowResized)

			self.conn.sendRequest(self, 'shell', '')

			#if hasattr(conn.transport, 'transport') :
			#	conn.transport.transport.setTcpNoDelay(1)

	def stopConnection() :
		if not self.OPTIONS["reconnect"] :
			reactor.callLater(0.1, _stopReactor)

	def _enterRawMode (self) :
		if self._inRawMode : return

		fd = sys.stdin.fileno()
		try :
			old = tty.tcgetattr(fd)
			new = old[:]
		except :
			log.msg('not a typewriter!')
		else :
			# iflage
			new[0] = new[0] | tty.IGNPAR
			new[0] = new[0] & ~(tty.ISTRIP | tty.INLCR | tty.IGNCR | tty.ICRNL | tty.IXON | tty.IXANY | tty.IXOFF)
			if hasattr(tty, 'IUCLC') :
				new[0] = new[0] & ~tty.IUCLC

			# lflag
			new[3] = new[3] & ~(tty.ISIG | tty.ICANON | tty.ECHO | tty.ECHO |
								tty.ECHOE | tty.ECHOK | tty.ECHONL)
			if hasattr(tty, 'IEXTEN') :
				new[3] = new[3] & ~tty.IEXTEN

			#oflag
			new[1] = new[1] & ~tty.OPOST

			new[6][tty.VMIN] = 1
			new[6][tty.VTIME] = 0

			self._savedMode = old
			tty.tcsetattr(fd, tty.TCSANOW, new)
			#tty.setraw(fd)
			self._inRawMode = True

	def _leaveRawMode(self) :
		if not self._inRawMode :
			return

		fd = sys.stdin.fileno()
		tty.tcsetattr(fd, tty.TCSANOW, self._savedMode)
		self._inRawMode = False

	def handleInput(self, char) :
		#log.msg('handling %s' % repr(char))
		if char in ('\n', '\r') :
			self.escapeMode = 1
			self.write(char)
		elif self.escapeMode == 1 and char == self.OPTIONS['escape'] :
			self.escapeMode = 2
		elif self.escapeMode == 2 :
			self.escapeMode = 1 # so we can chain escapes together
			if char == '.': # disconnect
				log.msg('disconnecting from escape')
				self.stopConnection()
				return
			elif char == '\x1a': # ^Z, suspend
				def _() :
					self._leaveRawMode()
					sys.stdout.flush()
					sys.stdin.flush()
					os.kill(os.getpid(), signal.SIGTSTP)
					self._enterRawMode()
				reactor.callLater(0, _)
				return
			elif char == 'R': # rekey connection
				log.msg('rekeying connection')
				self.conn.transport.sendKexInit()
				return
			elif char == '#': # display connections
				self.stdio.write('\r\nThe following connections are open:\r\n')
				channels = self.conn.channels.keys()
				channels.sort()
				for channelId in channels :
					self.stdio.write('  #%i %s\r\n' % (channelId, str(self.conn.channels[channelId])))
				return
			self.write('~' + char)
		else :
			self.escapeMode = 0
			self.write(char)

	def dataReceived(self, data) :
		self.stdio.write(data)

	def extReceived(self, t, data) :
		if t==connection.EXTENDED_DATA_STDERR :
			log.msg('got %s stderr data' % len(data))
			sys.stderr.write(data)

	def eofReceived(self) :
		log.msg('got eof')
		self.stdio.closeStdin()

	def closeReceived(self) :
		log.msg('remote side closed %s' % self)
		self.conn.sendClose(self)

	def closed(self) :
		global old
		log.msg('closed %s' % self)
		log.msg(repr(self.conn.channels))
		if not self.OPTIONS['nocache']: # fork into the background
			if os.fork() :
				if old :
					fd = sys.stdin.fileno()
					tty.tcsetattr(fd, tty.TCSANOW, old)
				if (self.OPTIONS['command'] and self.OPTIONS['tty']) or \
					not self.OPTIONS['notty'] :
					signal.signal(signal.SIGWINCH, signal.SIG_DFL)
				os._exit(0)
			os.setsid()
			for i in range(3) :
				try :
					os.close(i)
				except OSError, e :
					if e.errno != errno.EBADF :
						raise

	def request_exit_status(self, data) :
		global exitStatus
		exitStatus = int(struct.unpack('>L', data)[0])
		log.msg('exit status: %s' % exitStatus)

	def sendEOF(self) :
		self.conn.sendEOF(self)

	def stopWriting(self) :
		self.stdio.stopReading()

	def startWriting(self) :
		self.stdio.startReading()

	def _windowResized(self, *args) :
		winsz = fcntl.ioctl(0, tty.TIOCGWINSZ, '12345678')
		winSize = struct.unpack('4H', winsz)
		newSize = winSize[1], winSize[0], winSize[2], winSize[3]
		self.conn.sendRequest(self, 'window-change', struct.pack('!4L', *newSize))


class SSHListenClientForwardingChannel(forwarding.SSHListenClientForwardingChannel) : pass
class SSHConnectForwardingChannel(forwarding.SSHConnectForwardingChannel) : pass

# Rest of code in "run"
exitStatus = 0
old = None

class Run :
	OPTIONS = dict()
	OBJECTS = dict()

	CONNECTION_LIST = None

	def __init__ (self, **options) :

		self.OPTIONS = ClientOptions()

	def __parseOptions (self, options) :
		self.OPTIONS.update(options)

		if options.has_key("uid") and options["uid"].strip() :
			__data = self.__get_info_by_gconf(options["uid"])

			if not __data.has_key("host") or  \
					not __data.has_key("user") or not __data.has_key("port") :
				raise ValueError, "connection information was not completed."

			self.OPTIONS["host"] = __data["host"]
			self.OPTIONS["user"] = __data["user"]
			self.OPTIONS["port"] = __data["port"]

	def __get_info_by_gconf (self, uid) :
		self.CONNECTION_LIST = ItemConnection.Them()
		self.CONNECTION_LIST.__import__(Conf.FILE_CONNECTION_LIST)

		__data = self.CONNECTION_LIST.get_value(uid, "asDict")
		"""
			__data = { \ 
				'name': 'alias name',
				'host': 'host name',
				'password': 'spike's password',
				'__uid__': '....................0...0',
				'user': 'user name',
				'ssh_version': ssh version,
				'port': port number, usually 22.
			}
		"""

		if not __data :
			return None
		else :
			return __data

	def set_objects (self, objects) :
		self.OBJECTS = objects

	def run (self, **options) :

		if len(options) > 0 :
			self.__parseOptions(options)

		self.__doConnect()

		fd = sys.stdin.fileno()
		try :
			old = tty.tcgetattr(fd)
		except :
			old = None

		try :
			reactor.run()
		finally :
			if old: tty.tcsetattr(fd, tty.TCSANOW, old)

			if (self.OPTIONS['command'] and self.OPTIONS['tty']) or not self.OPTIONS['notty'] :
				signal.signal(signal.SIGWINCH, signal.SIG_DFL)

		if sys.stdout.isatty() and not self.OPTIONS['command'] :
			print 'Connection to %s closed.' % self.OPTIONS['host']

		sys.exit(0)

	def __doConnect (self) :
	#	log.deferr = self.__handleError # HACK
		if '@' in self.OPTIONS['host'] :
			self.OPTIONS['user'], self.OPTIONS['host'] = self.OPTIONS['host'].split('@',1)
		if not self.OPTIONS.identitys :
			self.OPTIONS.identitys = ['~/.ssh/id_rsa', '~/.ssh/id_dsa']
		host = self.OPTIONS['host']
		if not self.OPTIONS['user'] :
			self.OPTIONS['user'] = getpass.getuser()
		if not self.OPTIONS['port'] :
			self.OPTIONS['port'] = 22
		else :
			self.OPTIONS['port'] = int(self.OPTIONS['port'])
		host = self.OPTIONS['host']
		port = self.OPTIONS['port']
		vhk = default.verifyHostKey
		SC = SSHConnection()
		SC.set_parameters(self.OPTIONS, self.OBJECTS)
		uao = SSHUserAuthClient(self.OPTIONS['user'], self.OPTIONS, SC)

		connect.connect(host, port, self.OPTIONS, vhk, uao).addErrback(self.__ebExit)

	def __ebExit(self, f) :
		global exitStatus
		if hasattr(f.value, 'value') :
			s = f.value.value
		else :
			s = str(f)
		exitStatus = "conch: exiting with error %s" % f
		reactor.callLater(0.1, _stopReactor)

	def __handleError (self) :
		from twisted.python import failure
		global exitStatus
		exitStatus = 2
		reactor.callLater(0.01, _stopReactor)
		#log.err(failure.Failure())
		raise

if __name__ == '__main__' :
	R = Run(user="ekips", host="localhost", port=22, password="ekips")

	R.set_objects()

	R.run()

"""
Description
-----------


ChangeLog
---------


Usage
-----


"""

__author__ =  "Spike^ekipS <spikeekips@gmail.com>"
__version__=  "0.1"
__nonsense__ = ""

__file__ = "SSHClient.py"


