# -*- coding: utf-8 -*-
#    Copyright 2005 Spike^ekipS <spikeekips@gmail.com>
#
#       This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import os, gnome, gtk, gconf

# gnome.vfs was deprecated, use gnomevfs.
try :
	import gnomevfs
except ImportError :
	import gnome.vfs

from vee import func, child, __OS__, Conf__

def program_init (**kwargs) :
	if not kwargs.has_key("PARAM_APP_DATADIR") :
		raise ValueError, " must be given."

	if not kwargs.has_key("app_name") :
		raise ValueError, " must be given."

	if not kwargs.has_key("version") :
		raise ValueError, " must be given."

	props = {gnome.PARAM_APP_DATADIR : kwargs["PARAM_APP_DATADIR"], }

	gnome.program_init( \
		kwargs["app_name"], kwargs["version"], properties=props)

def get_default_monospace_font () :
	__client = gconf.client_get_default();
	__client.add_dir("/", gconf.CLIENT_PRELOAD_NONE)

	__v = __client.get("/desktop/gnome/interface/monospace_font_name")
	return __v.get_string()

def get_default_font () :
	__client = gconf.client_get_default();
	__client.add_dir("/", gconf.CLIENT_PRELOAD_NONE)

	__v = __client.get("/desktop/gnome/interface/font_name")
	return __v.get_string()

class __ICON__ :

	THEME = None
	DEFAULT_ICON_NAME = "gnome-mime-text"
	DEFAULT_ICON_SIZE = "ICON_SIZE_BUTTON"
	ICON_SIZE_POOL = dict()
	ICON_SIZE = 20

	def __init__ (self, icon_size=None, default_size_pool=dict()) :
		"""
		icon size must be on of,
			ICON_SIZE_BUTTON
			ICON_SIZE_DIALOG
			ICON_SIZE_DND
			ICON_SIZE_INVALID
			ICON_SIZE_LARGE_TOOLBAR
			ICON_SIZE_MENU
			ICON_SIZE_SMALL_TOOLBAR
		"""

		self.THEME = gtk.icon_theme_get_default()
		if not default_size_pool :
			self.ICON_SIZE_POOL = get_default_icon_size()

		if icon_size :
			self.ICON_SIZE = self.ICON_SIZE_POOL[icon_size]
		else :
			self.ICON_SIZE = self.ICON_SIZE_POOL[self.DEFAULT_ICON_SIZE]

	def setSize (self, size) :
		self.ICON_SIZE = size

	def getDefaultPixbuf (self) :
		return self.getPixbufByIcon(self.DEFAULT_ICON_NAME)

	def getPixbufByIcon (self, icon_name) :
		return \
			get_pixbuf_by_icon(self.THEME, icon_name, self.ICON_SIZE)

	def getPixbufFromFile (self, filepath) :
		return \
			get_pixbuf_from_file(filepath, self.ICON_SIZE)

	def getPixbufByFilename (self, filepath) :
		return \
			get_pixbuf_by_filename(self.THEME, filepath, self.ICON_SIZE)
	def getPixbufBlank (self) :
		return get_blank_pixbuf(self.ICON_SIZE)

class Icon (__ICON__) :

	def getPixbufByChildConfig (self, childname, config) :
		if config["layout"]["icon"]["image"] :
			__icon_path = \
				os.path.join( \
					child.get_path_child(childname), \
					config["layout"]["icon"]["image"] \
				)
			if not __OS__.is_file(__icon_path) :
				__pixbuf = self.getPixbufBlank()
			else :
				__pixbuf = self.getPixbufFromFile(__icon_path)
		elif config["layout"]["icon"]["stock_id"] :
			__pixbuf = \
				self.getPixbufByIcon(config["layout"]["icon"]["stock_id"])
		else :
			__pixbuf = self.getPixbufBlank()

		return __pixbuf

##################################################
# FUNCTION
def get_default_style () :
	return gtk.widget_get_default_style()

def get_color_from_default_style () :
	"""
	gtk.STATE_NORMAL
		The state of a sensitive widget that is not active and does not have the focus

	gtk.STATE_ACTIVE
		The state of a sensitive widget when it is active e.g. a button that is
		pressed but not yet released

	gtk.STATE_PRELIGHT
		The state of a sensitive widget that has the focus e.g. a button that has
		the mouse pointer over it.

	gtk.STATE_SELECTED
		The state of a widget that is selected e.g. selected text in a gtk.Entry
		widget

	gtk.STATE_INSENSITIVE
		The state of a widget that is insensitive and will not respond to any
		events e.g. cannot be activated, selected or prelit.
	"""
	style = gtk.widget_get_default_style()

	__colors = { \
		"text" : ( \
			style.text[gtk.STATE_NORMAL], \
			style.text[gtk.STATE_ACTIVE], \
			style.text[gtk.STATE_PRELIGHT], \
			style.text[gtk.STATE_SELECTED], \
			style.text[gtk.STATE_INSENSITIVE], \
		), \
		"base" : ( \
			style.base[gtk.STATE_NORMAL], \
			style.base[gtk.STATE_ACTIVE], \
			style.base[gtk.STATE_PRELIGHT], \
			style.base[gtk.STATE_SELECTED], \
			style.base[gtk.STATE_INSENSITIVE], \
		), \
		"bg" : ( \
			style.bg[gtk.STATE_NORMAL], \
			style.bg[gtk.STATE_ACTIVE], \
			style.bg[gtk.STATE_PRELIGHT], \
			style.bg[gtk.STATE_SELECTED], \
			style.bg[gtk.STATE_INSENSITIVE], \
		), \
		"fg" : ( \
			style.fg[gtk.STATE_NORMAL], \
			style.fg[gtk.STATE_ACTIVE], \
			style.fg[gtk.STATE_PRELIGHT], \
			style.fg[gtk.STATE_SELECTED], \
			style.fg[gtk.STATE_INSENSITIVE], \
		), \
	}

	return __colors

def get_default_icon_size () :
	"""
	Get the default size of icon by stock icon size.
	"""
	__icon_size = dict()
	for i  in dir(gtk) :
		if i.startswith("ICON_SIZE_") :
			__icon_size[i] = \
				get_size_from_stock(gtk.STOCK_FILE, getattr(gtk, i))[0]

	return __icon_size

def get_pixbuf_by_filename (theme, filepath, size) :
	if not __OS__.exists(filepath) :
		return None

	__icon_name = get_icon_name(filepath)

	return get_pixbuf_by_icon(theme,__icon_name, size)

def get_pixbuf_from_stock (stock_id, size) :
	if size == gtk.ICON_SIZE_INVALID :
		return None

	return gtk.Label().render_icon(stock_id, size)

def get_pixbuf_from_file_without_size (filepath) :
	if not __OS__.exists(filepath) :
		return None

	__image = gtk.Image()
	__image.set_from_file(filepath)

	return __image.get_pixbuf()

def get_pixbuf_from_file (filepath, width, height=None) :
	if not __OS__.exists(filepath) :
		return None
	if not height : height = width

	return gtk.gdk.pixbuf_new_from_file_at_size(filepath, width, height)

def get_pixbuf_by_icon (theme, icon_name, size=28) :
	"""
	gtk.ICON_LOOKUP_FORCE_SVG can handle svg and png, etc.
	"""
	if not theme.lookup_icon(icon_name, size, gtk.ICON_LOOKUP_FORCE_SVG) :
		return None
	else :
		return theme.load_icon(icon_name, size, gtk.ICON_LOOKUP_FORCE_SVG)

def get_size_from_stock (stock_id, size=gtk.ICON_SIZE_MENU) :
	__pixbuf = get_pixbuf_from_stock(stock_id, size)
	if not __pixbuf :
		return (None, None, )
	else :
		return (__pixbuf.get_property("width"), __pixbuf.get_property("height"), )

def get_mime_type (filepath) :
	if not __OS__.exists(filepath) :
		return None

	return gnomevfs.get_mime_type(filepath)

def get_mime_description (filepath) :
	if not __OS__.exists(filepath) :
		return None

	__mimetype = get_mime_type(filepath)
	if not __mimetype :
		return str()
	else :
		return gnomevfs.mime_get_description(__mimetype)

def get_icon_name (filepath) :

	STR_MIME_TYPE = "gnome-mime-%s"
	filepath = os.path.abspath(filepath)

	if not __OS__.exists(filepath) :
		return None

	__mimetype = gnomevfs.get_mime_type(filepath)
	if not __mimetype :
		return None

	# text/plain is 'text'
	if __mimetype == "text/plain" :
		__mimetype = "text"

	__icon_name = STR_MIME_TYPE % \
		("-".join(__mimetype.split("/")))

	return __icon_name

def get_blank_pixbuf(size=28) :
	return func.get_pixbuf_filled_with_color(None, (size, size, ))

def get_logo_image_filename (lang) :
	if lang != "ko" :
		lang = "en"

	return os.path.join(Conf__.DIR_PIXMAPS, "vee-logo-%s.svg" % lang)

"""
Description
-----------


ChangeLog
---------


Usage
-----


"""

__author__ =  "Spike^ekipS <spikeekips@gmail.com>"
__version__=  "0.1"
__nonsense__ = ""

__file__ = "GNOME.py"


