# -*- coding: utf-8 -*-
#    Copyright (C) 2005 Spike^ekipS <spikeekips@gmail.com>
#
#    http://vee.kldp.net
#
#       This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import gobject, gtk
from vee import WIDGET, __VEE__, i18n, Label, Conf, func
_ = i18n.gettext()

TEMPLATE_TITLE_WITH_LABEL = "%s : %s"
TEMPLATE_TITLE_WITHOUT_LABEL = "%s"

class Page (__VEE__.Core) :

	OBJ_CHILD = None
	NOTEBOOK = None
	MAIN_WINDOW = None
	LABEL = None

	WIDGET_CHILD = None
	BOX_CHILD = None

	PROPERTIES_CHILD = dict()

	EVENT_ID_TITLE_CHANGED = None

	def __init__ (self, state, notebook, obj_child) :

		__VEE__.Core.__init__(self, state=state)

		self.NOTEBOOK = notebook
		self.OBJ_CHILD = obj_child

		self.WIDGET_CHILD = self.OBJ_CHILD.getWidget()
		self.BOX_CHILD = self.OBJ_CHILD.getBox()

		self.PROPERTIES_CHILD = self.OBJ_CHILD.getProperties()
		self.CONFIG_CHILD = self.OBJ_CHILD.getConfig()

		self.TAB_LIST = self.STATE_LOCAL["tab-list"]
		self.MAIN_WINDOW = self.STATE_LOCAL["main-window"]

	def __del__ (self) :
		self.OBJ_CHILD.__del__()

	def getObjectChild (self) :
		return self.OBJ_CHILD

	def getWidgetChild (self) :
		return self.WIDGET_CHILD

	def getWidgetChildForNotebook (self) :
		return self.BOX_CHILD

	def getPageNumber (self) :
		return self.NOTEBOOK.page_num(self.BOX_CHILD)

	def createLabel (self) :
		if self.PROPERTIES_CHILD.has_key("label") :
			__labelname = self.PROPERTIES_CHILD["label"]
		else :
			__labelname = self.OBJ_CHILD.getLabelName()

		if not self.CONFIG_CHILD["layout"]["tab"].has_key("closebutton") :
			return None

		if self.CONFIG_CHILD["layout"]["tab"]["closebutton"] == "withclose" :
			__module_label = Label.TabWithClose
		else :
			__module_label = Label.TabWithClose

		self.OBJ_LABEL = __module_label( \
			labelname=__labelname, \
			without_closebutton=False \
		)

		self.OBJ_LABEL.connect("clicked", self.on_remove)

		return self.OBJ_LABEL.getWidget()

	def attach (self) :
		# CREATE CLOSE LABEL. ##############################################
		__label = self.createLabel()

		# ADD THIS TAB AS PAGE. ############################################
		self.NOTEBOOK.append_page(self.BOX_CHILD, __label)
		self.WIDGET_CHILD.grab_focus()

		self.OBJ_CHILD.attached(self)

		# ETC. #############################################################
		self.__setWindowTitle(self.OBJ_CHILD.getTitle())

		return True

	def remove (self) :
		page_num = self.NOTEBOOK.page_num(self.BOX_CHILD)
		self.NOTEBOOK.remove_page(page_num)
		del self.TAB_LIST[page_num]
		self.NOTEBOOK.emit("change-current-page", Conf.NOTEBOOK_REMOVE_OFFSET)
		self.__del__()

	def on_changed_title (self, widget) :

		__t = self.OBJ_CHILD.getTitle()
		#self.OBJ_LABEL.set_label(__title)
		if type(__t) not in (str, unicode, ) : return

		if self.PROPERTIES_CHILD.has_key("label") :
			__title = TEMPLATE_TITLE_WITH_LABEL % \
				(__t, self.PROPERTIES_CHILD["label"], )
		else :
			__title = TEMPLATE_TITLE_WITHOUT_LABEL % __t

		self.__setWindowTitle(__title)

		self.OBJ_CHILD.on_changed_title()

	def __setWindowTitle (self, title) :
		WIDGET.set_title( \
			self.MAIN_WINDOW, \
			self.GCONF_CLIENT, \
			title, \
		  	"this is file path" \
		)

	def on_remove (self, widget, parm0=None) :
		self.remove()

	def getName (self) :
		return self.OBJ_CHILD.myNameIs()

class List (list) :

	def getChild (self, index) :
		__one = self[index]

		return __one.WIDGET_CHILD

	def getChildWidget (self, index) :
		return self.getChild(index)

class History (list) :

	NOTEBOOK = None

	def __init__ (self, notebook) :
		self.NOTEBOOK = notebook
		list.__init__(self)

	def append (self, obj) :
		if len(self) > self.NOTEBOOK.get_n_pages() :
			del self[:len(self) - self.NOTEBOOK.get_n_pages() + 1]

		list.append(self, obj)

	def __getPrevious (self, page_list) :
		__latest = page_list[-1]

		if not isinstance(__latest, (gobject.GPointer, gtk.Widget, )) :
			return None

		return self.NOTEBOOK.page_num(__latest)

	def getPrevious (self) :
		if len(self) < 1 :
			return

		self.pop()
		if len(self) < 1 :
			return None

		__page_num = None
		page_list = self
		while not __page_num or __page_num != -1:
			if len(page_list) < 1 :
				break
			__page_num = self.__getPrevious(page_list)
			page_list = page_list[:-1]

		return __page_num

	def goPrevious (self) :
		__page_num = self.getPrevious()
		if type(__page_num) is int :
			gobject.idle_add(self.NOTEBOOK.set_current_page, __page_num)

			# add this page in history.
			__page = self.NOTEBOOK.get_nth_page(__page_num)
			self.append(__page)

			return True

		return False


"""
Description
-----------


ChangeLog
---------


Usage
-----


"""

__author__ =  "Spike^ekipS <spikeekips@gmail.com>"
__version__=  "0.1"
__nonsense__ = ""

__file__ = "TAB.py"

