# -*- coding: utf-8 -*-
#    Copyright 2005 Spike^ekipS <spikeekips@gmail.com>
#
#       This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import tempfile, os, pickle, time, sre, xmlrpclib
import gtk, gobject, shutil, gnome
from vee import __VEE__, Conf__, Conf, Accel, i18n, WIDGET, \
	func, State, TAB, Handler, __OS__, Panel, child, Dialog, \
	GNOME, Menu, Widgets, GConf, Clipboard, i18n

from vee.child import __PROPERTY__

_ = i18n.gettext()

class VEE (WIDGET.WIDGET) :

	STATE_GLOBAL = None
	STATE_LOCAL = None
	GCONF_CLIENT = None

	UID = None
	TAB_LIST = None

	ACCEL_GROUP_GLOBAL = None
	LIST_TAB_SHORTCUT = dict()

	IS_COMPACT = False
	IS_FULLSCREEN = False

	COMMAND = None

	FRONT_PAGE = None

	def __init__ (self, state_global, **kwargs) :
		######################################## GET ARGUMENT
		if not isinstance(state_global, State.VEE) :
			raise RuntimeError, "state_global must be given."
		else :
			self.STATE_GLOBAL = state_global

		if kwargs.has_key("args_options") :
			self.ARGS_OPTIONS = kwargs["args_options"]

		if kwargs.has_key("args_args") :
			self.ARGS_ARGS = kwargs["args_args"]

		######################################## get the state info.
		self.__doStateJob()

		############################# load and apply the default font for VEE.
		__use_system_default_font = \
			self.GCONF_CLIENT.getValue("use_system_default_font")
		if not __use_system_default_font :
			__fontname_vee = self.GCONF_CLIENT.getValue("font")
			if __fontname_vee.strip() :
				gtk.rc_parse_string(Conf.TEMPLATE_RC_FILE % {"fontname" : __fontname_vee})

		######################################## create main window.
		WIDGET.WIDGET.__init__(self, "vee")
		self.STATE_LOCAL.set_value("main-window", self.WIDGET_SELF)

		# load initial window.
		self.__loadInitialWindows()

		self.WIDGET_SELF.present()

	# INTERNAL METHOD ##################################################
	def __loadInitialWindows (self) :
		__data = self.GCONF_CLIENT.getValue("window_state")
		try :
			__b = xmlrpclib.loads(__data)
			if __b[0][0]["window-bookmark"] :
				Bookmark(state=self.STATE)
		except :
			return

	def __handlingARGV (self) :
		# fullscreening?
		if self.ARGS_OPTIONS.ARG_FULLSCREEN_WINDOW :
			self.IS_FULLSCREEN = False
			self.toggleFullScreen()
		else :
			# maximizing?
			if self.ARGS_OPTIONS.ARG_MAXIMIZE_WINDOW :
				self.WIDGET_SELF.maximize()
			elif self.ARGS_OPTIONS.ARG_GEOMETRY :
				# set geometry
				(w, h, x, y) = WIDGET.parse_geometry(self.ARGS_OPTIONS.ARG_GEOMETRY)
				WIDGET.set_window_geometry(self.WIDGET_SELF, (w, h, x, y, ))
			else :
				########### restore the latest window geomtry.
				try :
					(w, h, x, y, ) = \
						[ int(i) for i in \
							self.GCONF_CLIENT.getValue("latest_window_geometry").split(",")]
				except ValueError :
					(w, h, x, y, ) = (700, 700, 10, 10, )

				WIDGET.set_window_geometry(self.WIDGET_SELF, (w, h, x, y, ))

		if self.ARGS_OPTIONS.ARG_START_WITH_VIM : # start vim
			__initial_module = "vim"
		elif self.ARGS_OPTIONS.ARG_START_WITH_TERMINAL : # start terminal
			__initial_module = "terminal"
		else : # start initial module.
			__initial_module = self.GCONF_CLIENT.getValue("initial_module")

		if __initial_module != "None" :
			# When i run this routine without gobject.timeout_add, vte widget could
			# not handle the signal, child-exited, so i wrote like this.
			gobject.timeout_add(1, self.__runInitialModule, __initial_module)

	def __runInitialModule (self, child_name) :
		__config = dict()
		if child_name == "from_bookmark" :
			__uid = self.GCONF_CLIENT.getValue("initial_module_from_bookmark")
			__config = self.STATE_GLOBAL["bookmark"].getValuesByUID(__uid)
			if not __config :
				child_name = None
			else :
				child_name = __config["type"]

		if child_name :
			self.addPage(child_name, **__config)

		return False

	def __doStateJob (self) :
		#################### create it's state
		self.STATE_LOCAL = State.VEE()
		self.STATE = {"local": self.STATE_LOCAL, "global": self.STATE_GLOBAL}

		#################### create it's own uid
		self.UID = func.get_uid()
		self.STATE_GLOBAL.append("list-vee", self.UID)
		self.STATE_LOCAL.set_value("uid", self.UID)
		self.STATE_LOCAL.set_value("obj-vee", self)

		#################### create state and initialize instances.
		self.GCONF_CLIENT = self.STATE_GLOBAL.get_value("gconf-client")
		self.STATE_LOCAL.set_value("gconf-client", self.GCONF_CLIENT)

		self.COMMAND = self.STATE_GLOBAL["command"]

		self.CLIPBOARD = Clipboard.Clipboard()
		self.STATE_LOCAL.set_value("clipboard", self.CLIPBOARD)

		self.STATE_LOCAL.set_value("window-bookmark", None)

	def initialize (self) :
		########### set window geometry
		self.WIDGET_SELF.set_data("window-position", self.WIDGET_SELF.get_position())
		self.WIDGET_SELF.set_data("window-size", self.WIDGET_SELF.get_size())

		########### "NOTEBOOK"
		self.NOTEBOOK = self.getWidget("notebook")
		self.TAB_LIST = TAB.List()
		self.TAB_HISTORY = TAB.History(self.NOTEBOOK)
		self.STATE_LOCAL.set_value("tab-list", self.TAB_LIST)
		self.STATE_LOCAL.set_value("notebook", self.getWidget("notebook"))

		# enable DND in notebook
		self.NOTEBOOK.drag_dest_set( \
			gtk.DEST_DEFAULT_MOTION |
			gtk.DEST_DEFAULT_HIGHLIGHT |
			gtk.DEST_DEFAULT_DROP,
			[ \
				( "text/uri-list",  0, Conf.DND_TARGET_URI_LIST ),
			], \
			gtk.gdk.ACTION_COPY \
		)
		self.NOTEBOOK.connect("drag_data_received", \
			self.on_drag_data_received_data_in_notebook)

		########### "EVENTBOX"
		#EventBox_Notebook = self.getWidget("eventbox1")

		########### set FRONT PAGE.
		(__xml, self.FRONT_PAGE) = WIDGET.get_widget("box_frontpage")
		(__image_logo, __label_logo, ) = self.FRONT_PAGE.get_children()
		__image_logo.set_from_file( \
			GNOME.get_logo_image_filename(self.STATE_GLOBAL["lang"]) \
		)
		self.NOTEBOOK.remove_page(0)
		self.toggleLogo()

		########### add ACCELERATOR
		self.ACCEL_GROUP_GLOBAL = \
			self.__installAccelGroup(Conf.LIST_KEYBOARD_SHORTCUT)

		########## add signal
		self.WIDGET_SELF.connect("key-press-event", self.on_key_press_event)
		self.WIDGET_SELF.connect("event", self.on_event)

		########## initialize children
		for __child_name in self.STATE_GLOBAL["children"].keys() :
			try :
				__class = child.get_child_main(__child_name)
			except ValueError :
				print "%s could not be initialized." % __child_name
				continue
			else :
				__class(self, state=self.STATE)

		######################################## connect gconf notify
		__notify = GConf.Notify(state=self.STATE)
		__notify.add()
		__notify.addChild()

		# args and options.
		self.__handlingARGV()

	# EVENT HANDLER ##################################################
	def on_drag_data_received_data_in_notebook \
			(self, notebook, context, x, y, selection, info, etime) :
		__filelist = list()
		for i in __OS__.get_list_from_uri_list(selection.data) :
			(__scheme, __filepath, ) = __OS__.get_pathinfo_from_uri(i)
			if __scheme != "file" :
				continue
			if not __OS__.exists(__filepath) :
				continue

			__filelist.append(__filepath)

		for i in __filelist :
			if __OS__.is_file(i) :
				self.addPage("vim", filename=i)
			elif __OS__.is_dir(i) :
				self.addPage("terminal", path=i)

	def on_event (self, window, event) :
		# remember window geometry
		if event.type == gtk.gdk.CONFIGURE :
			(__x, __y, ) = window.get_position()
			(__x0, __y0, ) = window.get_data("window-position")
			(__width, __height, ) = window.get_size()
			(__width0, __height0, ) = window.get_data("window-size")

			if (__x, __y, ) != (__x0, __y0, ) : # it's move.
				window.set_data("window-position", (__x, __y, ))
			elif (__width, __height, ) != (__width0, __height0, ) : # it's resizing.
				window.set_data("window-size", window.get_size())

		return False

	def on_about (self, widget) :
		About()

	def on_key_press_event (self, widget, event) :
		__keyname = func.get_keyname(event)

		# run YELP and not send keystroke to childs.
		if __keyname == "F1" :
			self.on_help(None)
			return True

		return False

	def on_button_release_event (self, widget, event) :
		return False

	def nf_update_accel_key (self, client, parm1, entry, parm3) :
		uid = os.path.basename(entry.get_key())

		# get new key and mods.
		__key, __mod = \
			gtk.accelerator_parse(self.GCONF_CLIENT.getValueFromValue(entry.get_value()))

		if not __key or __mod == Conf.NULL_MODIFIER_MASK :
			return self.ACCEL_GROUP_GLOBAL.removeKey(uid)

		return self.ACCEL_GROUP_GLOBAL.updateKey(uid, __key, __mod)

	def on_set_toggle_button (self, widget, which, showOrHide=False) :
		__handler_id = which.get_data("handler_id")

		which.handler_block(__handler_id)
		which.set_active(showOrHide)
		which.handler_unblock(__handler_id)

	def on_toggle_fullscreen (self, widget) :
		self.toggleFullScreen()

	def on_preferences (self, widget) :
		Preferences(state=self.STATE)

	def on_quit (self, widget, parm0=None, parm1=None, parm2=None) :
		if len(self.TAB_LIST) > 0 :
			Dialog.AskCloseAllTabDialog( \
				callback_functions={"functions_quit" : self.quitVee})
		else :
			self.quitVee()

		return True

	def on_close (self, widget, parm0=None, parm1=None, parm2=None) :
		current_page = self.NOTEBOOK.get_current_page()

		if current_page == -1 : # there are no pages.
			return

		try :
			self.TAB_LIST[current_page].remove()
		except :
			self.NOTEBOOK.remove_page(current_page)
			try :
				del self.TAB_LIST[current_page]
			except IndexError :
				return

	def on_notebook_change_current_page (self, notebook, offset) :
		if offset == Conf.NOTEBOOK_REMOVE_OFFSET :
			self.toggleLogo()
			self.__removeAccelKeySwitchTab()

			if self.NOTEBOOK.get_n_pages() < 1 :
				self.toggleLogo()
			else :
				self.TAB_HISTORY.goPrevious()

	def on_notebook_switch_page (self, notebook, page, page_num) :
		# refresh the window title.
		try :
			__tab_current = self.TAB_LIST[page_num]
		except :
			WIDGET.set_title(self.WIDGET_SELF, self.GCONF_CLIENT, _("VEE"))
		else :
			__tab_current.on_changed_title(None)

	def on_insert_child (self, widget, childname, parm0=None) :
		#if parm0 : self.__changeAddNewButton(widget, childname)

		self.addPage(childname)

	def on_help (self, parm1) :
		gnome.help_display(Conf__.INFO["name"])

	# ACCELERATOR METHOD ##################################################
	def ac_open_file_select_dialog (self, parm1, parm2, parm3, parm4) :
		Dialog_SelectFileAndDirectory(state=self.STATE)

	def ac_tab_list (self, parm1, parm2, parm3, parm4) :
		TabList(state=self.STATE)

	def ac_toggle_bookmark_window (self, parm1, parm2, parm3, parm4) :
		if not self.STATE_LOCAL["window-bookmark"] :
			Bookmark(state=self.STATE)
		else :
			if self.STATE_LOCAL["window-bookmark"].isShown() :
				self.STATE_LOCAL["window-bookmark"].hide()
			else :
				self.STATE_LOCAL["window-bookmark"].show()

	def ac_duplication_tab (self, parm1, parm2, parm3, parm4) :
		__current_page = self.NOTEBOOK.get_current_page()
		try :
			__tab = self.TAB_LIST[__current_page]
		except IndexError :
			return

		__obj = __tab.getObjectChild()
		__properties = __obj.getProperties()
		__childname = __obj.myNameIs()
		self.addPage(__childname, **__properties)

	def ac_dashboard (self, parm1, parm2, parm3, parm4) :
		__current_page = self.NOTEBOOK.get_current_page()
		try :
			__tab = self.TAB_LIST[__current_page]
		except IndexError :
			pass

		DashBoard(state=self.STATE)
		self.WIDGET_SELF.set_property("sensitive", False)

	def ac_show_tab_list (self, parm1, parm2, parm3, parm4) :
		pass

	def ac_change_nth_tab (self, accelgroup, window, keyval, flag) :
		"""
		손경욱(vvs740@nospam@chol.com)님이 보내준 패치를 수정해서 적용.
		"""
		__n = self.LIST_TAB_SHORTCUT[keyval]
		if __n == self.NOTEBOOK.get_current_page() : return

		self.NOTEBOOK.grab_focus()
		self.NOTEBOOK.set_current_page(__n)
		self.setFocus()

		try :
			__page = self.TAB_LIST[__n].getWidgetChildForNotebook()
		except IndexError :
			pass
		else :
			self.TAB_HISTORY.append(__page) # append page in notebook.

	def ac_next_tab (self, parm1, parm2, parm3, parm4) :
		self.NOTEBOOK.next_page()

	def ac_previous_tab (self, parm1, parm2, parm3, parm4) :
		self.NOTEBOOK.prev_page()

	def ac_font_increase (self, parm1, parm2, parm3, parm4) :
		self.NOTEBOOK.grab_focus()

		__n = self.NOTEBOOK.get_current_page()
		try :
			self.TAB_LIST[__n].getObjectChild().increase_font()
		except :
			pass

		self.setFocus()
		return True

	def ac_font_decrease (self, parm1, parm2, parm3, parm4) :
		self.NOTEBOOK.grab_focus()

		__n = self.NOTEBOOK.get_current_page()
		try :
			self.TAB_LIST[__n].getObjectChild().decrease_font()
		except :
			pass

		self.setFocus()
		return True

	def ac_toggle_full_screen (self, widget, parm1, parm2, parm3) :
		self.toggleFullScreen()

	# ACTION METHOD ##################################################
	def addPage (self, childname, **kwargs) :
		# FIXME:
		#if self.GCONF_CLIENT.getValueByConf("use_%s" % childname) is False :
		#	return

		__obj_egg = child.get_child(childname, self.STATE, **kwargs)
		if not __obj_egg :
			return

		__page = TAB.Page(self.STATE, self.NOTEBOOK, __obj_egg)
		if not __page.attach() :
			return

		self.toggleLogo() # hide logo image.

		self.TAB_LIST.append(__page) # append page in notebook.
		self.TAB_HISTORY.append(__page.getWidgetChildForNotebook()) # append page in notebook.

		self.NOTEBOOK.set_current_page(self.NOTEBOOK.get_n_pages() - 1) # switch page.
		__page.getWidgetChild().grab_focus() # set focus in child.

		self.__addAccelKeySwitchTab() # add accel key for switching this page.
		gobject.idle_add(__obj_egg.reset)

		self.WIDGET_SELF.present()
		self.setFocus()

	# MISC METHOD ##################################################
	def toggleLogo (self) :
		if self.NOTEBOOK.get_n_pages() == 0 :
			self.NOTEBOOK.append_page(self.FRONT_PAGE)
			self.NOTEBOOK.set_property("show-tabs", False)

			del self.TAB_HISTORY[:]

			if self.STATE_LOCAL["window-bookmark"] :
				self.STATE_LOCAL["window-bookmark"].WIDGET_SELF.present()
		else :
			__page_num_logo = self.NOTEBOOK.page_num(self.FRONT_PAGE)
			if __page_num_logo >= 0 :
				self.NOTEBOOK.set_property("show-tabs", True)
				self.NOTEBOOK.remove_page(self.NOTEBOOK.page_num(self.FRONT_PAGE))

	def grab_focus_in_tab (self) :
		__n = self.NOTEBOOK.get_current_page()
		try :
			self.TAB_LIST.getChildWidget(__n).grab_focus()
		except :
			pass

		try :
			__tab = self.TAB_LIST[__n]
		except IndexError :
			return

	def checkIsTabs (self) :
		if len(self.TAB_LIST) < 1 :
			return False
		else :
			return True

	def setFocus (self) :
		"""
		To set the focus in main window.
		"""
		self.NOTEBOOK.grab_focus()
		__n = self.NOTEBOOK.get_current_page()
		try :
			self.TAB_LIST.getChildWidget(__n).grab_focus()
		except :
			pass

	def quitVee (self) :
		# remove uid from global state.
		try :
			__index = self.STATE_GLOBAL["list-vee"].index(self.UID)
		except IndexError :
			pass
		except ValueError :
			pass
		else :
			del(self.STATE_GLOBAL["list-vee"][__index])

		try :
			shutil.rmtree(self.STATE_GLOBAL.get_value("directory-temp"))
		except :
			pass

		self.COMMAND.saveXml(Conf.FILE_COMMAND)

		# save the latest state of windows.
		__window_state = dict()
		if self.STATE_LOCAL["window-bookmark"] :
			__window_state["window-bookmark"] = \
				self.STATE_LOCAL["window-bookmark"].WIDGET_SELF.get_property("visible")

		self.GCONF_CLIENT.setValue( \
			"window_state", xmlrpclib.dumps((__window_state, )))

		# close bookmark window
		if self.STATE_LOCAL["window-bookmark"] :
			self.STATE_LOCAL["window-bookmark"].on_close(None)

		if not self.IS_FULLSCREEN :
			WIDGET.remember_geometry(self.WIDGET_SELF, self.GCONF_CLIENT)

		if self.STATE_GLOBAL["use_twisted"] :
			from twisted.internet import reactor
			reactor.stop()
		else :
			gtk.main_quit()

		return True

	def toggleFullScreen (self) :
		if not self.IS_FULLSCREEN :
			self.WIDGET_SELF.fullscreen()
		else :
			self.WIDGET_SELF.unfullscreen()

		self.IS_FULLSCREEN = not self.IS_FULLSCREEN

	def __removeAccelKeySwitchTab (self) :
		"""
		Remove the unused switching shortcut keys.
		"""
		__c = Conf.LIST_KEYBOARD_SHORTCUT["keyboard_shortcut_modifier_switch_tab"]

		for __keyval, __pagenum in self.LIST_TAB_SHORTCUT.items() :
			if __pagenum in range(self.NOTEBOOK.get_n_pages()) :
				continue

			__accel_key = __c[0] % (__pagenum + 1)
			self.ACCEL_GROUP_GLOBAL.removeKey(__accel_key)
			del self.LIST_TAB_SHORTCUT[__keyval]

	def __addAccelKeySwitchTab (self) :
		"""
		그놈 터미널이나 에피파니처럼 Alt+숫자키로 탭이동하는 패치.
		 - 손경욱(vvs740@nospam@chol.com)님이 보내준 패치를 수정해서 적용
		"""
		__c = Conf.LIST_KEYBOARD_SHORTCUT["keyboard_shortcut_modifier_switch_tab"]

		num = len(self.LIST_TAB_SHORTCUT)
		__accel_key = __c[0] % (num + 1)
		__key, __mod = gtk.accelerator_parse(__accel_key)

		if self.LIST_TAB_SHORTCUT.has_key(__key) :
			return
		else :
			self.LIST_TAB_SHORTCUT[__key] = num
			self.ACCEL_GROUP_GLOBAL.addKey(__accel_key, __key, __mod, eval(__c[1]))

	def __installAccelGroup (self, conf_accel_keys) :
		__GROUP = Accel.Group(self.GCONF_CLIENT)

		for i, j in conf_accel_keys.items() :
			__key, __mod, __callback = __GROUP.getDataByConf(i)
			if not __key : continue

			if not __GROUP.addKey(i, __key, __mod, eval(__callback)) :
				raise RuntimeError, "The key, %s was failed to install." % \
					__key
			else :
				# add gconf notification.
				self.GCONF_CLIENT.CLIENT.notify_add( \
					self.GCONF_CLIENT.generatePath(i), \
					self.nf_update_accel_key \
				)

		__GROUP.addToWindow(self.WIDGET_SELF)

		return __GROUP

class PROPERTY :

	def __init__ (self, childname, uid, **kwargs) :
		if childname == "category" :
			BookmarkPropertyCategory(uid, **kwargs)
		else :
			# get child property class.
			__bookmark = child.get_object_from_main(childname, "bookmark_property")
			__bookmark(uid, **kwargs)

class BookmarkPropertyCategory (__VEE__.Core, WIDGET.WIDGET, __PROPERTY__.CORE) :

	NOTEBOOK = None
	PANEL_BOOKMARK = None
	PANEL_PATH = None
	TREEITER = None
	UID = None
	NODE = None
	DATA = None
	CHILD_CONFIG = None

	def __init__ (self, uid, **kwargs) :
		__VEE__.Core.__init__(self, **kwargs)
		self.BOOKMARK = self.STATE_GLOBAL["bookmark"]
		self.UID = uid

		if not kwargs.has_key("panel") :
			raise RunetimeError, "panel must be given."
		else :
			self.PANEL_BOOKMARK = kwargs["panel"]
			(__treestore, self.TREEITER, ) = self.PANEL_BOOKMARK.get_selected()
			self.PANEL_PATH = self.PANEL_BOOKMARK.get_path(self.TREEITER)

		self.NODE = self.BOOKMARK.getNodeByUID(self.UID)

		# get data by uid
		self.DATA = self.BOOKMARK.getValuesByUID(self.UID)

		# get config
		WIDGET.WIDGET.__init__(self, "bookmark_property_category")

		self.NOTEBOOK = self.getWidget("notebook")

	def initialize (self) :
		# GENERAL ########################################################
		# connect handler
		self.connect_handler(__PROPERTY__.PROPERTY_BOOKMARK)

		# set type image
		GNOME_ICON = GNOME.Icon(Conf.ICON_SIZE_PROPERTY_TYPE)
		__icon = GNOME_ICON.getPixbufByIcon(Conf.ICONS["category"])
		self.getWidget("image1").set_from_pixbuf(__icon)

		# set label
		self.getWidget("entry1").set_text(self.DATA["label"])

class Preferences (__VEE__.Core, WIDGET.WIDGET) :

	GNOME_ICON = None

	def __init__ (self, **kwargs) :
		__VEE__.Core.__init__(self, **kwargs)
		WIDGET.WIDGET.__init__(self, "preferences")

	def initialize (self) :
		#self.getWidget("image-preference").set_from_file( \
		#	os.path.join(Conf__.DIR_PIXMAPS, "vee-logo.png"))
		#self.getWidget("notebook4").remove_page(2)

		# set fontname for vee. ###########################################
		__use_system_default_font = \
			self.GCONF_CLIENT.getValue("use_system_default_font")
		self.getWidget("hbox130").set_property("sensitive", \
			not __use_system_default_font)
		self.getWidget("checkbutton1").set_active(__use_system_default_font)

		__fontname_vee = self.GCONF_CLIENT.getValue("font")
		if not __fontname_vee.strip() :
			__fontname_vee = \
				self.GCONF_CLIENT.getValueByPath(Conf.GCONF_GNOME_INTERFACE_FONTNAME)

		self.getWidget("fontbutton1").set_font_name(__fontname_vee)

		# STARTUP CHILD MODULE. ###########################################
		self.GNOME_ICON = GNOME.Icon("ICON_SIZE_DND")

		# NOTICE :
		# ComboBox, which taken from glade xml could not handle the cell, the
		# alignment of the cell was weird, so make gtk.ComboBox directly.
		#__combobox_startup = self.getWidget("combobox4")
		__combobox_startup = gtk.ComboBox()
		__combobox_startup.connect("changed", self.on_changed_initial_module)
		__liststore = gtk.ListStore( \
			gobject.TYPE_OBJECT, gobject.TYPE_STRING, gobject.TYPE_STRING)
		__combobox_startup.set_model(__liststore)

		cell0 = gtk.CellRendererPixbuf()
		cell1 = gtk.CellRendererText()
		cell0.set_property( \
			"width", self.GNOME_ICON.ICON_SIZE_POOL["ICON_SIZE_DND"] + 10)
		cell0.set_property("xalign", 0.0)
		cell1.set_property("xalign", 0.0)

		__combobox_startup.pack_start(cell0, False)
		__combobox_startup.pack_start(cell1, True)
		__combobox_startup.add_attribute(cell0, "pixbuf", 0)
		__combobox_startup.add_attribute(cell1, "text", 1)

		self.ITER_LIST_INITIAL_MODULE = dict()
		self.ITER_LIST_INITIAL_MODULE["None"] = \
			__liststore.append( \
				[self.GNOME_ICON.getPixbufByIcon("gtk-stop"), _("None"), "None", ] \
			)
		self.ITER_LIST_INITIAL_MODULE["from_bookmark"] = \
			__liststore.append( \
				[self.GNOME_ICON.getPixbufByIcon("gtk-home"), \
					_("From bookmark item : "), "from_bookmark", ] \
			)

		########## insert child button
		for __child_name, __config in self.STATE_GLOBAL["children"].items() :
			__pixbuf = \
				self.GNOME_ICON.getPixbufByChildConfig(__child_name, __config)
			__label = __config["layout"]["label"]["value"]
			self.ITER_LIST_INITIAL_MODULE[__child_name] = \
				__liststore.append([__pixbuf, _(__label), __child_name, ])

		__initial_module = self.GCONF_CLIENT.getValue("initial_module")
		if __initial_module and not __liststore.iter_is_valid( \
				self.ITER_LIST_INITIAL_MODULE[__initial_module]) :
			__combobox_startup.set_active_iter( \
				self.ITER_LIST_INITIAL_MODULE["none"])
		else :
			__combobox_startup.set_active_iter( \
				self.ITER_LIST_INITIAL_MODULE[__initial_module])

		# if initial module is 'from bookmark', set the button for bookmark item
		# to be visible.
		__button_from_bookmark = self.getWidget( \
				"initial_module_button_from_bookmark")
		if __initial_module == "from_bookmark" :
			__button_from_bookmark.set_property("visible", True)

		# set button_from_bookmark
		self.__setBoomarkItemInButton( \
			self.GCONF_CLIENT.getValue("initial_module_from_bookmark"))

		__combobox_startup.show()
		self.getWidget("box_combobox_for_initial_module").pack_start( \
			__combobox_startup, False)

		# enable dnd in button_from_bookmark.
		__button_from_bookmark.drag_dest_set( \
			gtk.DEST_DEFAULT_MOTION |
			gtk.DEST_DEFAULT_HIGHLIGHT |
			gtk.DEST_DEFAULT_DROP,
			[ \
				( "text/uri-list",  0, Conf.DND_TARGET_URI_LIST ),
			], \
			gtk.gdk.ACTION_COPY \
		)
		__button_from_bookmark.connect("drag_data_received", \
			self.on_drag_data_received_data_button_from_bookmark)

		# keyboard shortcut #############################################
		__panel_keyboardshortcut = \
			Panel.KeyboardShortcutForPreferences(self.xml, state=self.STATE)
		Panel.Preference(self.xml, self, state=self.STATE)

		# resize the window.
		__width = self.getWidget("treeview-preference").allocation.width
		__heigth = self.getWidget("treeview-preference").allocation.height
		self.getWidget("scrolledwindow5").set_size_request(__width, __heigth + 100)

	##################################################
	# EVENT HANDLER
	def on_drag_data_received_data_button_from_bookmark \
			(self, treeview, context, x, y, selection, info, etime) :
		for i in __OS__.get_list_from_uri_list(selection.data) :
			(__scheme, __uid, ) = __OS__.get_pathinfo_from_uri(i)

			if __scheme == "bookmark" :
				self.__setBoomarkItemInButton(__uid)
				self.GCONF_CLIENT.setValue("initial_module_from_bookmark", __uid)

	def on_changed_use_system_default_font (self, checkbutton) :
		__is_active = checkbutton.get_active()

		self.getWidget("hbox130").set_property("sensitive", not __is_active)
		self.GCONF_CLIENT.setValue("use_system_default_font", __is_active)

	def on_changed_fontname (self, widget) :
		__fontname_vee = widget.get_font_name()
		self.GCONF_CLIENT.setValue("font", __fontname_vee)

	def on_changed_initial_module (self, combobox) :
		__iter = combobox.get_active_iter()
		__module_name = combobox.get_model().get_value(__iter, 2)

		self.GCONF_CLIENT.setValue("initial_module", __module_name)

		# hide from_bookmark box
		__button = self.getWidget("initial_module_button_from_bookmark")
		if __module_name == "from_bookmark" :
			__button.set_property("visible", True)
		else :
			__button.set_property("visible", False)

	def on_initial_module_button_from_bookmark (self, button) :
		__uid = self.GCONF_CLIENT.getValue("initial_module_from_bookmark")
		__config = self.STATE_GLOBAL["bookmark"].getValuesByUID(__uid)

		self.STATE_LOCAL["obj-vee"].addPage(__config["type"], **__config)

	def __setBoomarkItemInButton (self, uid) :
		data = self.STATE_GLOBAL["bookmark"].getValuesByUID(uid)
		__button_from_bookmark = self.getWidget( \
				"initial_module_button_from_bookmark")
		__image = __button_from_bookmark.get_children( \
			)[0].get_children()[0].get_children()[0]
		__label = __button_from_bookmark.get_children( \
			)[0].get_children()[0].get_children()[1]

		if not data :
			__pixbuf = self.GNOME_ICON.getPixbufByIcon(Conf.ICONS["none"])
			__l = _("None")
		else :
			# get icon
			if data["type"] == "vim" :
				__pixbuf = self.GNOME_ICON.getPixbufByFilename(data["filename"])
			else :
				__pixbuf = self.GNOME_ICON.getPixbufByChildConfig( \
						data["type"],
						self.STATE_GLOBAL["children"][data["type"]] \
					)

			__l = data["label"]

		if len(__l) > 20 :
			__l = __l[:20]

		__image.set_from_pixbuf(__pixbuf)
		__label.set_text(__l)


class FileDialog_FileLogging (__VEE__.Core, WIDGET.WIDGET) :

	ENTRY = None

	def __init__ (self, **kwargs) :
		__VEE__.Core.__init__(self, **kwargs)

		if not kwargs.has_key("entry_filename") :
			raise ValueError, _("entry_filename must be included.")
		else :
			self.ENTRY = kwargs["entry_filename"]

		if not kwargs.has_key("filepath") :
			__filepath = os.getenv("HOME")
		else :
			__filepath = kwargs["filepath"]

		WIDGET.WIDGET.__init__(self, "filechooser_file_logging")

	def initialize (self) :
		if not os.path.isdir(__filepath) :
			__filename = os.path.basename(__filepath)
			self.WIDGET_SELF.set_current_name(__filename)

		self.WIDGET_SELF.set_filename(__filepath)

	def on_open (self, widget) :
		__new = self.WIDGET_SELF.get_filename()
		self.ENTRY.set_text(__new)
		self.on_close(None)

class Dialog_SelectFileAndDirectory (__VEE__.Core, WIDGET.WIDGET) :

	CURRENT_MODE = Conf.NOTEBOOK_PAGE_FILECHOOSER_SELECT_FILE_AND_DIRECTORY_FILE
	FILECHOOSER = None

	def __init__ (self, **kwargs) :
		__VEE__.Core.__init__(self, **kwargs)
		WIDGET.WIDGET.__init__(self, "filechooser_select_file_and_directory")
		self.addPage = self.STATE_LOCAL["obj-vee"].addPage

	def initialize (self) :
		self.FILECHOOSER_FILE = self.getWidget("filechooserwidget4")
		self.FILECHOOSER_DIRECTORY = self.getWidget("filechooserwidget3")

		self.getWidget("entry19").grab_focus()

	# EVENT HANDLER ##################################################
	def on_notebook_switch_page (self, notebook, event) :
		__keyname = func.get_keyname(event)
		if __keyname == "<Alt>1" :
			self.getWidget("notebook").set_current_page( \
				Conf.NOTEBOOK_PAGE_FILECHOOSER_SELECT_FILE_AND_DIRECTORY_FILE)
			self.getWidget("entry19").grab_focus()
		elif __keyname == "<Alt>2" :
			self.getWidget("notebook").set_current_page( \
				Conf.NOTEBOOK_PAGE_FILECHOOSER_SELECT_FILE_AND_DIRECTORY_DIRECTORY)
			self.getWidget("entry20").grab_focus()
		else :
			return

	def on_selected_file (self, widget) :
		__filename = widget.get_filename()
		self.on_close(None)
		self.addPage("vim", filename=__filename)

	def on_file_location (self, entry, event) :
		if func.get_keyname(event) != "Return" :
			return

		__path = entry.get_text()
		if self.CURRENT_MODE == \
				Conf.NOTEBOOK_PAGE_FILECHOOSER_SELECT_FILE_AND_DIRECTORY_FILE :

			__current_path = self.FILECHOOSER_FILE.get_current_folder()
			__path_full = os.path.join(__current_path, __path)
			if __OS__.is_file(__path_full) :
				self.addPage("vim", filename=__path_full)
			elif __OS__.is_dir(__path_full) :
				self.FILECHOOSER_FILE.set_current_folder(__path_full)
				entry.set_text(str())
			else :
				return
		elif self.CURRENT_MODE == \
				Conf.NOTEBOOK_PAGE_FILECHOOSER_SELECT_FILE_AND_DIRECTORY_DIRECTORY :
			__current_path = self.FILECHOOSER_DIRECTORY.get_current_folder()
			__path_full = os.path.join(__current_path, __path)
			if not __OS__.is_dir(__path_full) :
				return
			else :
				self.addPage("terminal", path=__path_full)

		return

	on_directory_location = on_file_location

	def on_open (self, widget) :
		if self.CURRENT_MODE == \
				Conf.NOTEBOOK_PAGE_FILECHOOSER_SELECT_FILE_AND_DIRECTORY_FILE :
			__filechooser = self.FILECHOOSER_FILE
		elif self.CURRENT_MODE == \
				Conf.NOTEBOOK_PAGE_FILECHOOSER_SELECT_FILE_AND_DIRECTORY_DIRECTORY :
			__filechooser = self.FILECHOOSER_DIRECTORY

		if self.CURRENT_MODE == \
				Conf.NOTEBOOK_PAGE_FILECHOOSER_SELECT_FILE_AND_DIRECTORY_FILE :
			__filename = __filechooser.get_filename()
			if __filename is None : return

			if __OS__.is_dir(__filename) :
				__filechooser.set_current_folder(__filename)
				return

			self.on_close(None)
			self.addPage("vim", filename=__filename)
		elif self.CURRENT_MODE == \
				Conf.NOTEBOOK_PAGE_FILECHOOSER_SELECT_FILE_AND_DIRECTORY_DIRECTORY :
			__path = __filechooser.get_current_folder()
			if __path is None : return
			self.on_close(None)
			self.addPage("terminal", path=__path)

		return False

	def on_notebook_select_page (self, notebook, page, page_num) :
		self.CURRENT_MODE = page_num

		return

class About (WIDGET.WIDGET) :
	"""
	 "authors"              GValueArray           : Write
	 "comments"             gchararray            : Read / Write
	 "copyright"            gchararray            : Read / Write
	 "documenters"          GValueArray           : Write
	 "logo"                 GdkPixbuf             : Write
	 "name"                 gchararray            : Read / Write
	 "translator-credits"   gchararray            : Read / Write
	 "version"              gchararray            : Read / Write
	"""

	def __init__ (self) :
		WIDGET.WIDGET.__init__(self, "about")

		for i, j in Conf__.INFO.items() :
			self.WIDGET_SELF.set_property(i, _(j))

	def on_ok (self, widget) :
		self.on_close(widget)

class DashBoard (__VEE__.Core, WIDGET.WIDGET) :
	OBJ_VEE = None
	HANDLER_ID_UPDATE_TIME = None

	def __init__ (self, **kwargs) :
		__VEE__.Core.__init__(self, **kwargs)
		self.OBJ_VEE = self.STATE_LOCAL["obj-vee"]

		WIDGET.WIDGET.__init__(self, "dashboard")

	def initialize (self) :
		########## initialize children
		for __child_name in self.STATE_GLOBAL["children"].keys() :
			try :
				__class = child.get_child_dashboard(__child_name)
			except ValueError :
				print "DASHBOARD : %s could not be initialized." % __child_name
				continue
			else :
				if not __class : continue
				__class(self, state=self.STATE)

		########## insert child button
		GNOME_ICON = GNOME.Icon(Conf.ICON_SIZE_DASHBOARD_CHILD_TYPE)
		__widget_box_start_new = self.getWidget("box_start_new")
		for __child_name, __config in self.STATE_GLOBAL["children"].items() :
			for __button in __config["layout"]["dashboard"]["buttons"] :
				__b = Widgets.ButtonLabelAndImage()
				__b.set_relief(gtk.RELIEF_NONE)
				__b.LABEL.set_property("use-markup", True)
				__b.set_markup( \
					"<span size=\"x-large\">%s</span>" % _(__button["label"]))

				__b.set_from_pixbuf( \
					GNOME_ICON.getPixbufByChildConfig(__child_name, __config) \
				)

				# connect handler
				__b.WIDGET.connect("clicked", self.on_insert_child, __child_name)

				# add widget in this box.
				__widget_box_start_new.pack_start(__b.WIDGET, False)

		########## clock
		self.getWidget("label_current_time").set_markup( \
			time.strftime("%c", time.localtime()))
		self.HANDLER_ID_UPDATE_TIME = gobject.timeout_add(1000, self.updateTime)

	def updateTime (self) :
		__label = self.getWidget("label_current_time")
		if __label is None : return False

		__label.set_markup(time.strftime("%c", time.localtime()))

		return True

	# EVENT HANDLER ##################################################
	def on_toggle_bookmark (self, button) :
		if self.STATE_LOCAL["window-bookmark"] :
			self.STATE_LOCAL["window-bookmark"].show()
		else :
			Bookmark(state=self.STATE)

		self.on_close(None)

	def on_toggle_fullscreen (self, button) :
		self.OBJ_VEE.toggleFullScreen()

	def on_insert_child (self, widget, childname, *parm0) :
		self.on_close(None)
		self.OBJ_VEE.on_insert_child(None, childname, *parm0)

	def on_preferences (self, widget) :
		self.on_close(None)
		self.OBJ_VEE.on_preferences(None)

	def on_help (self, widget) :
		self.OBJ_VEE.on_help(None)

	def on_close (self, widget) :
		gobject.source_remove(self.HANDLER_ID_UPDATE_TIME)

		WIDGET.WIDGET.on_close(self, widget)
		self.OBJ_VEE.WIDGET_SELF.set_property("sensitive", True)
		self.OBJ_VEE.setFocus()

class Bookmark (__VEE__.Core, WIDGET.WIDGET) :

	BOOKMARK = None
	OBJ_VEE = None
	TAB_LIST = None
	PANEL_BOOKMARK = None

	IS_ATTACHED_WITH_MAIN_WINDOW = False

	def __init__ (self, **kwargs) :
		__VEE__.Core.__init__(self, **kwargs)
		self.OBJ_VEE = self.STATE_LOCAL["obj-vee"]
		self.BOOKMARK = self.STATE_GLOBAL["bookmark"]
		self.TAB_LIST = self.STATE_LOCAL.get_value("tab-list")

		WIDGET.WIDGET.__init__(self, "bookmark")
		self.STATE_LOCAL.set_value("window-bookmark", self)

	def initialize (self) :
		# set cursor type.
		self.WIDGET_SELF.window.set_cursor(gtk.gdk.Cursor(gtk.gdk.HAND2))

		########### "BOOKMARK" PANEL.
		__callback_functions = { \
			"addPage" : self.OBJ_VEE.addPage, \
			"on_bookmark_property" : self.on_bookmark_property, \
			"on_bookmark_add_current" : self.on_bookmark_add_current, \
		}

		self.PANEL_BOOKMARK = Panel.Bookmark( \
			self.xml, \
			state=self.STATE, \
			callback_functions=__callback_functions \
		)

		########## GEOMETRY
		# set window geometry
		self.WIDGET_SELF.set_data("window-position", \
			self.WIDGET_SELF.get_position())
		self.WIDGET_SELF.set_data("window-size", self.WIDGET_SELF.get_size())

		(__x_main, __y_main) = self.OBJ_VEE.WIDGET_SELF.get_position()
		(__width_main, __height_main) = self.OBJ_VEE.WIDGET_SELF.get_size()

		# restore the latest window geomtry.
		try :
			(w, h, x, y, ) = [ int(i) for i in self.GCONF_CLIENT.getValue( \
				"latest_bookmark_window_geometry").split(",")]
		except ValueError :
			(w, h, x, y, ) = \
				(200, __height_main, __x_main + __width_main, __y_main, )

		WIDGET.set_window_geometry(self.WIDGET_SELF, (w, h, x, y, ))

		########### set default focus widget.
		self.WIDGET_SELF.connect("event", self.on_event)
		self.WIDGET_SELF.connect("delete-event", self.hide)

		########### set default focus widget.
		self.getWidget("treeview-bookmark").grab_focus()

		# add keyboard shortcut.
		self.OBJ_VEE.ACCEL_GROUP_GLOBAL.addToWindow(self.WIDGET_SELF)

	# EVENT HANDLER ##################################################
	def on_event (self, window, event) :
		if event.type == gtk.gdk.CONFIGURE :
			(__x, __y, ) = self.WIDGET_SELF.get_position()
			(__x0, __y0, ) = self.WIDGET_SELF.get_data("window-position")
			(__width, __height, ) = self.WIDGET_SELF.get_size()
			(__width0, __height0, ) = self.WIDGET_SELF.get_data("window-size")

			if self.checkIsAttachedWithMainWindow() :
				(__x_main, __y_main, ) = self.OBJ_VEE.WIDGET_SELF.get_position()
				(__width_main, __height_main, ) = \
					self.OBJ_VEE.WIDGET_SELF.get_data("window-size")

				self.WIDGET_SELF.move(__x_main + __width_main, __y_main)
				(__x, __y, ) = (__x_main + __width_main, __y_main)

			if (__x, __y, ) != (__x0, __y0, ) : # it's move.
				self.WIDGET_SELF.set_data("window-position", (__x, __y, ))
			elif (__width, __height, ) != (__width0, __height0, ) : # it's resizing.
				self.WIDGET_SELF.set_data("window-size", self.WIDGET_SELF.get_size())

		return False

	def on_close (self, window) :
		self.PANEL_BOOKMARK.save_tree_to_xml()

		WIDGET.remember_geometry( \
			self.WIDGET_SELF, self.GCONF_CLIENT, "latest_bookmark_window_geometry")

		WIDGET.WIDGET.on_close(self, window)
		self.OBJ_VEE.setFocus()
		self.WIDGET_SELF.hide()

	def on_bookmark_add_current (self, widget) :
		if len(self.TAB_LIST) < 1 : return

		__current_page = self.OBJ_VEE.NOTEBOOK.get_current_page()
		__tab = self.TAB_LIST[__current_page]

		# get type, label, label
		__egg = __tab.getObjectChild()
		__type = __tab.getName()
		__label = __egg.getLabelName()

		if __label is None or (__label and not __label.strip()) :
			__label = _("New Item")

		__data = __egg.getProperties()
		__data.update({"label" : __label, })
		__data.update({"uid" : func.get_uid(), })

		# INSERT DATA. ############################################
		# get parent uid, and find out it's node.
		__treestore, __treeiter = self.PANEL_BOOKMARK.get_selected()
		if not __treeiter :
			__iter_parent = None
		else :
			__iter_parent = __treestore.iter_parent(__treeiter)
			if not __iter_parent :
				__path_parent = tuple()
			else :
				__path_parent = self.PANEL_BOOKMARK.get_path(__iter_parent)

		__uid_parent = self.PANEL_BOOKMARK.getUID(__iter_parent)
		__node_parent = self.BOOKMARK.getNodeByUID(__uid_parent)

		# ADD DATA IN XML #########################################
		__new_node = self.BOOKMARK.addNew(__type, __node_parent)
		self.BOOKMARK.updateValues(__new_node, __data)

		# ADD DATA IN PANEL ########################################3
		# if current item is not category, set the parent category as it's parent.
		if __iter_parent :
			self.PANEL_BOOKMARK.WIDGET_TREEVIEW.expand_row( \
				__path_parent, False)

		__iter = self.PANEL_BOOKMARK.add_row(__new_node, __iter_parent)
		self.PANEL_BOOKMARK.set_editable( \
			__iter, 0, Conf.TREEVIEW_BOOKMARK_PANEL_COLUMN_NAME)

	def on_bookmark_add_category (self, widget) :
		# INSERT DATA. ############################################
		# get parent uid, and find out it's node.
		__treestore, __treeiter = self.PANEL_BOOKMARK.get_selected()
		if not __treeiter :
			__iter_parent = None
		else :
			__iter_parent = __treestore.iter_parent(__treeiter)
			if not __iter_parent :
				__path_parent = tuple()
			else :
				__path_parent = self.PANEL_BOOKMARK.get_path(__iter_parent)

		__uid_parent = self.PANEL_BOOKMARK.getUID(__iter_parent)
		__node_parent = self.BOOKMARK.getNodeByUID(__uid_parent)

		# ADD DATA IN XML #########################################
		__new_node = self.BOOKMARK.addNew("category", __node_parent)
		self.BOOKMARK.updateValues(__new_node, {"label" : _("New Category")})

		# ADD DATA IN PANEL ########################################3
		# if current item is not category, set the parent category as it's parent.
		if __iter_parent :
			self.PANEL_BOOKMARK.WIDGET_TREEVIEW.expand_row( \
				__path_parent, False)

		__iter = self.PANEL_BOOKMARK.add_row(__new_node, __iter_parent)
		self.PANEL_BOOKMARK.set_editable( \
			__iter, 0, Conf.TREEVIEW_BOOKMARK_PANEL_COLUMN_NAME)

	def on_bookmark_property(self, widget) :
		__treestore, __treeiter = self.PANEL_BOOKMARK.get_selected()
		if not __treeiter : return

		__uid = self.PANEL_BOOKMARK.getUID(__treeiter)
		if not self.BOOKMARK.getValuesByUID(__uid) : return

		# run the property dialog by it's TYPE.
		__data = self.BOOKMARK.getValuesByUID(__uid)

		PROPERTY(__data["type"], __uid, panel=self.PANEL_BOOKMARK, state=self.STATE)
		return

	def on_bookmark_remove(self, widget) :
		__treestore, __treeiter = self.PANEL_BOOKMARK.get_selected()
		if not __treeiter : return

		__uid = self.PANEL_BOOKMARK.getUID(__treeiter)

		# remove from xml
		__this_node = self.BOOKMARK.getNodeByUID(__uid)
		__parent_node = self.BOOKMARK.getParent(__this_node)

		if not __parent_node : return
		__parent_node.removeChild(__this_node)

		# remove from panel.
		self.PANEL_BOOKMARK.remove(treeiter=__treeiter)

	# MISC HANDLER ##################################################
	def checkIsAttachedWithMainWindow (self) :
		# compare the window position with main window
		(__x, __y, ) = self.WIDGET_SELF.get_position()
		(__x_main, __y_main, ) = self.OBJ_VEE.WIDGET_SELF.get_position()
		(__width_main, __height_main, ) = \
			self.OBJ_VEE.WIDGET_SELF.get_size()

		__diff_y = __y - (__y_main)
		if __diff_y < 0 :
			__diff_y = -1 * __diff_y

		__diff_x = __x - (__x_main + __width_main)
		if __diff_x < 0 :
			__diff_x = -1 * __diff_x

		if __diff_x < 30 and __diff_y < 30 :
			return True

		return False

	def isShown (self) :
		return self.WIDGET_SELF.get_property("visible")

	def hide (self, window=None, event=None) :
		self.WIDGET_SELF.hide()

		return True

	def show (self, window=None) :
		self.WIDGET_SELF.show()

class TabList (__VEE__.Core, WIDGET.WIDGET) :

	PANEL = None

	def __init__ (self, **kwargs) :
		__VEE__.Core.__init__(self, **kwargs)

		WIDGET.WIDGET.__init__(self, "tab_list")

	def initialize (self) :
		self.PANEL = Panel.TabList(self.xml, self.WIDGET_SELF, state=self.STATE)
		self.getWidget("treeview-tab-list").grab_focus()

		gobject.timeout_add(100, self.resizeWindow)

		# add window title event.
		self.getWidget("eventbox2").connect("event", self.on_button_event)

	IS_PRESSED = False
	X_DIFF = 0
	Y_DIFF = 0
	def on_button_event (self, widget, event) :
		if event.type not in [gtk.gdk.MOTION_NOTIFY, \
				gtk.gdk.BUTTON_PRESS, gtk.gdk.BUTTON_RELEASE, ] :
			return False

		if event.type is gtk.gdk.MOTION_NOTIFY :
			if not self.IS_PRESSED :
				return False
			else :
				__x = int(event.x_root - self.X_DIFF)
				__y = int(event.y_root - self.Y_DIFF)
				self.WIDGET_SELF.move(__x, __y)
		elif event.type is gtk.gdk.BUTTON_PRESS :
			self.IS_PRESSED = True
			__x, __y =self.WIDGET_SELF.get_position()
			self.X_DIFF = event.x_root - __x
			self.Y_DIFF = event.y_root - __y

		elif event.type is gtk.gdk.BUTTON_RELEASE :
			self.IS_PRESSED = False

	def resizeWindow (self) :
		__width_current = self.WIDGET_SELF.get_size()
		__width_screen = self.WIDGET_SELF.get_screen().get_width()
		__height_screen = self.WIDGET_SELF.get_screen().get_height()

		__height = __height_screen - 700
		__width_max = __width_screen - 200
		if __width_current > (__width_max) :
			__width_current = __width_max

		# change policy of scrolledwindow.
		self.getWidget("scrolledwindow22").set_policy( \
			gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)

		self.WIDGET_SELF.resize(__width_current, __height)

		return False

"""

Description
-----------


ChangeLog
---------


Usage
-----


"""

__author__ =  "Spike^ekipS <spikeekips@gmail.com>"
__version__=  "0.1"
__nonsense__ = ""

__file__ = "Window.py"


