# -*- coding: utf-8 -*-
#    Copyright 2005 Spike^ekipS <spikeekips@gmail.com>
#
#       This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import xml
import xml.parsers.expat as Expat
from xml.dom import minidom
from vee import func, __XML__, __OS__

TEMPLATE_DEFAULT = { \
	"uid" : {"default" : None, "type" :"str",}, \
	"timestamp_added" : {"default" : None, "type" :"float",}, \
	"label" : {"default" : None, "type" :"unicode",}, \
	"type" : {"default" : None, "type" :"str",}, \
}

class Bookmark (__XML__.__XML_PARSER__) :

	CONFIG_CHILDREN = dict()

	def __init__ (self, xml_filename=None, **kwargs) :
		if not xml_filename :
			self.DOM = minidom.parseString("<?xml version=\"1.0\"?><them></them>")
		else :
			try :
				__XML__.__XML_PARSER__.__init__(self, xml_filename)
			except Expat.ExpatError :
				self.DOM = minidom.parseString("<?xml version=\"1.0\"?><them></them>")

		if not kwargs.has_key("children") :
			raise ValueError, "children configuration must be given."
		else :
			self.CONFIG_CHILDREN = kwargs["children"]

	def __makeXPath (self, path, uid=None) :
		"""
		Read gtk treestore style 'path' and make xpath.

		@param :
			path : tuple(), ex) (0, 1,)

		@return :
			xpath : str()
		"""
		if type(path) is not tuple : return None

		if not uid :
			__xpath = "/them/%s" % "/".join(["one", ] * len(path))
		else :
			__xpath = "/them/%s[@uid = '%s']" % ("/".join(["one", ] * len(path)), uid)

		return __xpath

	def __getSkel (self, **kwargs) :
		"""
		kwargs is, type, node. One of the 'type', 'node' must be given.
		"""
		if not kwargs.has_key("node") and not kwargs.has_key("type") :
			return dict()

		if not kwargs.has_key("type") :
			type = self.getType(kwargs["node"])
		else :
			type = kwargs["type"]

		if not self.CONFIG_CHILDREN.has_key(type) :
			__skel = dict()
		else :
			__skel = self.CONFIG_CHILDREN[type]["setting"]

		__skel.update(TEMPLATE_DEFAULT)

		return __skel

	def getType (self, node) :
		__a = __XML__.get_attributes(node.attributes)

		if not __a.has_key("type") : return None
		if not self.CONFIG_CHILDREN.has_key(__a["type"]) : return None

		return __a["type"]

	def getOnes (self, path, uid=None) :

		xpath = self.__makeXPath(path, uid)
		if not xpath : return list()
		nodes = __XML__.evaluate_path(self.DOM, xpath)

		return nodes

	def getNodeByUID (self, uid) :
		nodes = __XML__.evaluate_path(self.DOM, "//one[ @uid = '%s' ]" % uid)
		if not nodes : return None
		return nodes[0]

	def getValuesByUID (self, uid) :
		return self.getValues(self.getNodeByUID(uid))

	def getValues (self, node) :
		if not hasattr(node, "attributes") : return None

		__skel = self.__getSkel(node=node)
		return __XML__.get_attributes_extended(node.attributes, __skel)

	def getParent (self, node) :
		return node.parentNode

	def getChild (self, node_parent=None) :
		"""
		If node_parent is None, start top root.
		"""
		if not node_parent :
			__child = list()
			for i in self.getOnes((0,)) :
				if i.nodeType in [i.TEXT_NODE, i.COMMENT_NODE, ] : continue
				__child.append(i)
		else :
			__child = list()
			for i in node_parent.childNodes :
				if i.nodeType in [i.TEXT_NODE, i.COMMENT_NODE, ] : continue
				__child.append(i)

		return __child

	def createNewOne (self, type) :
		new = self.DOM.createElement("one")

		__skel = self.__getSkel(type=type)
		for i in __skel :
			new.setAttribute(i, str())

		new.setAttribute("uid", func.get_uid())
		new.setAttribute("type", type)

		return new

	def getValue (self, node, key) :
		if not hasattr(node, "attributes") : return None
		if not node.attributes.has_key(key) : return None

		__skel = self.__getSkel(node=node)
		if not __skel.has_key(key) :
			__type = None
		else :
			__type = __skel[key]

		return __XML__.casting(__type, node.attributes.get(key).nodetype)

	def setValue (self, node, key, value) :
		if not hasattr(node, "attributes") : return False

		__skel = self.__getSkel(node=node)
		#if not __skel.has_key(key) : return False

		node.setAttribute(key, str(value))

		return True

	def updateValues (self, node, values=dict()) :
		if not hasattr(node, "attributes") : return False

		__skel = self.__getSkel(node=node)
		for __key, __value in values.items() :
			if not __skel.has_key(__key) : continue
			node.setAttribute(__key, str(__value))

		return True

	def addNew (self, type, node_parent=None) :
		#if not self.CONFIG_CHILDREN.has_key(type) : return False
		if type not in self.CONFIG_CHILDREN.keys() + ["category", ] :
			return False

		# if node_parent is None, parent node is "/them"
		if not node_parent :
			__nodes = __XML__.evaluate_path(self.DOM, "/them")
			if not __nodes : return False
			if len(__nodes) < 1 : return False

			node_parent = __nodes[0]

		# create new node.
		__node_new = self.createNewOne(type)

		# add node to parent node
		return node_parent.appendChild(__node_new)


VALUE_TYPE_COMMAND = { \
	"value" : "str", \
	"uid" : "str", \
}

class Command (__XML__.__XML_PARSER__) :

	def __init__ (self, xml_filename=None) :
		if xml_filename :
			try :
				__XML__.__XML_PARSER__.__init__(self, xml_filename)
			except Expat.ExpatError :
				self.DOM = minidom.parseString("<?xml version=\"1.0\"?><commands></commands>")
		else :
			self.DOM = minidom.parseString("<?xml version=\"1.0\"?><commands></commands>")

	def __makeXPath (self, path, uid=None) :
		"""
		Read gtk treestore style 'path' and make xpath.

		@param :
			path : tuple(), ex) (0, 1,)

		@return :
			xpath : str()
		"""
		if type(path) is not tuple : return None

		if not uid :
			__xpath = "/commands/%s" % "/".join(["command", ] * len(path))
		else :
			__xpath = "/commands/%s[@uid = '%s']" % ("/".join(["command", ] * len(path)), uid)

		return __xpath

	def getByValue (self, value) :
		__xpath = "/commands/command[@value = '%s']" % value
		nodes = __XML__.evaluate_path(self.DOM, __xpath)
		return nodes

	def getOnes (self, path, uid=None) :

		xpath = self.__makeXPath(path, uid)
		if not xpath : return list()
		nodes = __XML__.evaluate_path(self.DOM, xpath)

		return nodes

	def getNodeByUID (self, uid) :
		nodes = __XML__.evaluate_path(self.DOM, "//command[ @uid = '%s' ]" % uid)
		if not nodes : return None

		return nodes[0]

	def getValuesByUID (self, uid) :
		return self.getValues(self.getNodeByUID(uid))

	def getValues (self, node) :
		if not hasattr(node, "attributes") : return None

		return __XML__.get_attributes(node.attributes, VALUE_TYPE_COMMAND)

	def getParent (self, node) :
		return node.parentNode

	def getChild (self, node_parent=None) :
		"""
		If node_parent is None, start top root.
		"""
		if not node_parent :
			__child = list()
			for i in self.getOnes((0,)) :
				if i.nodeType in [i.TEXT_NODE, i.COMMENT_NODE, ] : continue
				__child.append(i)
		else :
			__child = list()
			for i in node_parent.childNodes :
				if i.nodeType in [i.TEXT_NODE, i.COMMENT_NODE, ] : continue
				__child.append(i)

		return __child

	def createNewOne (self) :
		new = self.DOM.createElement("command")
		new.setAttribute("uid", func.get_uid())

		return new

	def getValue (self, node, key) :
		if not hasattr(node, "attributes") : return None
		if not node.attributes.has_key(key) : return None

		return __XML__.casting( \
			VALUE_TYPE_COMMAND[key], node.attributes.get(key).nodeValue)

	def setValue (self, node, key, value) :
		if not hasattr(node, "attributes") : return False
		if not VALUE_TYPE_COMMAND.has_key(key) : return False

		node.setAttribute(key, str(value))

		return True

	def updateValues (self, node, values=dict()) :
		if not hasattr(node, "attributes") : return False

		for __key, __value in values.items() :
			if not VALUE_TYPE_COMMAND.has_key(__key) : continue
			node.setAttribute(__key, str(__value))

		return True

	def addNew (self) :
		__nodes = __XML__.evaluate_path(self.DOM, "/commands")
		if not __nodes : return False
		if len(__nodes) < 1 : return False

		node_parent = __nodes[0]

		# create new node.
		__node_new = self.createNewOne()

		# add node to parent node
		return node_parent.appendChild(__node_new)

VALUE_TYPE_SCROLLBAR = { \
	"vertical" : "bool", \
	"horizontal" : "bool", \
}

class __CONFIG__ (__XML__.__XML_PARSER__) :

	def __init__ (self, xml_filename) :
		try :
			__XML__.__XML_PARSER__.__init__(self, xml_filename)
		except Expat.ExpatError :
			raise Expat.ExpatError, \
				"Configuration file, %s was broken." % xml_filename

		# import SKEL.
		self.__getImport()

	def __getImport (self) :
		__r = self.getValue(__OS__.normalize_path("/config/import"))
		if len(__r) > 0 :
			__r = __r[0]
			__value = __XML__.get_attributes(__r.attributes)
			if not __value["name"].strip() :
				return

	def getValue (self, xpath, uid=None) :
		nodes = __XML__.evaluate_path(self.DOM, xpath)

		return nodes

	def getSetting (self, prefix="") :
		__r = self.getValue( \
			__OS__.normalize_path("/config/%s/setting" % prefix))[0]

		return get_setting(__r)

	def getWidget (self, prefix="") :

		# get map
		__r = self.getValue( \
			__OS__.normalize_path("/config/%s/widget/map" % prefix))
		__map = get_map(__r)

		# get handler
		__r = self.getValue( \
			__OS__.normalize_path("/config/%s/widget/handler" % prefix))
		__handler = get_handler(__r)

		return {"map" : __map, "handler": __handler, }

	def getGConf (self, prefix="") :
		# get notification
		__r = self.getValue( \
			__OS__.normalize_path("/config/%s/gconf/notifications" % prefix))

		__notification = get_notification(__r)

		# get keyboard shortcut
		__r = self.getValue( \
			__OS__.normalize_path("/config/%s/gconf/keyboard_shortcut" % prefix))

		__keyboard_shortcut = get_keyboard_shortcut(__r)
		return { \
			"notification" : __notification, \
			"keyboard_shortcut" : __keyboard_shortcut, \
		}

	def getLayout (self, prefix="") :
		__closebutton = dict()
		__scrollbar = dict()
		__buttons_toolbar = list()
		__icons = dict()
		__buttons_dashboard = list()
		__preferences = dict()

		__r = self.getValue( \
			__OS__.normalize_path("/config/%s/layout/tab/closebutton" % prefix))
		__closebutton = get_closebutton(__r)

		__r = self.getValue( \
			__OS__.normalize_path("/config/%s/layout/tab/scrollbar" % prefix))
		__scrollbar = get_scrollbar(__r)

		__r = self.getValue( \
			__OS__.normalize_path("/config/%s/layout/preferences" % prefix))
		__preferences = get_preferences(__r)

		__r = self.getValue( \
			__OS__.normalize_path("/config/%s/layout/dashboard" % prefix))
		__buttons_dashboard = get_buttons_dashboard(__r)

		__r = self.getValue( \
			__OS__.normalize_path("/config/%s/layout/label" % prefix))
		__label = get_label(__r)

		__r = self.getValue( \
			__OS__.normalize_path("/config/%s/layout/icon" % prefix))
		__icons = get_icons(__r)

		__r = self.getValue( \
			__OS__.normalize_path("/config/%s/layout/toolbar" % prefix))
		__buttons_toolbar = get_buttons_toolbar(__r)

		return \
			{ \
				"icon" : __icons, \
				"label" : __label, \
				"tab" : \
					{ \
						"closebutton" : __closebutton, \
						"scrollbar" : __scrollbar, \
					}, \
				"preferences" : __preferences, \
				"dashboard" : \
					{ \
						"buttons" : __buttons_dashboard, \
					}, \
				"toolbar" : \
					{ \
						"buttons" : __buttons_toolbar, \
					}, \
			}

	def getEvents (self, prefix="") :
		__events = list()
		__r = self.getValue( \
			__OS__.normalize_path("/config/%s/events" % prefix))
		if len(__r) > 0 :
			for i in __r :
				if i.nodeType in [i.TEXT_NODE, i.COMMENT_NODE, ] : continue

				for j in i.childNodes :
					if j.nodeType in [j.TEXT_NODE, j.COMMENT_NODE, ] : continue
					__events.append(__XML__.get_attributes(j.attributes))

		return __events


class ChildConfig (__CONFIG__) :
	"""
	Read config.xml of child module.
	"""

	def getAll (self) :
		return { \
			"setting" : self.getSetting(), \
			"layout" : self.getLayout(), \
			"gconf" : self.getGConf(), \
			"widget" : self.getWidget(), \
		}

class GlobalConfig (__CONFIG__) :

	def getSetting (self, id="") :
		__r = self.getValue( \
			__OS__.normalize_path("/config/modules/module[@id = '%s']/setting" % id))[0]

		return get_setting(__r)

	def getWidget (self, id="") :

		# get map
		__r = self.getValue( \
			__OS__.normalize_path("/config/modules/module[@id = '%s']/widget/map" % id))
		__map = get_map(__r)

		# get handler
		__r = self.getValue( \
			__OS__.normalize_path("/config/modules/module[@id = '%s']/widget/handler" % id))
		__handler = get_handler(__r)

		return {"map" : __map, "handler": __handler, }

	def getGConf (self, id="") :
		return dict()

	def getLayout (self, id="") :
		__closebutton = dict()
		__scrollbar = dict()
		__buttons_toolbar = list()
		__icons = dict()
		__buttons_dashboard = list()
		__preferences = dict()

		__r = self.getValue( \
			__OS__.normalize_path("/config/modules/module[@id = '%s']/layout/tab/closebutton" % id))
		__closebutton = get_closebutton(__r)

		__r = self.getValue( \
			__OS__.normalize_path("/config/modules/module[@id = '%s']/layout/tab/scrollbar" % id))
		__scrollbar = get_scrollbar(__r)

		__r = self.getValue( \
			__OS__.normalize_path("/config/modules/module[@id = '%s']/layout/preferences" % id))
		__preferences = get_preferences(__r)

		__r = self.getValue( \
			__OS__.normalize_path("/config/modules/module[@id = '%s']/layout/dashboard" % id))
		__buttons_dashboard = get_buttons_dashboard(__r)

		__r = self.getValue( \
			__OS__.normalize_path("/config/modules/module[@id = '%s']/layout/icon" % id))
		__icons = get_icons(__r)

		__r = self.getValue( \
			__OS__.normalize_path("/config/modules/module[@id = '%s']/layout/toolbar" % id))
		__buttons_toolbar = get_buttons_toolbar(__r)

		return \
			{ \
				"icon" : __icons, \
				"tab" : \
					{ \
						"closebutton" : __closebutton, \
						"scrollbar" : __scrollbar, \
					}, \
				"preferences" : __preferences, \
				"dashboard" : \
					{ \
						"buttons" : __buttons_dashboard, \
					}, \
				"toolbar" : \
					{ \
						"buttons" : __buttons_toolbar, \
					}, \
			}

	def getAll (self) :
		__all = dict()

		__all["modules"] = dict()
		__r = self.getValue(__OS__.normalize_path("/config/modules"))
		for i in __r :
			# get the list of modules.
			for j in i.childNodes :
				__data = __XML__.get_attributes(j.attributes)
				if not __data : continue

				__all["modules"][__data["id"]] = { \
					"setting"	: self.getSetting(__data["id"]), \
					"layout"		: self.getLayout(__data["id"]), \
					"gconf"		: self.getGConf(__data["id"]), \
					"widget"		: self.getWidget(__data["id"]), \
				}

		return __all

###########################################################
# functions for CONFIG
def get_setting (nodes) :
	__setting = dict()
	for i in nodes.childNodes :
		if i.nodeType in [i.TEXT_NODE, i.COMMENT_NODE, ] : continue

		__value = i.childNodes[0].nodeValue
		__type = __XML__.get_attributes(i.attributes)
		__setting[str(__value).strip()] = __type

	return __setting

def get_map (nodes) :
	__map = dict()
	for i in nodes :
		if i.nodeType in [i.TEXT_NODE, i.COMMENT_NODE, ] : continue

		for j in i.childNodes :
			if j.nodeType in [j.TEXT_NODE, j.COMMENT_NODE, ] : continue
			__value = __XML__.get_attributes(j.attributes)
			__map[__value["name"]] = __value

	return __map

def get_handler (nodes) :
	__handler = dict()
	for i in nodes :
		if i.nodeType in [i.TEXT_NODE, i.COMMENT_NODE, ] : continue

		for j in i.childNodes :
			if j.nodeType in [j.TEXT_NODE, j.COMMENT_NODE, ] : continue
			__value = __XML__.get_attributes(j.attributes)

			__handler[__value["name"]] = list()
			for k in j.childNodes :
				if k.nodeType in [k.TEXT_NODE, k.COMMENT_NODE, ] : continue
				__handler[__value["name"]].append(__XML__.get_attributes(k.attributes))

	return __handler

def get_closebutton (nodes) :
	if len(nodes) < 1 : return dict()

	nodes = nodes[0]
	return __XML__.get_attributes( \
		nodes.attributes)

def get_scrollbar (nodes) :
	if len(nodes) < 1 : return dict()

	nodes = nodes[0]
	return __XML__.get_attributes(nodes.attributes, VALUE_TYPE_SCROLLBAR)

def get_preferences (nodes) :
	if len(nodes) < 1 : return dict()

	nodes = nodes[0]

	__child = __XML__.get_attributes(nodes.attributes)

	__preferences = dict()
	__values = dict()
	for j in nodes.childNodes :
		if j.nodeType in [j.TEXT_NODE, j.COMMENT_NODE, ] : continue
		__value = __XML__.get_attributes(j.attributes)

		if not __value["parent"].strip() :
			__parent = "None"
		else :
			__parent = __value["parent"]

		if not __values.has_key(__parent) :
			__values[__parent] = list()

		__values[__parent].append(__value)

	__preferences["label"] = __child["label"]
	__preferences["pages"] = __values

	return __preferences

def get_buttons_dashboard (nodes) :
	__buttons_dashboard = list()

	for i in nodes :
		if i.nodeType in [i.TEXT_NODE, i.COMMENT_NODE, ] : continue
		for j in i.childNodes :
			if j.nodeType in [j.TEXT_NODE, j.COMMENT_NODE, ] : continue
			__buttons_dashboard.append(__XML__.get_attributes(j.attributes))

	return __buttons_dashboard

def get_label (node) :
	node = node[0]

	__l = dict()
	if node.nodeType not in [node.TEXT_NODE, node.COMMENT_NODE, ] :
		__l = __XML__.get_attributes(node.attributes)

	return __l

def get_icons (node) :
	node = node[0]

	__icons = dict()
	if node.nodeType not in [node.TEXT_NODE, node.COMMENT_NODE, ] :
		__icons = __XML__.get_attributes(node.attributes)

	return __icons

def get_keyboard_shortcut (nodes) :
	__value = list()
	for i in nodes :
		if i.nodeType in [i.TEXT_NODE, i.COMMENT_NODE, ] : continue
		__value.append(__XML__.get_attributes(i.attributes))

	return __value

def get_notification (nodes) :
	__notification = list()
	for i in nodes :
		if i.nodeType in [i.TEXT_NODE, i.COMMENT_NODE, ] : continue

		for j in i.childNodes :
			if j.nodeType in [j.TEXT_NODE, j.COMMENT_NODE, ] : continue
			__notification.append(__XML__.get_attributes(j.attributes))

	return __notification

def get_buttons_toolbar (nodes) :
	__buttons_toolbar = list()
	for i in nodes :
		if i.nodeType in [i.TEXT_NODE, i.COMMENT_NODE, ] : continue

		for j in i.childNodes :
			if j.nodeType in [j.TEXT_NODE, j.COMMENT_NODE, ] : continue
			__buttons_toolbar.append(__XML__.get_attributes(j.attributes))

	return __buttons_toolbar


if __name__ == "__main__" :
	c = ChildConfig("child/terminal/config.xml")
	print c.getAll()

	c = GlobalConfig("child/global.xml")
	print c.getAll()

"""
Description
-----------


ChangeLog
---------


Usage
-----


"""

__author__ =  "Spike^ekipS <spikeekips@gmail.com>"
__version__=  "0.1"
__nonsense__ = ""

__file__ = "XMLHandler.py"


