# -*- coding: utf-8 -*-
#    Copyright 2005 Spike^ekipS <spikeekips@gmail.com>
#
#       This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import gtk
from vee import Conf

class Core :

	"""
	The basic treeview class.

	Properties
	===========
		enable-search
		expander-column
		fixed-height-mode
		hadjustment
		headers-clickable
		headers-visible
		hover-expand
		hover-selection
		model
		reorderable
		rules-hint
		search-column
		vadjustment

	"""

	XML = None
	WIDGET_TREEVIEW = None
	WIDGET_TREESTORE = None
	MODEL = None
	HANDLER_ID = dict()

	PROPERTIES = { \
		"enable-search" : True, \
		"fixed-height-mode" : True, \
		"headers-clickable" : True, \
		"headers-visible" : True, \
		"hover-expand" : True, \
		"hover-selection" : True, \
		"reorderable" : True, \
		"rules-hint" : True, \
		#"search-column" : -1, \
		#"model" : , \
		#"expander-column" : , \
		#"hadjustment" : , \
		#"vadjustment" : , \
	}

	SELECTION_MODE = gtk.SELECTION_SINGLE
	SELECTION_CURRENT = None

	PATH_CURRENT = (0,)

	DND_TARGET_TABLE = [ \
		('MY_TREE_MODEL_ROW', gtk.TARGET_SAME_WIDGET, 0), \
		( "text/uri-list",  0, Conf.DND_TARGET_URI_LIST ), \
	]

	def __init__ (self, xml, treeview_name) :

		self.XML = xml
		self.WIDGET_TREEVIEW = self.XML.get_widget(treeview_name)

		# set property
		for i, j in self.PROPERTIES.items() :
			self.WIDGET_TREEVIEW.set_property(i, j)

		self.create_frame()
		self.insert_row()
		self.initialize()
		self.connectSignal()

	def initialize (self) :
		self.SELECTION = self.WIDGET_TREEVIEW.get_selection()
		self.SELECTION.set_mode(self.SELECTION_MODE)

	def connectSignal (self) :
		self.HANDLER_ID["selection-changed"] = \
			self.SELECTION.connect("changed", self.on_change_selection)
		self.HANDLER_ID["row-activated"] = \
			self.WIDGET_TREEVIEW.connect("row-activated", self.on_row_activated)
		self.HANDLER_ID["row-changed"] = \
			self.WIDGET_TREESTORE.connect("row-changed", self.on_changed_row)

	def create_frame (self) : pass
	def insert_row (self) : pass
	def add_row (self, *args) : pass

	################################################################
	# HANDLER
	def on_changed_row (self, treemodel, path, iter, user_param1=None) :
		pass

	def on_change_selection (self, selection) :
		self.SELECTION_CURRENT = selection

	def on_row_activated \
			(self, treeview, path, view_column, user_param1=None) :
		pass

	def on_drag_begin (self, widget, context) :
		"""
		NOT IMPLEMENTED
		"""
		pass

	def on_drag_end (self, widget, context) :
		"""
		NOT IMPLEMENTED
		"""
		pass

	def on_drag_data_get_data \
			(self, treeview, context, selection, target_id, etime) :
		"""
		NOT IMPLEMENTED
		"""
		pass

	def on_drag_data_received_data \
			(self, treeview, context, x, y, selection, info, etime) :
		"""
		NOT IMPLEMENTED
		"""
		pass

	###############################################################
	# MISC methods
	def enableDND (self) :
		"""
		NOTE :
		--------------------------------------------------------------------------
		If you does not enable to use DND in treeview, this code will be useless,
		but to enable DND in treeview and the target of DND is outside of treeview
		widget, the handler of 'drag_data_received' and 'drag_data_get' must be
		implemented by hand.
		--------------------------------------------------------------------------
		"""
		self.WIDGET_TREEVIEW.enable_model_drag_source( \
			gtk.gdk.BUTTON1_MASK, \
			self.DND_TARGET_TABLE, \
			gtk.gdk.ACTION_DEFAULT | gtk.gdk.ACTION_MOVE | gtk.gdk.ACTION_COPY \
		)
		self.WIDGET_TREEVIEW.enable_model_drag_dest( \
			self.DND_TARGET_TABLE, gtk.gdk.ACTION_DEFAULT \
		)

		self.WIDGET_TREEVIEW.connect("drag-begin", self.on_drag_begin)
		self.WIDGET_TREEVIEW.connect("drag-end", self.on_drag_end)

		self.WIDGET_TREEVIEW.connect("drag_data_get", self.on_drag_data_get_data)
		self.WIDGET_TREEVIEW.connect("drag_data_received", self.on_drag_data_received_data)

	def set_editable (self, iter, column_number, cell_number) :
		__path = self.get_path(iter)

		__column = self.WIDGET_TREEVIEW.get_column(column_number)
		cell = __column.get_cell_renderers()[cell_number]
		cell.set_property("editable", True)

		self.WIDGET_TREEVIEW.set_cursor(__path, __column, True)

	def clear (self) :
		return self.WIDGET_TREESTORE.clear()

	def append (self, rows, parent=None) :
		return self.WIDGET_TREESTORE.append(parent, rows)

	def prepend (self, rows, parent=None) :
		self.WIDGET_TREESTORE.prepend(parent, rows)

	def set_cursor (self, path) :
		self.WIDGET_TREEVIEW.set_cursor(path)

	def move (self, from_path, to_path) :
		if from_path == to_path : return

		__iter_from = self.WIDGET_TREESTORE.get_iter(from_path)
		if not __iter_from : return
		__iter_to = self.WIDGET_TREESTORE.get_iter(to_path)
		if not __iter_to : return

		self.WIDGET_TREESTORE.move_after(__iter_from, __iter_to)

	def remove (self, **kwargs) :
		"""
		One of path, iter must be given.

		@return : bool()
		"""

		treeiter = None
		if kwargs.has_key("treeiter") : treeiter = kwargs["treeiter"]
		if kwargs.has_key("path") :
			treeiter = self.WIDGET_TREESTORE.get_iter(kwargs["path"])

		if not treeiter : return False

		self.WIDGET_TREESTORE.remove(treeiter)

		return True

	def get_path (self, iter) :
		return self.WIDGET_TREESTORE.get_path(iter)

	def get_iter (self, path) :
		__treeiter = self.WIDGET_TREESTORE.get_iter(path)

		return __treeiter

	def get_value (self, path, column_number) :
		__treeiter = self.WIDGET_TREESTORE.get_iter(path)
		__value = self.WIDGET_TREESTORE.get_value(__treeiter, column_number)

		return __value

	def set_value (self, path, column_number, value) :
		__iter = self.WIDGET_TREESTORE.get_iter(path)

		if not __iter : return False

		self.WIDGET_TREESTORE.set_value( \
			__iter, \
			column_number, \
			value \
		)

	def get_selected (self) :
		__selection = self.WIDGET_TREEVIEW.get_selection()
		if not __selection : return (None, None, )

		if __selection.get_mode() == \
				gtk.SELECTION_MULTIPLE :
			(__treestore, __treeiter) = \
				__selection.get_selected_rows()
		else :
			(__treestore, __treeiter) = \
				__selection.get_selected()

		return (__treestore, __treeiter, )

	def get_value_selected (self, column_number) :
		(__treestore, __treeiter, ) = self.get_selected()
		if not __treeiter : return

		return self.WIDGET_TREESTORE.get_value(__treeiter, column_number)

	def get_current_path (self, path) :
		return self.PATH_CURRENT

	def set_current_path (self, path) :
		self.PATH_CURRENT = path

	def goUpOneRow (self) :
		(__treestore, __treeiter, ) = self.get_selected()
		if not __treeiter :
			return

		__path = self.get_path(__treeiter)
		__column = self.WIDGET_TREEVIEW.get_column(0)

		__path = get_previous_path(__path)

		try :
			self.get_iter(__path)
		except ValueError :
			return
		except TypeError :
			return
		else :
			self.WIDGET_TREEVIEW.set_cursor(__path, __column, True)

	def goDownOneRow (self) :
		(__treestore, __treeiter, ) = self.get_selected()
		if not __treeiter :
			return

		__path = self.get_path(__treeiter)
		__column = self.WIDGET_TREEVIEW.get_column(0)

		# check whether current path has child or not.
		if self.WIDGET_TREEVIEW.row_expanded(__path) and \
				self.WIDGET_TREESTORE.iter_has_child(__treeiter) :
			__path = __path + (0, )
		else :
			# get next path
			__path = get_next_path(__path)

		# check whether next path is valid or not.
		try :
			self.get_iter(__path)
		except ValueError :
			if len(__path) > 1 :
				__path = get_next_path(__path[:-1])
			else :
				return
		except TypeError :
			return

		# move cursor.
		try :
			self.get_iter(__path)
		except ValueError :
			return
		except TypeError :
			return
		else :
			self.WIDGET_TREEVIEW.set_cursor(__path, __column, True)

	def expandRow (self) :
		(__treestore, __treeiter, ) = self.get_selected()
		if not __treeiter :
			return

		__path = self.get_path(__treeiter)
		self.WIDGET_TREEVIEW.expand_to_path(__path)

	def collapseRow (self) :
		"""
		If this iter has no child and so can not collapse, move the cursor
		to it's parent row.
		"""
		(__treestore, __treeiter, ) = self.get_selected()
		if not __treeiter :
			return

		__path = self.get_path(__treeiter)

		if self.WIDGET_TREESTORE.iter_has_child(__treeiter) :
			self.WIDGET_TREEVIEW.collapse_row(__path)
		else :
			# get parent
			__iter_parent = self.WIDGET_TREESTORE.iter_parent(__treeiter)
			if __iter_parent :
				self.set_cursor(self.get_path(__iter_parent))

from vee import func, __OS__
class TreeViewWithMultipleItemSelectionDND :

	ROWS_CURRENT_SELECTED = list()
	IS_MULTIPLE_SELECTED = False
	IS_DND_STARTED = False

	"""
	DND_TARGET_TABLE = [
		( "application/x-color", 0, DND_TARGET_COLOR ),
		( "property/bgimage",    0, DND_TARGET_BGIMAGE ),
		( "x-special/gnome-reset-background", 0, DND_TARGET_RESET_BG ),
		( "text/uri-list",  0, DND_TARGET_URI_LIST ),
		( "text/x-moz-url",  0, DND_TARGET_MOZ_URL ),
		( "UTF8_STRING", 0, DND_TARGET_UTF8_STRING ),
		( "COMPOUND_TEXT", 0, DND_TARGET_COMPOUND_TEXT ),
		( "TEXT", 0, DND_TARGET_TEXT ),
		( "STRING",     0, DND_TARGET_STRING ),
		( "text/plain", 0, DND_TARGET_TEXT_PLAIN )
	]
	"""

	DND_TARGET_TABLE = [
		( "text/uri-list",  0, Conf.DND_TARGET_URI_LIST ),
	]

	def __init__ (self) :
		self.enableDND()

	def enableDND (self) :
		self.WIDGET_TREEVIEW.get_selection().set_mode(gtk.SELECTION_SINGLE)

		self.WIDGET_TREEVIEW.enable_model_drag_source( \
			gtk.gdk.BUTTON1_MASK, \
			self.DND_TARGET_TABLE, \
			gtk.gdk.ACTION_COPY \
		)

		self.WIDGET_TREEVIEW.enable_model_drag_dest( \
			self.DND_TARGET_TABLE, gtk.gdk.ACTION_DEFAULT)

		self.WIDGET_TREEVIEW.drag_source_set( \
			gtk.gdk.BUTTON1_MASK, \
			self.DND_TARGET_TABLE, \
			gtk.gdk.ACTION_COPY \
		)

		self.WIDGET_TREEVIEW.get_selection().set_mode(gtk.SELECTION_MULTIPLE)

		self.WIDGET_TREEVIEW.connect("motion-notify-event", \
			self.on_motion_notify_event)
		self.WIDGET_TREEVIEW.connect("button-press-event", \
			self.on_button_press_event)
		self.WIDGET_TREEVIEW.connect_after("button-release-event", \
			self.on_button_release_event)
		self.WIDGET_TREEVIEW.connect("key-press-event", \
			self.on_key_press_event)
		self.WIDGET_TREEVIEW.connect("key-release-event", \
			self.on_key_release_event)

		self.WIDGET_TREEVIEW.connect("drag-begin", self.on_drag_begin)
		self.WIDGET_TREEVIEW.connect("drag-end", self.on_drag_end)

		self.WIDGET_TREEVIEW.connect("drag_data_get", self.on_drag_data_get_data)
		self.WIDGET_TREEVIEW.connect("drag_data_received", self.on_drag_data_received_data)

	def on_motion_notify_event (self, treeview, event) :
		if self.IS_MULTIPLE_SELECTED and self.IS_DND_STARTED :
			self.IS_DND_STARTED = False

	def on_button_release_event (self, treeview, event) :
		try :
			path, column, x, y = self.WIDGET_TREEVIEW.get_path_at_pos(int(event.x), int(event.y))
		except TypeError :
			return

		selection = self.WIDGET_TREEVIEW.get_selection()
		if self.IS_MULTIPLE_SELECTED :
			selection.unselect_all()
			selection.select_path(path)
			self.IS_MULTIPLE_SELECTED = False
			return

		self.ROWS_CURRENT_SELECTED = selection.get_selected_rows()[1]

		if len(self.ROWS_CURRENT_SELECTED) < 2 :
			self.IS_MULTIPLE_SELECTED = False
		else :
			self.IS_MULTIPLE_SELECTED = True

		return False

	def on_button_press_event (self, treeview, event) :

		if event.button == 1 :
			selection = self.WIDGET_TREEVIEW.get_selection()
			try :
				path, column, x, y = self.WIDGET_TREEVIEW.get_path_at_pos(int(event.x), int(event.y))
			except TypeError :
				return

			if self.IS_MULTIPLE_SELECTED :
				self.IS_DND_STARTED = True

				return True
			else :
				self.IS_DND_STARTED = False

		return False

	def on_key_press_event (self, treeview, event) :
		__keyname = func.get_keyname(event)

		if __keyname == "Return" :
			__iters = self.get_selected()[1]
			if len(__iters) > 1 :
				return True

		return False

	def on_key_release_event (self, treeview, event) :

		selection = self.WIDGET_TREEVIEW.get_selection()
		self.ROWS_CURRENT_SELECTED = selection.get_selected_rows()[1]

		if len(self.ROWS_CURRENT_SELECTED) < 2 :
			self.IS_MULTIPLE_SELECTED = False
		else :
			self.IS_MULTIPLE_SELECTED = True

		return False

	def on_drag_begin (self, widget, context) :
		self.WIDGET_TREEVIEW.drag_dest_unset()

	def on_drag_end (self, widget, context) :
		# re-store the selection
		selection = self.WIDGET_TREEVIEW.get_selection()
		if len(self.ROWS_CURRENT_SELECTED) > 1 :
			selection.unselect_all()
			for i in self.ROWS_CURRENT_SELECTED :
				selection.select_path(i)

			self.IS_MULTIPLE_SELECTED = True

		self.WIDGET_TREEVIEW.enable_model_drag_dest( \
			self.DND_TARGET_TABLE, gtk.gdk.ACTION_DEFAULT)

	def on_drag_data_get_data (self, treeview, context, selection, target_id, etime) :
		"""
		NOT IMPLEMENTED
		"""
		pass

	def on_drag_data_received_data (self, treeview, context, x, y, selection, info, etime) :
		"""
		NOT IMPLEMENTED
		"""
		pass


######################################
# FUNCTIONS
def get_next_path (path) :
	__n = path[-1] + 1

	if len(path) < 2 :
		__path = (__n, )
	else :
		__path = path[:-1] + (__n, )

	return __path

def get_previous_path (path) :
	__n = path[-1] - 1

	if len(path) < 2 :
		if __n < 0 :
			__path = None
		else :
			__path = (__n, )
	else :
		if __n < 0 :
			__path = path[:-1]
		else :
			__path = path[:-1] + (__n, )

	return __path

"""
Description
-----------


ChangeLog
---------


Usage
-----


"""

__author__ =  "Spike^ekipS <spikeekips@gmail.com>"
__version__=  "0.1"
__nonsense__ = ""

__file__ = "__TREEVIEW__.py"


