# -*- coding: utf-8 -*-
#    Copyright 2005 Spike^ekipS <spikeekips@gmail.com>
#
#       This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import os
from vee import Conf, __VEE__, func, GNOME, WIDGET, Conf_i18n, child
from vee.child import __WIDGET__, main

PROPERTY_BOOKMARK = { \
	"on_changed_label" : "on_changed_label", \
}

class CORE (__VEE__.Core, __WIDGET__.WIDGET) :

	NOTEBOOK = None
	PANEL_BOOKMARK = None
	TREEITER = None
	PANEL_PATH = None
	UID = None
	NODE = None
	DATA = None
	CHILD_CONFIG = None
	CHILD_NAME = None

	def __init__ (self, childname, uid, **kwargs) :

		__VEE__.Core.__init__(self, **kwargs)
		self.UID = uid
		self.CHILD_NAME = childname
		self.BOOKMARK = self.STATE_GLOBAL["bookmark"]

		if not kwargs.has_key("panel") :
			raise RunetimeError, "panel must be given."
		else :
			self.PANEL_BOOKMARK = kwargs["panel"]
			(__treestore, self.TREEITER, ) = self.PANEL_BOOKMARK.get_selected()
			self.PANEL_PATH = self.PANEL_BOOKMARK.get_path(self.TREEITER)

		# get config
		self.CHILD_CONFIG = self.STATE_GLOBAL["children"][childname]
		self.NODE = self.BOOKMARK.getNodeByUID(self.UID)
		self.DATA = self.BOOKMARK.getValuesByUID(self.UID)

		self.normalizeData()

		__WIDGET__.WIDGET.__init__( \
			self, childname, "bookmark_property", self.CHILD_CONFIG)

		self.NOTEBOOK = self.getWidget("notebook")

	def initialize (self) :

		# set icon.
		self.setIcon()

		# set label
		self.getWidget("entry1").set_text(self.DATA["label"])

	def normalizeData (self) :
		for i, j in self.CHILD_CONFIG["setting"].items() :
			if not self.DATA.has_key(i) :
				self.DATA[i] = func.casting(j["type"], j["default"])

	def connect_handler (self, handler=dict()) :
		handler.update(PROPERTY_BOOKMARK)
		__WIDGET__.WIDGET.connect_handler(self, handler)

	def setIcon (self) :
		# set type image
		GNOME_ICON = GNOME.Icon(Conf.ICON_SIZE_PROPERTY_TYPE)
		if self.CHILD_CONFIG["layout"]["icon"]["stock_id"] :
			__icon = GNOME_ICON.getPixbufByIcon( \
				self.CHILD_CONFIG["layout"]["icon"]["stock_id"])
		elif self.CHILD_CONFIG["layout"]["icon"]["image"]:
			__icon_path = os.path.join( \
				child.get_path_child(self.CHILD_NAME), \
				self.CHILD_CONFIG["layout"]["icon"]["image"] \
			)
			__icon = GNOME_ICON.getPixbufFromFile(__icon_path)

		self.getWidget("image1").set_from_pixbuf(__icon)

	def on_close (self, widget) :
		# save bookmark.
		self.PANEL_BOOKMARK.save_tree_to_xml()

		__WIDGET__.WIDGET.on_close(self, widget)

	def on_changed_label (self, entry) :
		__new_label = entry.get_text().strip()

		if len(__new_label) < 1 : return

		self.BOOKMARK.setValue(self.NODE, "label", __new_label)

		if self.TREEITER :
			self.PANEL_BOOKMARK.set_value( \
				self.PANEL_PATH, \
				Conf.TREEVIEW_BOOKMARK_PANEL_COLUMN_NAME, \
				__new_label \
			)

HANDLER_PROPERTY_VTE = { \
	"color" : { \
		"on_color_use_system_theme"								: "on_color_use_system_theme", \
		"on_color_fore_background_changed_builtin_schemes" \
			: "on_color_fore_background_changed_builtin_schemes", \
		"on_color_foreground"								: "on_color_foreground", \
		"on_color_background"								: "on_color_background", \
		"on_color_palette_changed_builtin_schemes" \
			: "on_color_palette_changed_builtin_schemes", \
	}, \
	"font" : { \
		"on_font_set"								: "on_font_set", \
		"on_use_system_font"						: "on_use_system_font", \
	}, \
	"background" : { \
		"on_changed_background_image_file"	: "on_changed_background_image_file", \
		"on_changed_saturation"					: "on_changed_saturation", \
	}, \
	"etc" : { \
		"on_changed_max_scroll_line"			: "on_changed_max_scroll_line", \
		"on_changed_enable_logging"			: "on_changed_enable_logging", \
		"on_changed_enable_scrollbar"			: "on_changed_enable_scrollbar", \
		"on_changed_directory_logging"		: "on_changed_directory_logging", \
	}, \
}

class VTE (__VEE__.Core, CORE) :

	XML_FONT = None
	XML_COLOR = None
	XML_BACKGROUND = None
	XML_ETC = None
	HANDLER_ID = dict()

	def __init__ (self, childname, uid, **kwargs) :
		"""
		str childname : terminal, vim, external_command, etc
		str uid : uid of bookmark item.
		"""
		__VEE__.Core.__init__(self, **kwargs)
		self.CHILDNAME = childname
		CORE.__init__(self, self.CHILDNAME, uid, **kwargs)

	XML_TYPE = dict()
	def initialize (self) :
		CORE.initialize (self)

		for __type in ["font", "color", "background", "etc", ] :
			(self.XML_TYPE[__type], __widget_type, ) = \
				WIDGET.get_widget("box_%s" % __type)
			__box = self.getWidget("box_%s" % __type)

			if not __box :
				continue

			# initialize widget.
			if hasattr(self, "in_activate_%s" % __type) :
				getattr(self, "in_activate_%s" % __type)()

			__box.pack_start(__widget_type, True)

			__dict_handler = dict()
			for k, m in HANDLER_PROPERTY_VTE[__type].items() :
				__dict_handler[k] = eval("self.%s" % m)

			if len(__dict_handler.keys()) :
				self.XML_TYPE[__type].signal_autoconnect(__dict_handler)

		# END ############################################################
		self.WIDGET_SELF.set_property("resizable", False)
		self.getWidget("entry1").grab_focus()

	def in_activate_background (self) :
		xml = self.XML_TYPE["background"]

		__radio_none					= xml.get_widget("radiobutton19")
		__radio_background_image	= xml.get_widget("radiobutton20")
		__radio_transparent			= xml.get_widget("radiobutton21")

		__vte_background_type = self.DATA["vte_background_type"]
		__vte_background_image_file = \
			self.DATA["vte_background_image_file"]
		__vte_background_shade = \
			self.DATA["vte_background_shade"]

		if self.DATA["vte_background_type"] == "transparent" :
			__radio_transparent.set_active(True)

			xml.get_widget("hbox122").set_property("sensitive", False)
		elif __vte_background_type == "image-file" :
			__radio_background_image.set_active(True)
			xml.get_widget("vbox147").set_property("sensitive", True)

			if __vte_background_image_file :
				xml.get_widget("filechooserbutton18").set_filename( \
					__vte_background_image_file)
		else :
			__radio_none.set_active(True)

			xml.get_widget("hbox122").set_property("sensitive", False)
			xml.get_widget("vbox147").set_property("sensitive", False)

		__scale_shade = __vte_background_shade
		if __scale_shade < 0 or __scale_shade > 1 :
			__scale_shade = 0.2

		# set shade scale.
		xml.get_widget("hscale5").set_value(__scale_shade)

	def in_activate_font (self) :
		xml = self.XML_TYPE["font"]

		# font #############################################
		xml.get_widget("checkbutton37").set_active( \
			self.DATA["use_system_default_font"] \
		)
		xml.get_widget("fontbutton1").set_font_name(self.DATA["font"])
		xml.get_widget("table79").set_property("sensitive", \
			not xml.get_widget("checkbutton37").get_active())

	def in_activate_color (self) :
		xml = self.XML_TYPE["color"]

		# USE SYSTEM THEME.
		if not self.DATA.has_key("use_system_theme_color") :
			self.DATA["use_system_theme_color"] = False

		xml.get_widget("checkbutton39").set_active( \
			self.DATA["use_system_theme_color"])

		xml.get_widget("hbox125").set_property( \
			"sensitive", not self.DATA["use_system_theme_color"])
		xml.get_widget("table84").set_property( \
			"sensitive", not self.DATA["use_system_theme_color"])

		# FOREGROUND & BACKGROUND COLOR AS BUILT-IN SCHEMES TYPE
		__combobox_fgbgcolor_type = xml.get_widget("combobox2")
		[__combobox_fgbgcolor_type.append_text(_(i[1])) \
			for i in main.VTE_FGCOLOR_BUILTIN_SCHEMES_LIST]

		if not self.DATA.has_key("foreground_background_color_builtin_schemes_type") :
			self.DATA["foreground_background_color_builtin_schemes_type"] = \
				"black_on_white"

		__idx = 0
		__schemes = main.VTE_FGCOLOR_BUILTIN_SCHEMES_LIST[__idx][0]
		for i in range(len(main.VTE_FGCOLOR_BUILTIN_SCHEMES_LIST)):
			if self.DATA["foreground_background_color_builtin_schemes_type"] == \
				main.VTE_FGCOLOR_BUILTIN_SCHEMES_LIST[i][0] :
				__idx = i
				__schemes = main.VTE_FGCOLOR_BUILTIN_SCHEMES_LIST[i][0]
				break

		__combobox_fgbgcolor_type.set_active(__idx)

		__color_foreground = xml.get_widget("colorbutton21")
		__color_background = xml.get_widget("colorbutton22")

		if self.DATA["foreground_background_color_builtin_schemes_type"] == \
				"custom" :
			__fg_color = self.DATA["color_foreground"]
			__bg_color = self.DATA["color_background"]
		else :
			(__fg_color, __bg_color, ) = main.VTE_FGCOLOR_BUILTIN_SCHEMES_LIST[__idx][2]

		__color_foreground.set_color(func.str_to_color(__fg_color))
		__color_background.set_color(func.str_to_color(__bg_color))

		# PALETTE
		__combobox_palette_select = xml.get_widget("combobox1")
		[__combobox_palette_select.append_text(_(i[1])) \
			for i in main.VTE_PALETTE_BUILTIN_SCHEMES_LIST]

		if not self.DATA.has_key("color_palette_builtin_schemes_type") :
			self.DATA["color_palette_builtin_schemes_type"] = "linux"

		__idx = 0
		__schemes = main.VTE_PALETTE_BUILTIN_SCHEMES_LIST[__idx][0]
		for i in range(len(main.VTE_PALETTE_BUILTIN_SCHEMES_LIST)):
			if self.DATA["color_palette_builtin_schemes_type"] == \
				main.VTE_PALETTE_BUILTIN_SCHEMES_LIST[i][0] :
				__idx = i
				__schemes = main.VTE_PALETTE_BUILTIN_SCHEMES_LIST[i][0]
				break

		__combobox_palette_select.set_active(__idx)

		if main.TERMINAL_PALETTE.has_key(__schemes) :
			for i in range(1, 17) :
				__button = xml.get_widget("colorbutton-picker-%s" % i)
				__button.set_color( \
						func.str_to_color(main.TERMINAL_PALETTE[__schemes][i - 1]) \
					)

				# disable 'color-set' signal.
				__button.connect("button-press-event", WIDGET.on_no_response)
				__button.connect("key-press-event", WIDGET.on_no_response)

	def in_activate_etc (self) :
		xml = self.XML_TYPE["etc"]

		__enable_logging = self.DATA["enable_logging"]
		__log_directory = os.path.expanduser( \
			self.DATA["log_directory"] \
		)
		__enable_scrollbar = self.DATA["enable_scrollbar"]

		xml.get_widget("checkbutton36").set_active(__enable_logging)
		xml.get_widget("table78").set_property( \
			"sensitive", __enable_logging)
		if __log_directory :
			xml.get_widget("filechooserbutton19").set_current_folder( \
				__log_directory)

		xml.get_widget("spinbutton7").set_value( \
			int(self.DATA["max_scroll_line"]))

		xml.get_widget("checkbutton38").set_active(__enable_scrollbar)

	def getWidgetCurrent (self, type, name) :
		if not self.XML_TYPE.has_key(type) :
			return None
			#raise KeyError, "type, %s does not exist." % type
		else :
			return self.XML_TYPE[type].get_widget(name)

	##################################################
	# EVENT HANDLER
	def on_color_fore_background_changed_builtin_schemes (self, combobox) :
		xml = self.XML_TYPE["color"]
		__idx = combobox.get_active()

		try :
			__schemes = main.VTE_FGCOLOR_BUILTIN_SCHEMES_LIST[__idx][0]
		except IndexError :
			__schemes = "black_on_white"

		__color_foreground = xml.get_widget("colorbutton21")
		__color_background = xml.get_widget("colorbutton22")

		if __schemes == "custom" :
			__fg_color = self.DATA["color_foreground"]
			__bg_color = self.DATA["color_background"]
		else :
			(__fg_color, __bg_color, ) = main.VTE_FGCOLOR_BUILTIN_SCHEMES_LIST[__idx][2]

		__color_foreground.set_color(func.str_to_color(__fg_color))
		__color_background.set_color(func.str_to_color(__bg_color))

		self.BOOKMARK.setValue(self.NODE, "color_foreground", __fg_color)
		self.BOOKMARK.setValue(self.NODE, "color_background", __bg_color)
		self.BOOKMARK.setValue( \
			self.NODE, "foreground_background_color_builtin_schemes_type", __schemes)

	def on_color_foreground (self, button) :
		# If set by manually, schemes type must be set to 'custom'.
		xml = self.XML_TYPE["color"]
		__custom_idx = \
			[i for i in range(len(main.VTE_FGCOLOR_BUILTIN_SCHEMES_LIST)) \
				if main.VTE_FGCOLOR_BUILTIN_SCHEMES_LIST[i][0] == "custom" ]
		if len(__custom_idx) == 1 :
			self.DATA["color_foreground"] = func.color_to_str(button.get_color())
			xml.get_widget("combobox2").set_active(__custom_idx[0])

		self.BOOKMARK.setValue( \
			self.NODE, "foreground_background_color_builtin_schemes_type", "custom")
		self.BOOKMARK.setValue( \
			self.NODE, "color_foreground", func.color_to_str(button.get_color()))

	def on_color_background (self, button) :
		# If set by manually, schemes type must be set to 'custom'.
		xml = self.XML_TYPE["color"]
		__custom_idx = \
			[i for i in range(len(main.VTE_FGCOLOR_BUILTIN_SCHEMES_LIST)) \
				if main.VTE_FGCOLOR_BUILTIN_SCHEMES_LIST[i][0] == "custom" ]
		if len(__custom_idx) == 1 :
			self.DATA["color_background"] = func.color_to_str(button.get_color())
			xml.get_widget("combobox2").set_active(__custom_idx[0])

		self.BOOKMARK.setValue( \
			self.NODE, "foreground_background_color_builtin_schemes_type", "custom")
		self.BOOKMARK.setValue( \
			self.NODE, "color_background", func.color_to_str(button.get_color()))

	def on_color_use_system_theme (self, checkbutton) :
		self.DATA["use_system_theme_color"] = checkbutton.get_active()

		xml = self.XML_TYPE["color"]
		xml.get_widget("hbox125").set_property( \
			"sensitive", not checkbutton.get_active())
		xml.get_widget("table84").set_property( \
			"sensitive", not checkbutton.get_active())

		self.BOOKMARK.setValue( \
			self.NODE, "use_system_theme_color", checkbutton.get_active())

	def on_color_palette_changed_builtin_schemes (self, combobox) :
		xml = self.XML_TYPE["color"]
		__idx = combobox.get_active()

		try :
			__schemes = main.VTE_PALETTE_BUILTIN_SCHEMES_LIST[__idx][0]
		except IndexError :
			__schemes = "linux"

		if main.TERMINAL_PALETTE.has_key(__schemes) :
			for i in range(1, 17) :
				__button = xml.get_widget("colorbutton-picker-%s" % i)
				__button.set_color( \
						func.str_to_color(main.TERMINAL_PALETTE[__schemes][i - 1]) \
					)
				# disable 'color-set' signal.
				__button.connect("button-press-event", WIDGET.on_no_response)
				__button.connect("key-press-event", WIDGET.on_no_response)


		self.BOOKMARK.setValue( \
			self.NODE, "color_palette_builtin_schemes_type", __schemes)

	def on_font_set (self, fontbutton) :
		self.BOOKMARK.setValue(self.NODE, "font", fontbutton.get_font_name())

	def on_use_system_font (self, checkbutton) :
		__active = checkbutton.get_active()
		self.BOOKMARK.setValue( \
			self.NODE, "use_system_default_font", __active \
		)

		self.getWidgetCurrent("font", "table79").set_property( \
			"sensitive", not __active \
		)
		try :
			self.getWidgetCurrent("font", "table79").set_property( \
				"sensitive", not __active \
			)
		except AttributeError :
			pass

	def on_changed_vte_background_type (self, radiobutton, which) :
		if which == "transparent" :
			self.getWidgetCurrent("background", "vbox147").set_property("sensitive", True)
			self.getWidgetCurrent("background", "hbox122").set_property("sensitive", False)
		elif which == "image-file" :
			#self.getWidgetCurrent("background", "vbox147").set_property("sensitive", False)
			self.getWidgetCurrent("background", "hbox122").set_property("sensitive", True)
		else : # none
			self.getWidgetCurrent("background", "hbox122").set_property("sensitive", False)
			self.getWidgetCurrent("background", "vbox147").set_property("sensitive", False)

		self.BOOKMARK.setValue(self.NODE, "vte_background_type", which)

	def on_changed_background_image_file (self, chooser) :
		__new_filename = chooser.get_filename()
		if not __new_filename : return

		self.BOOKMARK.setValue(self.NODE, "vte_background_type", "image-file")
		self.BOOKMARK.setValue(self.NODE, "vte_background_image_file", __new_filename)

	def on_changed_saturation (self, hscale) :
		__value = round(hscale.get_value(), 1)
		self.BOOKMARK.setValue(self.NODE, "vte_background_shade", __value)

	def on_changed_max_scroll_line (self, spin) :
		__value = spin.get_value()

		self.BOOKMARK.setValue(self.NODE, "max_scroll_line", int(__value))

	def on_changed_enable_scrollbar (self, checkbutton) :
		__enable = checkbutton.get_active()
		self.BOOKMARK.setValue(self.NODE, "enable_scrollbar", __enable)

	def on_changed_enable_logging (self, checkbutton) :
		__enable = checkbutton.get_active()
		try :
			self.getWidgetCurrent( \
				"etc", "table78").set_property("sensitive", __enable)
		except AttributeError :
			pass

		self.BOOKMARK.setValue(self.NODE, "enable_logging", __enable)

	def on_changed_directory_logging (self, chooser) :
		__directory = chooser.get_current_folder()
		if not __directory : return

		self.BOOKMARK.setValue(self.NODE, "log_directory", __directory)

LABEL_BOOKMARK_PROPERTY_CATEGORY = \
{   'bookmark_property_category': [   (   'GtkDialog',
                                          'bookmark_property_category',
                                          'Category Property',
                                          'title')],
    'label1': [('GtkLabel', 'label1', 'General', 'label')]}


"""
Description
-----------


ChangeLog
---------


Usage
-----


"""

__author__ =  "Spike^ekipS <spikeekips@gmail.com>"
__version__=  "0.1"
__nonsense__ = ""

__file__ = "__PROPERTY__.py"


