# -*- coding: utf-8 -*-
#	 Copyright (C) 2005 Spike^ekipS <spikeekips@gmail.com>
#
#	 http://vee.kldp.net
#
#		 This program is free software; you can redistribute it and/or modify
#	 it under the terms of the GNU General Public License as published by
#	 the Free Software Foundation; either version 2 of the License, or
#	 (at your option) any later version.
#
#	 This program is distributed in the hope that it will be useful,
#	 but WITHOUT ANY WARRANTY; without even the implied warranty of
#	 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	 GNU General Public License for more details.
#
#	 You should have received a copy of the GNU General Public License
#	 along with this program; if not, write to the Free Software
#	 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import sys, os, time, sre
import gobject, gtk, vte
from vee import Conf, WIDGET, GConf, func, \
	i18n, __OS__, __VEE__, GNOME, Conf_i18n
from vee.child import __BASIC__, main

_ = i18n.gettext()

DEFAULT_ENVIRONMENT_FOR_SHELL = (\
	"PROMPT_COMMAND='echo -ne \"\\033]0;${PWD}\\007\"'", \
)

SIGNAL_REMOVE = "child-exited"
SIGNAL_TITLE_CHANGED = "window-title-changed"

class egg (__BASIC__.egg) :

	WIDGET = None
	BOX = None

	# set default value.
	COLOR_BACKGROUND = "black"
	COLOR_FOREGROUND = "ivory"
	COLOR_DIM = "blue"
	COLOR_HIGHLIGHT = "blue"
	COLOR_CURSOR = "red"
	DEFAULT_FONTNAME = "monospace 16"
	EMULATION = "xterm"
	UNIT_FONT_DELM = 1000
	RANGE_FONT_SIZE = (10000, 30000)
	MAX_SCROLL_LINE = 100
	MOUSE_AUTO_HIDE = True

	CURRENT_CURSOR_ROW = 0
	CURRENT_CURSOR_COLUMN = 0
	PREVIOUS_CURSOR_ROW = 0
	PREVIOUS_CURSOR_COLUMN = 0

	IDENTIFIER_LOGGING = None
	FD_LOGGING = None
	LOGGING_ENABLED = True

	def normalizeProperty (self, kwargs=dict()) :
		__BASIC__.egg.normalizeProperty(self, kwargs)

		if self.PROPERTIES.has_key("enable_logging") and \
				self.PROPERTIES["enable_logging"]:
			if self.PROPERTIES.has_key("log_directory") and \
					self.PROPERTIES["log_directory"] :
				self.LOGGING_ENABLED = True
		else :
			self.LOGGING_ENABLED = False

	def getWidget (self) :
		return self.WIDGET

	def getBox (self) :
		return self.BOX

	def attached (self, obj_tab) :
		__BASIC__.egg.attached(self, obj_tab)

		self.WIDGET.connect(SIGNAL_REMOVE, self.OBJ_TAB.on_remove)

		self.EVENT_ID_TITLE_CHANGED = \
			self.WIDGET.connect( \
				SIGNAL_TITLE_CHANGED, \
				self.OBJ_TAB.on_changed_title \
			)

	def make (self) :
		self.WIDGET = vte.Terminal()
		self.WIDGET.set_property("visible", False)
		self.BOX = self.createBox()

		self.set_environment()

		self.activate_dnd()

		# connect key-press, key-release event.
		self.WIDGET.connect("key-press-event", self.on_key_press_event)
		self.WIDGET.connect("key-release-event", self.on_key_release_event)
		self.WIDGET.connect("button-press-event", self.attachPopupMenu)
		self.WIDGET.connect("size-allocate", self.on_resize)

		#gobject.idle_add(self.fork_command)

	def done (self) :
		self.fork_command()
		self.WIDGET.set_property("visible", True)
		if self.PROPERTIES.has_key("log_directory") :
			self.startLogging(self.PROPERTIES["log_directory"])

	def attachPopupMenu (self, vte, event) :
		show_menu(event, self.WIDGET, dict())

	def createDefaultEnvironmentForShell (self) :
		return ( \
			"UID_VEE=%s" % self.STATE_LOCAL["uid"], \
			"TERM=%s" % self.EMULATION, \
		)

	def parse_command (self) :
		__command_line = __OS__.parse_command_line_argument(self.PROPERTIES["command"])

		# default home directory.
		__directory = os.getcwd()

		__argument = (__command_line["command"], ) + __command_line["argument"]
		__environment = \
			self.createDefaultEnvironmentForShell() + \
			__command_line["environment"]

		self.setProperty("executive-command", __command_line["command"])

		return (__command_line["command"], __argument, __environment, __directory, )

	def fork_command (self) :
		(__command, __argument, __environment, __directory, ) = self.parse_command()

		self.WIDGET.fork_command( \
			__command, \
			argv=__argument, \
			envv=__environment, \
			directory=__directory, \
		)

	def checkEnvironment (self) :
		# If this failed, it will raise RuntimeError.
		try :
			__OS__.check_environment( \
				log_directory=self.PROPERTIES["log_directory"] \
			)
		except RuntimeError :
			self.LOGGING_ENABLED = True
		else :
			self.LOGGING_ENABLED = False

	def changeEnvironment (self) :
		for i in DEFAULT_ENVIRONMENT_FOR_SHELL :
			self.WIDGET.feed_child("export %s\n" % i)

		self.WIDGET.feed_child("clear\n")

	def reset (self, parm1=None, parm2=None, parm3=None) :
		self.WIDGET.reset(True, False)
		#self.WIDGET.set_size(80, 24)

	def getTitle (self) :
		__title = self.WIDGET.get_window_title()
		self.setProperty("title", __title)

		return __title

	def decrease_font (self, size=int()) :

		pd = self.WIDGET.get_font()
		__s = pd.get_size() - self.UNIT_FONT_DELM

		if __s not in \
				range(self.RANGE_FONT_SIZE[0], self.RANGE_FONT_SIZE[1]) :
			return

		pd.set_size(__s)
		self.WIDGET.set_font(pd)

		return

	def increase_font (self, size=int()) :

		pd = self.WIDGET.get_font()

		__s = pd.get_size() + self.UNIT_FONT_DELM
		if __s not in \
				range(self.RANGE_FONT_SIZE[0], self.RANGE_FONT_SIZE[1]) :
			return

		pd.set_size(__s)
		self.WIDGET.set_font(pd)

		return

	def set_environment (self) :
		self.reset()

		#self.WIDGET.set_size(30, 5)
		self.WIDGET.set_emulation(self.EMULATION)
		self.WIDGET.set_audible_bell(False)
		self.WIDGET.set_visible_bell(True)
		self.WIDGET.set_scroll_on_output(False)

		self.WIDGET.set_mouse_autohide(self.MOUSE_AUTO_HIDE)
		__max_scroll_line = self.MAX_SCROLL_LINE
		if self.PROPERTIES.has_key("max_scroll_line") :
			__max_scroll_line = int(self.PROPERTIES["max_scroll_line"])

		self.WIDGET.set_scrollback_lines(__max_scroll_line)

		self.set_color()
		self.set_background()
		self.set_font()

	def set_font (self) :
		if not self.PROPERTIES["use_system_default_font"] :
			__font = self.PROPERTIES["font"]
		else :
			__font = self.GCONF_CLIENT.getValueByPath( \
				"/desktop/gnome/interface/monospace_font_name")

		self.WIDGET.set_font_from_string(__font)
		self.reset()

	def set_background (self) :
		__background_type = None
		__background_shade = None
		__background_image_file = None
		__color_background = self.COLOR_BACKGROUND

		if self.PROPERTIES.has_key("color_background") :
			__color_background = self.PROPERTIES["color_background"]

		if self.PROPERTIES.has_key("vte_background_type") :
			__background_type = self.PROPERTIES["vte_background_type"]
		if self.PROPERTIES.has_key("vte_background_shade") :
			__background_shade = self.PROPERTIES["vte_background_shade"]

		if self.PROPERTIES.has_key("vte_background_image_file") :
			__background_image_file = self.PROPERTIES["vte_background_image_file"]

		# set background
		if __background_type == "transparent" :
			self.WIDGET.set_background_transparent(True)
			self.WIDGET.set_background_tint_color(func.str_to_color(__color_background))
			self.WIDGET.set_background_saturation(__background_shade)
		elif __background_type == "image-file" :
			self.WIDGET.set_background_image_file(__background_image_file)
			self.WIDGET.set_background_saturation(__background_shade)

	def set_color (self) :
		# initiate terminal color set.
		self.WIDGET.set_default_colors()

		if not self.PROPERTIES.has_key("color_palette_builtin_schemes_type") :
			self.PROPERTIES["color_palette_builtin_schemes_type"] = "linux"
		if not main.TERMINAL_PALETTE.has_key( \
				self.PROPERTIES["color_palette_builtin_schemes_type"]) :
			self.PROPERTIES["color_palette_builtin_schemes_type"] = "linux"

		__palette = [func.str_to_color(i) \
			for i in \
				main.TERMINAL_PALETTE[ \
					self.PROPERTIES["color_palette_builtin_schemes_type"] \
				] \
			]

		# get fore-background color
		if not self.PROPERTIES.has_key( \
				"foreground_background_color_builtin_schemes_type") :
			self.PROPERTIES["foreground_background_color_builtin_schemes_type"] = \
				"black_on_white"

		__foreground_background_color_builtin_schemes_type = \
			self.PROPERTIES["foreground_background_color_builtin_schemes_type"]

		if not self.PROPERTIES.has_key("use_system_theme_color") :
			self.PROPERTIES["use_system_theme_color"] = False

		if self.PROPERTIES["use_system_theme_color"] :
			__style = GNOME.get_color_from_default_style()
			__color_foreground = __style["text"][0]
			__color_background = __style["base"][0]
		elif __foreground_background_color_builtin_schemes_type == "custom" :
			__color = self.COLOR_FOREGROUND
			if self.PROPERTIES.has_key("color_foreground") :
				__color = self.PROPERTIES["color_foreground"]
			__color_foreground = func.str_to_color(__color)

			__color = self.COLOR_BACKGROUND
			if self.PROPERTIES.has_key("color_background") :
				__color = self.PROPERTIES["color_background"]
			__color_background = func.str_to_color(__color)
		else :
			__a = [i for i in main.VTE_FGCOLOR_BUILTIN_SCHEMES_LIST \
				if i[0] == __foreground_background_color_builtin_schemes_type]

			if len(__a) < 1 :
				__a = main.VTE_FGCOLOR_BUILTIN_SCHEMES_LIST[0]

			__color_foreground = func.str_to_color(__a[0][2][0])
			__color_background = func.str_to_color(__a[0][2][1])

		"""
		README : To set palette
		-------------------------------------------------------------------------
		palette_size must be either 0, 8, 16, or 24. If foreground is NULL and
		palette_size is greater than 0, the new foreground color is taken from
		palette[7]. If background is NULL and palette_size is greater than 0, the
		new background color is taken from palette[0]. If palette_size is 8 or 16,
		the third (dim) and possibly the second (bold) 8-color palettes are
		extrapolated from the new background color and the items in palette.
		-------------------------------------------------------------------------
		"""
		__palette[7] = __color_foreground
		__palette[0] = __color_background
		self.WIDGET.set_colors(__color_foreground, __color_background, __palette)

	def feed (self, str) :
		self.WIDGET.feed_child(str)

	def on_change_color (self, client, parm1, entry, type) :

		__c = self.GCONF_CLIENT.get_value_from_value(entry.get_value())
		__c0 = func.str_to_color(__c)

		try :
			__func = getattr(self.WIDGET, "set_color_%s" % type)
		except AttributeError :
			raise AttributeError, "What type, % is it?" % type
		else :
			__func(__c0)

	def startLogging (self, log_directory) :
		if not self.LOGGING_ENABLED :
			return

		if not __OS__.is_dir(log_directory) :
			return

		# get log filename.
		__filename = get_filename_logging(log_directory)
		if not __OS__.exists(__filename) :
			if not __OS__.is_writable(log_directory) :
				return

			self.FD_LOGGING = file(__filename, "w")
		else :
			self.FD_LOGGING = file(__filename, "a")

		self.FD_LOGGING.write("\n")
		self.FD_LOGGING.write( \
			"== (VEE) Truncated at %s.\n" % time.strftime("%c"))

		if not self.IDENTIFIER_LOGGING :
			self.IDENTIFIER_LOGGING = \
				self.WIDGET.connect_after("contents-changed", \
				self.on_content_changed \
			)

	def stopLogging (self) :
		if not self.LOGGING_ENABLED :
			return

		self.WIDGET.disconnect(self.IDENTIFIER_LOGGING)
		self.FD_LOGGING.close()
		self.IDENTIFIER_LOGGING = None

	def __is_selected (self, widget, column, row, data) :
		if row >= self.PREVIOUS_CURSOR_ROW and row < self.CURRENT_CURSOR_ROW :
			return True

		return False

	def activate_dnd (self) :
		self.WIDGET.drag_dest_set( \
			gtk.DEST_DEFAULT_MOTION |
			gtk.DEST_DEFAULT_HIGHLIGHT |
			gtk.DEST_DEFAULT_DROP,
			[ \
				( "text/uri-list",  0, Conf.DND_TARGET_URI_LIST ),
			], \
			gtk.gdk.ACTION_COPY \
		)
		self.WIDGET.connect("drag_data_received", \
			self.on_drag_received_data)

	###########################################################################
	# EVENT HANDLER
	def on_drag_received_data \
			(self, widget, context, x, y, selection, targetType, time) :
		# this is examples.
		__filelist = list()
		for i in __OS__.get_list_from_uri_list(selection.data) :
			(__scheme, __filepath, ) = __OS__.get_pathinfo_from_uri(i)
			if __scheme != "file" :
				continue
			if not __OS__.exists(__filepath) :
				continue

			__filelist.append(__filepath)

		for i in __filelist :
			self.WIDGET.feed_child(i)

	def on_content_changed (self, widget) :
		(self.CURRENT_CURSOR_COLUMN, self.CURRENT_CURSOR_ROW, ) = \
			widget.get_cursor_position()

		__text = widget.get_text(self.__is_selected)

		self.FD_LOGGING.write(__text)
		self.FD_LOGGING.flush()

		self.PREVIOUS_CURSOR_COLUMN = self.CURRENT_CURSOR_COLUMN
		self.PREVIOUS_CURSOR_ROW = self.CURRENT_CURSOR_ROW

	def on_resize (self, vte, rectangle) :
		width_char = vte.get_char_width()
		height_char = vte.get_char_height()
		width_vte = rectangle.width
		height_vte = rectangle.height

		columns = width_vte / width_char
		rows = height_vte / height_char

		vte.set_size(columns, rows)

	def on_key_release_event (self, vte, requisition) :
		pass

	def on_key_press_event (self, vte, event) :
		"""
		Prevent from the shortcut key for switching between tabs send to
		child modules.
		"""
		__keyname = func.get_keyname(event)
		__shortcut = \
			self.GCONF_CLIENT.getValue("keyboard_shortcut_modifier_switch_tab")
		__a = sre.compile(sre.escape("%")).split(__shortcut)
		if sre.compile("^%s[0-9][0-9]*$" % __a[0]).findall(__keyname) :
			return True
		elif __keyname in ["<Control>s", "<Shift><Control>s", ]:
			return True

		return False

HANDLER_PREFERENCES = { \
	"color" : { \
		"on_color_use_system_theme"								: "on_color_use_system_theme", \
		"on_color_fore_background_changed_builtin_schemes" \
			: "on_color_fore_background_changed_builtin_schemes", \
		"on_color_foreground"								: "on_color_foreground", \
		"on_color_background"								: "on_color_background", \
		"on_color_palette_changed_builtin_schemes" \
			: "on_color_palette_changed_builtin_schemes", \
	}, \
	"font" : { \
		"on_font_set"						: "on_font_set", \
		"on_use_system_font"				: "on_use_system_font", \
	}, \
	"background" : { \
		"on_changed_background_image_file"	: "on_changed_background_image_file", \
		"on_changed_saturation"					: "on_changed_saturation", \
	}, \
	"etc" : { \
		"on_changed_max_scroll_line"		: "on_changed_max_scroll_line", \
		"on_changed_enable_logging"		: "on_changed_enable_logging", \
		"on_changed_enable_scrollbar"		: "on_changed_enable_scrollbar", \
		"on_changed_directory_logging"	: "on_changed_directory_logging", \
	}, \
}

LIST_PREFERENCES_CHILD = [ \
	"color", \
	"font", \
	"background", \
	"etc", \
]

DATA_PREFERENCES_CHILD = { \
	"color" : { \
		"label" : "Color", \
		"class" : "color", \
	}, \
	"font" : { \
		"label" : "Font", \
		"class" : "font", \
	}, \
	"background" : { \
		"label" : "Background", \
		"class" : "background", \
	}, \
	"etc" : { \
		"label" : "Etc", \
		"class" : "etc", \
	}, \
}

class __PREFERENCES__ (__VEE__.Core) :
	XML = None
	WIDGET = None
	TYPE = None
	CHILDNAME = None
	GCONF_CLIENT = None

	def __init__ (self, childname, type, **kwargs) :
		"""
		str childname : terminal, vim, external_command, etc
		str type : color, font, etc, background, etc
		"""
		__VEE__.Core.__init__(self, **kwargs)

		if not HANDLER_PREFERENCES.has_key(type) :
			raise RuntimeError, "This type, %s was bad." % type
		else :
			self.TYPE = type
			self.CHILDNAME = childname

		(self.XML, self.WIDGET, ) = WIDGET.get_widget( \
			"box_%s" % type
		)

		self.initialize()
		self.__connectHandler()

	def getXMLAndWidget (self) :
		return (self.XML, self.WIDGET, )

	def getWidgetCurrent (self, widget_name) :
		return self.getWidget(widget_name)

	def getWidget (self, widget_name) :
		return self.XML.get_widget(widget_name)

	def initialize (self) :
		# set gconf app dir by childname.
		self.GCONF_CLIENT = \
			GConf.Client(os.path.join(Conf.GCONF_APP_DIR, self.CHILDNAME, ))

		# initialize widget and it's values.
		if not hasattr(self, "in_activate_%s" % self.TYPE) :
			raise RuntimeError, "Any widget was not activated."
		else :
			__func = getattr(self, "in_activate_%s" % self.TYPE)
			__func()

	def __connectHandler (self) :
		__handler = dict()
		for i, j in HANDLER_PREFERENCES[self.TYPE].items() :
			if hasattr(self, j) :
				__handler.update({i : getattr(self, j),} )

		self.XML.signal_autoconnect(__handler)

	def in_activate_background (self) :
		__radio_none					= self.getWidgetCurrent("radiobutton19")
		__radio_background_image	= self.getWidgetCurrent("radiobutton20")
		__radio_transparent			= self.getWidgetCurrent("radiobutton21")

		__vte_background_type = self.GCONF_CLIENT.getValue("vte_background_type")
		__vte_background_image_file = \
			self.GCONF_CLIENT.getValue("vte_background_image_file")
		__vte_background_shade = \
			self.GCONF_CLIENT.getValue("vte_background_shade")

		if __vte_background_type == "transparent" :
			__radio_transparent.set_active(True)

			self.getWidgetCurrent("hbox122").set_property("sensitive", False)
		elif __vte_background_type == "image-file" :
			__radio_background_image.set_active(True)
			self.getWidgetCurrent("vbox147").set_property("sensitive", True)

			if __vte_background_image_file :
				self.getWidgetCurrent("filechooserbutton18").set_filename( \
					__vte_background_image_file)
		else :
			__radio_none.set_active(True)

			self.getWidgetCurrent("hbox122").set_property("sensitive", False)
			self.getWidgetCurrent("vbox147").set_property("sensitive", False)

		__scale_shade = __vte_background_shade
		if __scale_shade < 0 or __scale_shade > 1 :
			__scale_shade = 0.2

		# set shade scale.
		self.getWidgetCurrent("hscale5").set_value(__scale_shade)

		__radio_none.connect( \
			"toggled", self.on_changed_vte_background_type, "none")
		__radio_background_image.connect( \
			"toggled", self.on_changed_vte_background_type, "image-file")
		__radio_transparent.connect( \
			"toggled", self.on_changed_vte_background_type, "transparent")

	def in_activate_font (self) :
		# font #############################################
		self.getWidgetCurrent("checkbutton37").set_active( \
			self.GCONF_CLIENT.getValue("use_system_default_font") \
		)
		self.getWidgetCurrent("fontbutton1").set_font_name( \
			self.GCONF_CLIENT.getValue("font"))
		self.getWidgetCurrent("table79").set_property("sensitive", \
			not self.getWidgetCurrent("checkbutton37").get_active())

	def in_activate_color (self) :
		# USE SYSTEM THEME.
		__use_system_theme_color = \
			self.GCONF_CLIENT.getValue("use_system_theme_color", False)

		self.getWidgetCurrent("checkbutton39").set_active( \
			__use_system_theme_color)

		self.getWidgetCurrent("hbox125").set_property( \
			"sensitive", not __use_system_theme_color)
		self.getWidgetCurrent("table84").set_property( \
			"sensitive", not __use_system_theme_color)

		# FOREGROUND & BACKGROUND COLOR AS BUILT-IN SCHEMES TYPE
		__combobox_fgcolor_type = self.getWidgetCurrent("combobox2")
		[__combobox_fgcolor_type.append_text(_(i[1])) \
			for i in main.VTE_FGCOLOR_BUILTIN_SCHEMES_LIST]

		__foreground_background_color_builtin_schemes_type = \
			self.GCONF_CLIENT.getValue( \
				"foreground_background_color_builtin_schemes_type", "black_on_white")

		__idx = 0
		__schemes = main.VTE_FGCOLOR_BUILTIN_SCHEMES_LIST[__idx][0]
		for i in range(len(main.VTE_FGCOLOR_BUILTIN_SCHEMES_LIST)):
			if __foreground_background_color_builtin_schemes_type == \
				main.VTE_FGCOLOR_BUILTIN_SCHEMES_LIST[i][0] :
				__idx = i
				__schemes = main.VTE_FGCOLOR_BUILTIN_SCHEMES_LIST[i][0]
				break

		__combobox_fgcolor_type.set_active(__idx)

		__color_foreground = self.getWidgetCurrent("colorbutton21")
		__color_background = self.getWidgetCurrent("colorbutton22")

		if __foreground_background_color_builtin_schemes_type == \
				"custom" :
			__fg_color = self.GCONF_CLIENT.getValue("color_foreground")
			__bg_color = self.GCONF_CLIENT.getValue("color_background")
		else :
			(__fg_color, __bg_color, ) = main.VTE_FGCOLOR_BUILTIN_SCHEMES_LIST[__idx][2]

		__color_foreground.set_color(func.str_to_color(__fg_color))
		__color_background.set_color(func.str_to_color(__bg_color))

		# PALETTE
		__combobox_palette_select = self.getWidgetCurrent("combobox1")
		[__combobox_palette_select.append_text(_(i[1])) \
			for i in main.VTE_PALETTE_BUILTIN_SCHEMES_LIST]

		__color_palette_builtin_schemes_type = \
			self.GCONF_CLIENT.getValue("color_palette_builtin_schemes_type", "linux")

		__idx = 0
		__schemes = main.VTE_PALETTE_BUILTIN_SCHEMES_LIST[__idx][0]
		for i in range(len(main.VTE_PALETTE_BUILTIN_SCHEMES_LIST)):
			if __color_palette_builtin_schemes_type == \
					main.VTE_PALETTE_BUILTIN_SCHEMES_LIST[i][0] :
				__idx = i
				__schemes = main.VTE_PALETTE_BUILTIN_SCHEMES_LIST[i][0]
				break

		__combobox_palette_select.set_active(__idx)

		# if schemes is linux console
		if main.TERMINAL_PALETTE.has_key(__schemes) :
			for i in range(1, 17) :
				__button = self.getWidgetCurrent("colorbutton-picker-%s" % i)
				__button.set_color( \
						func.str_to_color(main.TERMINAL_PALETTE[__schemes][i - 1]) \
					)

				# disable 'color-set' signal.
				__button.connect("button-press-event", WIDGET.on_no_response)
				__button.connect("key-press-event", WIDGET.on_no_response)

	def in_activate_etc (self) :
		__enable_logging = self.GCONF_CLIENT.getValue("enable_logging")
		__log_directory = os.path.expanduser( \
			self.GCONF_CLIENT.getValue("log_directory") \
		)
		__enable_scrollbar = self.GCONF_CLIENT.getValue("enable_scrollbar")

		self.getWidgetCurrent("checkbutton36").set_active(__enable_logging)
		self.getWidgetCurrent("table78").set_property( \
			"sensitive", __enable_logging)
		if __log_directory :
			self.getWidgetCurrent("filechooserbutton19").set_current_folder( \
				__log_directory)

		self.getWidgetCurrent("spinbutton7").set_value( \
			int(self.GCONF_CLIENT.getValue("max_scroll_line")))

		self.getWidgetCurrent("checkbutton38").set_active(__enable_scrollbar)

	##################################################
	# EVENT HANDLER
	def on_font_set (self, fontbutton) :
		self.GCONF_CLIENT.setValue("font", fontbutton.get_font_name())

	def on_use_system_font (self, checkbutton) :
		__active = checkbutton.get_active()
		self.GCONF_CLIENT.setValue( \
			"use_system_default_font", __active \
		)

		self.getWidgetCurrent("table79").set_property( \
			"sensitive", not __active \
		)
		try :
			self.getWidgetCurrent("table79").set_property( \
				"sensitive", not __active \
			)
		except AttributeError :
			pass

	def on_changed_vte_background_type (self, radiobutton, which) :
		if which == "transparent" :
			self.getWidgetCurrent("vbox147").set_property("sensitive", True)
			self.getWidgetCurrent("hbox122").set_property("sensitive", False)
		elif which == "image-file" :
			#self.getWidgetCurrent("vbox147").set_property("sensitive", False)
			self.getWidgetCurrent("hbox122").set_property("sensitive", True)
		else : # none
			self.getWidgetCurrent("hbox122").set_property("sensitive", False)
			self.getWidgetCurrent("vbox147").set_property("sensitive", False)

		self.GCONF_CLIENT.setValue("vte_background_type", which)

	def on_changed_background_image_file (self, chooser) :
		__new_filename = chooser.get_filename()
		if not __new_filename : return

		self.GCONF_CLIENT.setValue("vte_background_type", "image-file")
		self.GCONF_CLIENT.setValue("vte_background_image_file", __new_filename)

	def on_changed_saturation (self, hscale) :
		__value = round(hscale.get_value(), 1)
		self.GCONF_CLIENT.setValue("vte_background_shade", __value)

	def on_changed_max_scroll_line (self, spin) :
		__value = spin.get_value()

		self.GCONF_CLIENT.setValue("max_scroll_line", int(__value))

	#def on_changed_color (self, button, which) :
	#	self.GCONF_CLIENT.setValue(which, func.color_to_str(button.get_color()))
	def on_color_fore_background_changed_builtin_schemes (self, combobox) :
		__idx = combobox.get_active()

		try :
			__schemes = main.VTE_FGCOLOR_BUILTIN_SCHEMES_LIST[__idx][0]
		except IndexError :
			__schemes = "black_on_white"

		__color_foreground = self.getWidgetCurrent("colorbutton21")
		__color_background = self.getWidgetCurrent("colorbutton22")

		if __schemes == "custom" :
			__fg_color = self.GCONF_CLIENT.getValue("color_foreground")
			__bg_color = self.GCONF_CLIENT.getValue("color_background")
		else :
			(__fg_color, __bg_color, ) = main.VTE_FGCOLOR_BUILTIN_SCHEMES_LIST[__idx][2]

		__color_foreground.set_color(func.str_to_color(__fg_color))
		__color_background.set_color(func.str_to_color(__bg_color))

		self.GCONF_CLIENT.setValue("color_foreground", __fg_color)
		self.GCONF_CLIENT.setValue("color_background", __bg_color)
		self.GCONF_CLIENT.setValue( \
			"foreground_background_color_builtin_schemes_type", __schemes)

	def on_color_foreground (self, button) :
		self.GCONF_CLIENT.setValue( \
			"color_foreground", func.color_to_str(button.get_color()))

	def on_color_background (self, button) :
		self.GCONF_CLIENT.setValue( \
			"color_background", func.color_to_str(button.get_color()))

	def on_color_use_system_theme (self, checkbutton) :
		self.getWidgetCurrent("hbox125").set_property( \
			"sensitive", not checkbutton.get_active())
		self.getWidgetCurrent("table84").set_property( \
			"sensitive", not checkbutton.get_active())

		self.GCONF_CLIENT.getValue("use_system_theme_color")
		self.GCONF_CLIENT.setValue( \
			"use_system_theme_color", checkbutton.get_active())

	def on_color_palette_changed_builtin_schemes (self, combobox) :
		__idx = combobox.get_active()
		try :
			__schemes = main.VTE_PALETTE_BUILTIN_SCHEMES_LIST[__idx][0]
		except IndexError :
			__schemes = "linux"

		if main.TERMINAL_PALETTE.has_key(__schemes) :
			for i in range(1, 17) :
				__button = self.getWidgetCurrent("colorbutton-picker-%s" % i)
				__button.set_color( \
						func.str_to_color(main.TERMINAL_PALETTE[__schemes][i - 1]) \
					)
				# disable 'color-set' signal.
				__button.connect("button-press-event", WIDGET.on_no_response)
				__button.connect("key-press-event", WIDGET.on_no_response)

		self.GCONF_CLIENT.setValue( \
			"color_palette_builtin_schemes_type", __schemes)

	def on_changed_enable_scrollbar (self, checkbutton) :
		__enable = checkbutton.get_active()
		self.GCONF_CLIENT.setValue("enable_scrollbar", __enable)

	def on_changed_enable_logging (self, checkbutton) :
		__enable = checkbutton.get_active()
		try :
			self.getWidgetCurrent("table61").set_property("sensitive", __enable)
		except AttributeError :
			pass

		self.GCONF_CLIENT.setValue("enable_logging", __enable)

	def on_changed_directory_logging (self, chooser) :
		__directory = chooser.get_current_folder()
		if not __directory : return

		self.GCONF_CLIENT.setValue("log_directory", __directory)

# menu
def show_menu (event, WIDGET_ROOT, callback_functions) :
	if event.button == 3 :
		__MENU__( \
			event, widget_root=WIDGET_ROOT, callback_functions=callback_functions)

class __MENU__ (WIDGET.WIDGET) :

	CALLBACK_FUNCTIONS = dict()

	def __init__ (self, event, **kwargs) :
		if not kwargs.has_key("widget_root") :
			raise ValueError, _("widget_root must be included.")
		else :
			__widget_root = kwargs["widget_root"]

		if not kwargs.has_key("callback_functions") :
			raise ValueError, _("callback_functions must be included.")
		else :
			self.CALLBACK_FUNCTIONS = kwargs["callback_functions"]

		WIDGET.WIDGET.__init__(self, "menu_vte", __widget_root)
		self.WIDGET_SELF.popup(None, None, None, event.button, event.time)

	def initialize (self) :

		##################################################
		# INPUT METHOD MODULES.
		(__xml, __menu_input_methods) = WIDGET.get_widget("menu_input_method")
		self.WIDGET_ROOT.im_append_menuitems(__menu_input_methods)

		__item_input_method = self.xml.get_widget("input_methods2")
		__item_input_method.set_submenu(__menu_input_methods)

		__menu = gtk.SeparatorMenuItem()
		__menu.show()
		self.WIDGET_SELF.prepend(__menu)

		__menu = gtk.ImageMenuItem(gtk.STOCK_PASTE)
		__menu.connect("activate", self.paste)
		__menu.show()
		self.WIDGET_SELF.prepend(__menu)

		__menu = gtk.ImageMenuItem(gtk.STOCK_COPY)
		__menu.connect("activate", self.copy)
		__menu.show()
		self.WIDGET_SELF.prepend(__menu)

	def copy (self, widget) :
		self.WIDGET_ROOT.copy_clipboard()
	def paste (self, widget) :
		self.WIDGET_ROOT.paste_clipboard()

####################################################################
# FUNCTIONS
def get_filename_logging (log_directory) :
	return os.path.join(log_directory, "%s.log" % time.strftime("%F.%T"))

"""
Description
-----------


ChangeLog
---------


Usage
-----


"""

__author__ =  "Spike^ekipS <spikeekips@gmail.com>"
__version__=  "0.1"
__nonsense__ = ""

__file__ = "__VTE__.py"


