# -*- coding: utf-8 -*-
#    Copyright (C) 2005 Spike^ekipS <spikeekips@gmail.com>
#
#    http://vee.kldp.net
#
#       This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import os, sys, glob
from vee import Conf, XMLHandler, __OS__

def __check_is_value_module (childname) :
	"""
	Check whether the child module property organized or not.
	"""
	__path_main = os.path.join(Conf.DIR_LIB, "child", childname, "main.py")
	__path_config_xml = \
		os.path.join(Conf.DIR_LIB, "child", childname, "config.xml")

	if not os.path.exists(__path_main) or not os.path.isfile(__path_main) :
		return False

	if not os.path.exists(__path_config_xml) or \
			not os.path.isfile(__path_config_xml) :
		return False

	return True

def load_module (childname, type) :
	"""
	import child module
		@return : instance
	"""

	if not __check_is_value_module(childname) : return None

	try :
		__module = \
			__import__("vee/child/%s/%s" % (childname, type, ), globals(), locals(), [type,])
	except ImportError, e:
		print "= ERROR MESSAGE =============================================="
		print e
		raise ValueError, \
			"\"%s\" is not a correctly written child. Please check it." % childname

	return __module

def load_class (module, type) :
	if not hasattr(module, type) :
		return None
	else :
		return getattr(module, type)

def get_child (childname, state, **kwargs) :
	"""
	get the child module
		@return : egg class
	"""

	__test = get_object_from_main(childname, "test")
	if __test and not __test(state, **kwargs) :
		return None

	__egg = load_class(load_module(childname, "egg"), "egg")
	if not __egg :
		return None
	else :
		return __egg(state, **kwargs)

def get_object_from_main (childname, objectname) :
	return load_class(load_module(childname, "main"), objectname)

def get_child_main (childname) :
	"""
	get the child main module
		@return : main class
	"""
	return load_class(load_module(childname, "main"), "main")

def get_child_dashboard (childname) :
	"""
	get the child dashboard module
		@return : dashboard class
	"""
	return load_class(load_module(childname, "main"), "dashboard")

def get_child_config (childname) :
	"""
	get the child config
		@return : dict()
	"""
	if not __check_is_value_module(childname) :
		return None

	__path_config_xml = \
		os.path.join(Conf.DIR_LIB, "child", childname, "config.xml")

	return XMLHandler.ChildConfig(__path_config_xml).getAll()

def get_children () :
	__path_child = "%s/*" % os.path.join(Conf.DIR_LIB, "child")

	__children = dict()
	for i in glob.glob(__path_child) :
		if not __OS__.is_dir(i) : continue
		__childname = os.path.basename(i)
		if __check_is_value_module(__childname) :
			__children[__childname] = get_child_config(__childname)

	return __children

def get_child_name_by_object (self) :
	return str(self.__module__).split(os.sep)[2]

def get_path_child (childname) :
	return os.path.join(Conf.DIR_LIB, "child", childname)

if __name__ == "__main__" :
	#print get_child("terminal")
	#print get_child_config("terminal")
	print get_children()

"""
Description
-----------


ChangeLog
---------


Usage
-----


"""

__author__ =  "Spike^ekipS <spikeekips@gmail.com>"
__version__=  "0.1"
__nonsense__ = ""

__file__ = "__init__.py"


