# -*- coding: utf-8 -*-
#	Copyright 2005 Spike^ekipS <spikeekips@gmail.com>
#
#	   This program is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; either version 2 of the License, or
#	(at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import os, struct, errno, stat, time, fcntl, tty, sre
import gtk

from twisted.internet import defer
from twisted.python import failure
from twisted.conch.ssh import transport, filetransfer, common
from twisted.conch.ssh.common import NS, getNS

from vee import __OS__
from vee.child.filebrowser_ssh import error

class FileTransferClient (filetransfer.FileTransferClient) :
	WINDOW = None
	CURRENT_DIRECTORY = None
	DIRECTORY_HOME = None

	def __init__ (self, window) :
		filetransfer.FileTransferClient.__init__(self)
		self.WINDOW = window

	def connectionMade (self) :
		filetransfer.FileTransferClient.connectionMade(self)

	def makeConnection(self, transport):
		filetransfer.FileTransferClient.makeConnection(self, transport)
		self.connected = True
		self.initialize()

		self.WINDOW.toggleConnected()
		self.WINDOW.connectionMade()

		print "========================================"
		print "connected"

	def initialize (self) :
		self.realPath(str()).addCallback(self._cb_initialize)

	def _cb_initialize (self, path) :
		self.DIRECTORY_HOME = path

	def readLink (self, path):
		d = self._sendRequest(filetransfer.FXP_READLINK, NS(path))
		d.addCallback(self._cbRealPath0, path)
		d.addErrback(self._ebReadLink, path)

		return d

	def _ebReadLink (self, f, path):
		return (path, None, None)

	def _cbRealPath0 (self, result, path):
		name, longname, attrs = result[0]

		# read attr of name
		d = self.getAttrs(name)
		return d.addCallback(self._cbRealPath1, name, path)

	def _cbRealPath1 (self, result, name, path) :
		return (path, name, result, )

class FileTransfer :
	"""
	Available commands :
	=====================
		cd path									Change remote directory to 'path'.
		chgrp gid path							Change gid of 'path' to 'gid'.
		chmod mode path						Change mode of 'path' to 'mode'.
		chown uid path							Change uid of 'path' to 'uid'.
		exit										Disconnect from the server.
		get remote-path [local-path]		Get remote file.
		help										Get a list of available commands.
		lcd path									Change local directory to 'path'.
		lls [ls-options] [path]				Display local directory listing.
		lmkdir path								Create local directory.
		ln linkpath targetpath				Symlink remote file.
		lpwd										Print the local working directory.
		ls [-l] [path]							Display remote directory listing.
		mkdir path								Create remote directory.
		progress									Toggle progress bar.
		put local-path [remote-path]		Put local file.
		pwd										Print the remote working directory.
		quit										Disconnect from the server.
		rename oldpath newpath				Rename remote file.
		rmdir path								Remove remote directory.
		rm path									Remove remote file.
		version									Print the SFTP version.
		?											Synonym for 'help'.
	"""

	CURRENT_DIRECTORY = ""
	TRANSFER = None
	WINDOW = None

	def __init__ (self, window) :
		self.WINDOW = window

		self.TRANSFER = FileTransferClient(window)

	def getClient (self) :
		return self.TRANSFER

	###########################################################
	# COMMAND method
	def getRealPath (self, path) :
		if path.find("~") > 0 :
			__path = path.split("~")[1]
			if __path.startswith("/") :
				__path = __path[1:]

			path = os.path.join(self.TRANSFER.DIRECTORY_HOME, __path)

		if not path.strip().startswith("/") :
				path = os.path.join(self.CURRENT_DIRECTORY, path.strip())

		return path

	def getAttrs (self, path):
		return self.TRANSFER.getAttrs(self.getRealPath(path))

	def readLink (self, linkpath) :
		d = self.TRANSFER.readLink(self.getRealPath(linkpath))
		#if d and isinstance(d, failure.Failure) :
		#	return None

		return d

	def cmd_CD (self, path=None) :
		if not path :
			return self.TRANSFER.realPath(str()).addCallback(self._cbCdNone)

		d = self.TRANSFER.openDirectory(self.getRealPath(path))
		d.addCallback(self._cbCd)
		d.addErrback(self._ebCd)

		return d

	def cmd_CHGRP(self, gid, path) :
		gid = int(gid)
		d = self.TRANSFER.getAttrs(self.getRealPath(path))
		d.addCallback(self._cbSetUsrGrp, path, grp=gid)
		return d

	def cmd_CHMOD (self, mod, path) :
		try :
			mod = int(mod, 8)
		except ValueError :
			self._printFailure(failure.Failure())

		d = self.TRANSFER.setAttrs( \
			self.getRealPath(path), {'permissions':mod} \
		)
		#d.addCallback(_ignore)
		return d

	def cmd_CHOWN(self, uid, path) :
		usr = int(uid)
		d = self.TRANSFER.getAttrs(self.getRealPath(path))
		d.addCallback(self._cbSetUsrGrp, path, usr=uid)
		return d

	def cmd_GET(self, remotefilepath, localfilepath, ProgressBar=None) :

		if os.path.isdir(localfilepath) :
			localfilepath = os.path.join( \
				localfilepath,
				os.path.basename(remotefilepath) \
			)

		try :
			#lf = file(localfilepath, 'w', 0)
			lf = DummyFileRemote(localfilepath)
		except IOError :
			return failure.Failure()

		remotefilepath = self.getRealPath(remotefilepath)

		d = self.TRANSFER.openFile( \
			remotefilepath, filetransfer.FXF_READ, {} \
		)
		d.addCallback(self._cbGetOpenFile, lf, ProgressBar)
		d.addErrback(self._ebCloseLf, lf)
		return d

	def cmd_PUT(self, localfilepath, remotefilepath, ProgressBar=None) :

		#lf = file(localfilepath, 'r')
		lf = DummyFileLocal(localfilepath)

		d = self.TRANSFER.openFile(remotefilepath, FXF_WRITE | FXF_CREAT, {})

		d.addCallback(self._cbPutOpenFile, lf, ProgressBar)
		d.addErrback(self._ebCloseLf, lf)
		return d

	def cmd_LN(self, source, target) :
		linkpath, targetpath = \
			map(lambda x: \
				os.path.join(self.CURRENT_DIRECTORY, x), \
				(source, target) \
			)

		#return self.TRANSFER.makeLink( \
		#	linkpath, targetpath).addCallback(_ignore)
		return self.TRANSFER.makeLink(linkpath, targetpath)

	def cmd_LS_RECURSIVE (self, path=None) :
		if not path :
			path = self.CURRENT_DIRECTORY

		__dump_filelist = dict()
		d = self._remoteGlob(self.getRealPath(path))
		d.addCallback(self._cbGetFileList, __dump_filelist, path)

		return d

	def _cbGetFileList (self, filelist, dump_filelist, path_parent) :
		__list = [i for i in filelist if i[0] not in [".", ".."]]
		__list = [list(i) for i in __list]
		[i.__setitem__(0, os.path.join(path_parent, i[0])) for i in __list]

		dump_filelist[path_parent] = __list

		dList = list()
		for i in __list :
			if not __OS__.is_directory_by_st_mode(i[2]["permissions"]) :
				continue

			__path = os.path.join(path_parent, i[0])
			d = self._remoteGlob(__path)
			d.addCallback(self._cbGetFileList, dump_filelist, __path)

			dList.append(d)

		if len(dList) < 1 :
			return dump_filelist
		else :
			dl = defer.DeferredList(dList, fireOnOneErrback=1)
			dl.addCallback(self._cbGetFileListDone, dump_filelist)

			return dl

	def _cbGetFileListDone (self, ignored, dump_filelist) :
		return dump_filelist

	def cmd_LS(self, path=None) :
		if not path :
			path = self.CURRENT_DIRECTORY

		d = self._remoteGlob(self.getRealPath(path))
		d.addCallback(self._cbReturnFiles)

		return d

	def cmd_MKDIR(self, path) :
		#return self.TRANSFER.makeDirectory( \
		#	self.getRealPath(path), {}).addCallback(_ignore)
		return self.TRANSFER.makeDirectory(path, {})
			#self.getRealPath(path), {})

	def cmd_RMDIR(self, path) :
		return self.TRANSFER.removeDirectory(self.getRealPath(path))

	def cmd_RM(self, path) :
		return self.TRANSFER.removeFile(self.getRealPath(path))

	def cmd_RENAME(self, old, new) :
		return self.TRANSFER.renameFile(old, new)

	def cmd_EXIT(self, ignored) :
		self.TRANSFER.transport.loseConnection()

	cmd_QUIT = cmd_EXIT

	def cmd_VERSION(self, ignored) :
		return "SFTP version %i" % self.TRANSFER.version

	def cmd_PWD(self, ignored) :
		return self.CURRENT_DIRECTORY

	def cmd_EXEC(self, rest) :
		shell = pwd.getpwnam(getpass.getuser())[6]

		if rest :
			cmds = ['-c', rest]
			return utils.getProcessOutput(shell, cmds, errortoo=1)
		else :
			os.system(shell)

	#########################################################################
	# MISC handler
	def _cbCdNone (self, path) :
		return self.cmd_CD(path)

	def _ebCd (self, f) :
		self.WINDOW.writeLog("%s\n" % f.value, "remote", "error")

		return f

	def _cbCd (self, directory) :

		directory.close()
		d = self.TRANSFER.realPath(directory.name)
		d.addCallback(self._cbCurDir)
		return d

	def _cbCurDir(self, path) :
		self.CURRENT_DIRECTORY = path

		return path

	def _cbSetUsrGrp(self, attrs, path, usr=None, grp=None) :
		new = {}
		new['uid'] = (usr is not None) and usr or attrs['uid']
		new['gid'] = (grp is not None) and grp or attrs['gid']
		d = self.TRANSFER.setAttrs(path, new)
		#d.addCallback(_ignore)
		return d

	def _ebCloseLf(self, f, lf) :
		lf.close("Don't swap")
		return f

	def _cbGetOpenFile(self, rf, lf, ProgressBar=None) :
		return rf.getAttrs().addCallback(self._cbGetFileSize, rf, lf, ProgressBar)

	def _cbGetFileSize(self, attrs, rf, lf, ProgressBar=None) :
		if not stat.S_ISREG(attrs['permissions']) :
			rf.close()
			lf.close("remove it!")
			return "Can't get non-regular file: %s" % rf.name

		rf.size = attrs['size']

		bufferSize = \
			self.TRANSFER.transport.conn.CONF["options"]['buffersize']
		numRequests = \
			self.TRANSFER.transport.conn.CONF["options"]['requests']

		rf.total = 0.0

		dList = []
		chunks = []
		startTime = time.time()
		ProgressBar.starting(lf, rf, startTime)

		for i in range(numRequests) :
			d = self._cbGetRead('', rf, lf, chunks, 0, bufferSize, ProgressBar)
			dList.append(d)

		dl = defer.DeferredList(dList, fireOnOneErrback=1)
		dl.addCallback(self._cbGetDone, rf, lf)
		return dl

	def _getNextChunk(self, chunks) :
		end = 0
		for chunk in chunks :
			if end == 'eof':
				return # nothing more to get

			if end != chunk[0]:
				i = chunks.index(chunk)
				chunks.insert(i, (end, chunk[0]))
				return (end, chunk[0] - end)

			end = chunk[1]

		bufSize = int( \
			self.TRANSFER.transport.conn.CONF["options"]['buffersize'])
		chunks.append((end, end + bufSize))
		return (end, bufSize)

	IS_STOP_TRANSFER = False
	def _cbGetRead (self, data, rf, lf, chunks, start, size, progressBar=None) :
		if self.IS_STOP_TRANSFER :
			self._cbStopTransfer(rf, lf)
			#raise RuntimeError, "stop this operation."
			if progressBar : progressBar.stop()
			return

		if lf is None :
			if progressBar : progressBar.stop()
			return

		if data and isinstance(data, failure.Failure) :
			reason = data
			reason.trap(EOFError)
			i = chunks.index((start, start + size))
			del chunks[i]
			chunks.insert(i, (start, 'eof'))
		elif data :
			time.sleep(0.001)
			lf.seek(start)
			lf.write(data)

			############################
			# NOTE :
			#  In twisted.conch.ssh.filetransfer, openFile and
			# ClientFile.readChunk() can not handle properly Big size file, so i
			# added this routine and sys.setrecursionlimit(1000*10) for preventing
			# "max recursion depth excedded" error.
			if rf.size > 1000000 :
				time.sleep(0.01)

			if len(data) != size :
				i = chunks.index((start, start + size))
				del chunks[i]
				chunks.insert(i, (start, start + len(data)))

			rf.total += len(data)

		chunk = self._getNextChunk(chunks)
		if not chunk :
			return
		else :
			start, length = chunk

		d = rf.readChunk(start, length)
		d.addBoth( \
			self._cbGetRead, rf, lf, chunks, start, length, progressBar)

		return d

	def stopTransfer (self) :
		self.IS_STOP_TRANSFER = True

	def _cbStopTransfer (self, rf, lf) :
		self.IS_STOP_TRANSFER = False
		rf.close()
		lf.close()

	def _cbGetDone(self, ignored, rf, lf) :
		rf.close()
		lf.close()

		# refresh local file tree.
		if os.path.abspath(os.path.dirname(lf.NAME_REAL)) == \
			os.path.abspath(self.TRANSFER.WINDOW.CURRENT_DIRECTORY_LOCAL) :
			self.WINDOW.listAllLocal()

		return "Transferred %s to %s" % (rf.name, lf.name)

	def _remoteGlob(self, fullPath) :
		d = self.TRANSFER.openDirectory(fullPath)
		d.addCallback(self._cbOpenList)

		return d

	def _cbReadFiles (self, result, fullpath, files) :
		dList = []
		for i in result :
			files.extend([os.path.join(fullpath, data[0]), ] + list(i)[1:])
			d = self._cbGetDirectory(i, fullpath, files)
			dList.append(d)

		dl = defer.DeferredList(dList)
		dl.addCallback(self._cbReadFiles, rf, lf)
		return dl

	def _cbOpenList(self, directory) :
		files = []
		d = directory.read()
		d.addBoth(self._cbReadFile, files, directory)
		return d

	def _ebNotADirectory(self, reason, path, glob) :
		d = self.TRANSFER.openDirectory(path)
		d.addCallback(self._cbOpenList, glob)
		return d

	def _cbReadFile(self, files, l, directory) :
		if not isinstance(files, failure.Failure) :
			l.extend(files)

			d = directory.read()
			d.addBoth(self._cbReadFile, l, directory)
			return d
		else :
			reason = files
			reason.trap(EOFError)
			directory.close()
			return l

	def _cbPutOpenFile(self, rf, lf, progressBar=None) :

		startTime = time.time()
		progressBar.starting(lf, rf, startTime)

		numRequests = self.TRANSFER.transport.conn.CONF["options"]['requests']
		dList = []
		chunks = []
		for i in range(numRequests) :
			d = self._cbPutWrite(None, rf, lf, chunks, progressBar)
			if d :
				dList.append(d)

		dl = defer.DeferredList(dList, fireOnOneErrback=1)
		dl.addCallback(self._cbPutDone, rf, lf, progressBar)
		return dl

	def _cbPutWrite(self, ignored, rf, lf, chunks, progressBar=None) :
		chunk = self._getNextChunk(chunks)
		start, size = chunk
		lf.seek(start)
		data = lf.read(size)
		lf.total += len(data)

		if not data or self.IS_STOP_TRANSFER:
			return
		else :
			d = rf.writeChunk(start, data)
			time.sleep(0.001)
			d.addCallback(self._cbPutWrite, rf, lf, chunks, progressBar)

			return d

	def _cbPutDone(self, ignored, rf, lf, progressBar) :
		lf.close()
		rf.close()

		if self.IS_STOP_TRANSFER :
			progressBar.stop()
			self.IS_STOP_TRANSFER = False

		return 'Transferred %s to %s' % (lf.name, rf.name)

	def _cbReturnFiles (self, files) :
		__list = [i for i in files if i[0] not in [".", ".."]]

		__link_list = list()
		__not_link_list = list()
		for i in __list :
			if __OS__.is_link_by_st_mode(i[2]["permissions"]) :
				__link_list.append(i)
			else :
				__not_link_list.append(i)

		dList = []
		for i in __link_list :
			d = self.readLink(self.getRealPath(i[0]))
			if d :
				dList.append(d)

		dl = defer.DeferredList(dList)
		dl.addCallback(self._cbReturnFiles0, __link_list, __not_link_list)

		return dl

	def _cbReturnFiles0 (self, result, link_list, not_link_list) :

		d = dict()
		[d.update({os.path.basename(i[1][0]) : i[1][1:]}, ) \
			for i in result if len(i) > 1 and i[0] and i[1] and i[1][2]]

		# findout broken link.
		__link_broken = [i for i in result if len(i) > 1 and not i[1][2]]

		[i[2].update({"target" : d[i[0]], }) for i in link_list if d.has_key(i[0])]
		[i[2].update({"target" : tuple(), }) for i in link_list if not d.has_key(i[0])]
		[i[2].update({"target" : tuple(), }) for i in not_link_list]

		del d

		return link_list + not_link_list

	def _cbReadLinkDone (self, filelist) :
		return filelist

	def _cbDisplayFiles(self, files) :
		files.sort()

		files = [f for f in files if not f[0].startswith('.')]
		lines = [f[1] for f in files]

		if not lines :
			return None
		else :
			return '\n'.join(lines)

	def _printFailure(self, f):
		e = f.trap( \
			NotImplementedError,\
			filetransfer.SFTPError, \
			OSError, \
			IOError \
		)

		if e == NotImplementedError :
			self.transport.write(self.cmd_HELP(''))
		elif e == filetransfer.SFTPError :
			self.transport.write( \
				"remote error %i: %s\n" % (f.value.code, f.value.message))
		elif e in (OSError, IOError) :
			self.transport.write( \
				"local error %i: %s\n" % (f.value.errno, f.value.strerror))

import tempfile, shutil, os
class DummyFileLocal (file) :
	total = 0
	size = 0
	def __init__ (self, filename, mode="r", buffering=0) :
		file.__init__(self, filename, mode, buffering)
		self.size = os.fstat(self.fileno())[6]

	def close (self, isRemove=False) :
		file.close(self)

class DummyFileRemote (file) :
	"""
	store data in temporary file.
	"""

	NAME_REAL = ""
	NAME_TEMPORARY = ""

	def __init__ (self, filename) :

		self.NAME_REAL = filename

		self.NAME_TEMPORARY = \
			tempfile.mktemp(prefix="vee-scpfile-", suffix=".tmp")

		file.__init__(self, self.NAME_TEMPORARY, "w")

	def close (self, isRemove=False) :
		file.close(self)
		if isRemove :
			os.remove(self.NAME_TEMPORARY)
		else :
			self.swap()

	def swap (self) :
		shutil.move(self.NAME_TEMPORARY, self.NAME_REAL)
		#shutil.copy(self.NAME_TEMPORARY, self.NAME_REAL)

def _ignore (*args) : pass

"""
DATA
"""
FILEXFER_ATTR_ACMODTIME   = filetransfer.FILEXFER_ATTR_ACMODTIME
FILEXFER_ATTR_EXTENDED    = filetransfer.FILEXFER_ATTR_EXTENDED
FILEXFER_ATTR_OWNERGROUP  = filetransfer.FILEXFER_ATTR_OWNERGROUP
FILEXFER_ATTR_PERMISSIONS = filetransfer.FILEXFER_ATTR_PERMISSIONS
FILEXFER_ATTR_SIZE        = filetransfer.FILEXFER_ATTR_SIZE
FILEXFER_TYPE_DIRECTORY   = filetransfer.FILEXFER_TYPE_DIRECTORY
FILEXFER_TYPE_REGULAR     = filetransfer.FILEXFER_TYPE_REGULAR
FILEXFER_TYPE_SPECIAL     = filetransfer.FILEXFER_TYPE_SPECIAL
FILEXFER_TYPE_SYMLINK     = filetransfer.FILEXFER_TYPE_SYMLINK
FILEXFER_TYPE_UNKNOWN     = filetransfer.FILEXFER_TYPE_UNKNOWN
FXF_APPEND                = filetransfer.FXF_APPEND
FXF_CREAT                 = filetransfer.FXF_CREAT
FXF_EXCL                  = filetransfer.FXF_EXCL
FXF_READ                  = filetransfer.FXF_READ
FXF_TEXT                  = filetransfer.FXF_TEXT
FXF_TRUNC                 = filetransfer.FXF_TRUNC
FXF_WRITE                 = filetransfer.FXF_WRITE
FXP_ATTRS                 = filetransfer.FXP_ATTRS
FXP_CLOSE                 = filetransfer.FXP_CLOSE
FXP_DATA                  = filetransfer.FXP_DATA
FXP_EXTENDED              = filetransfer.FXP_EXTENDED
FXP_EXTENDED_REPLY        = filetransfer.FXP_EXTENDED_REPLY
FXP_FSETSTAT              = filetransfer.FXP_FSETSTAT
FXP_FSTAT                 = filetransfer.FXP_FSTAT
FXP_HANDLE                = filetransfer.FXP_HANDLE
FXP_INIT                  = filetransfer.FXP_INIT
FXP_LSTAT                 = filetransfer.FXP_LSTAT
FXP_MKDIR                 = filetransfer.FXP_MKDIR
FXP_NAME                  = filetransfer.FXP_NAME
FXP_OPEN                  = filetransfer.FXP_OPEN
FXP_OPENDIR               = filetransfer.FXP_OPENDIR
FXP_READ                  = filetransfer.FXP_READ
FXP_READDIR               = filetransfer.FXP_READDIR
FXP_READLINK              = filetransfer.FXP_READLINK
FXP_REALPATH              = filetransfer.FXP_REALPATH
FXP_REMOVE                = filetransfer.FXP_REMOVE
FXP_RENAME                = filetransfer.FXP_RENAME
FXP_RMDIR                 = filetransfer.FXP_RMDIR
FXP_SETSTAT               = filetransfer.FXP_SETSTAT
FXP_STAT                  = filetransfer.FXP_STAT
FXP_STATUS                = filetransfer.FXP_STATUS
FXP_SYMLINK               = filetransfer.FXP_SYMLINK
FXP_VERSION               = filetransfer.FXP_VERSION
FXP_WRITE                 = filetransfer.FXP_WRITE
FX_BAD_MESSAGE            = filetransfer.FX_BAD_MESSAGE
FX_CONNECTION_LOST        = filetransfer.FX_CONNECTION_LOST
FX_EOF                    = filetransfer.FX_EOF
FX_FAILURE                = filetransfer.FX_FAILURE
FX_NO_CONNECTION          = filetransfer.FX_NO_CONNECTION
FX_NO_SUCH_FILE           = filetransfer.FX_NO_SUCH_FILE
FX_OK                     = filetransfer.FX_OK
FX_OP_UNSUPPORTED         = filetransfer.FX_OP_UNSUPPORTED
FX_PERMISSION_DENIED      = filetransfer.FX_PERMISSION_DENIED


class TransferFile :

	FILEBROWSER = None

	def __init__ (self, filebrowser) :
		self.FILEBROWSER = filebrowser

	def printError (self, f) :
		self.FILEBROWSER.errBackRemote(f)

	def get (self, remotepath, dest) :
		d = self.FILEBROWSER.FILE_TRANSFER.getAttrs(remotepath)
		d.addCallback(self.__get, remotepath, dest)
		d.addErrback(self.printError)

	def __get (self, fileattr, remotepath, dest) :
		if dest.strip() != "/" and dest.endswith("/") :
			dest = dest[:-1]
		if remotepath.endswith("/") :
			remotepath = remotepath[:-1]

		localpath = os.path.join(dest, os.path.basename(remotepath))
		if __OS__.is_directory_by_st_mode(fileattr["permissions"]) : # is directory.
			# create directory 1st.
			__return = __OS__.mkdir(localpath)
			if __return == __OS__.ERROR_NO_PERMISSION :
				return failure.Failure(error.PermissionError())

			if os.path.abspath(os.path.dirname(localpath)) == \
				os.path.abspath(self.FILEBROWSER.CURRENT_DIRECTORY_LOCAL) :
				self.FILEBROWSER.listAllLocal()
			self.__getDirectory(remotepath, localpath)
		else : # is file.
			self.__getFile(fileattr, remotepath, localpath)
		return

	def __getFile (self, fileattr, remotepath, localpath) :
			self.FILEBROWSER.PANEL_FILETRANSFER_STATUS.addRow( \
				**{ \
					"filepath" : remotepath, \
					"filesize" : fileattr["size"], \
					"method" : "get", \
					"filepath-local" : localpath, \
					"filepath-remote" : remotepath, \
				} \
			)

	def __getDirectory (self, remotepath, dest) :
		d = self.FILEBROWSER.FILE_TRANSFER.cmd_LS(remotepath)
		d.addCallback(self.__getDirectory0, remotepath, dest)

	def __getDirectory0 (self, filelist, remotepath, dest) :
		for i in filelist :
			__localpath = os.path.join(dest, i[0])
			__remotepath = os.path.join(remotepath, i[0])

			if __OS__.is_directory_by_st_mode(i[2]["permissions"]) : # is directory
				__return = __OS__.mkdir(__localpath)

				if __return == __OS__.ERROR_NO_PERMISSION :
					return failure.Failure(error.PermissionError())

				if os.path.abspath(os.path.dirname(__localpath)) == \
					os.path.abspath(self.FILEBROWSER.CURRENT_DIRECTORY_LOCAL) :
					self.FILEBROWSER.listAllLocal()

				self.__getDirectory(__remotepath , __localpath)
			else : # is file
				self.__getFile(i[2], __remotepath, __localpath)

	def put (self, localpath, dest) :
		if dest.strip() != "/" and dest.endswith("/") :
			dest = dest[:-1]

		if localpath.endswith("/") :
			localpath = remotepath[:-1]

		remotepath = os.path.join(dest, os.path.basename(localpath))
		if __OS__.is_dir(localpath) : # is directory
			d = self.FILEBROWSER.FILE_TRANSFER.cmd_MKDIR(remotepath)
			d.addCallback(self.__putDirectory, localpath, remotepath, True)
			d.addErrback(self.FILEBROWSER.ignore)
		else : # is file
			self.__putFile(localpath, remotepath)
		return True

	def __putFile (self, localpath, remotepath) :
		self.FILEBROWSER.PANEL_FILETRANSFER_STATUS.addRow( \
			**{ \
				"filepath" : localpath, \
				"filesize" : int(os.stat(localpath)[6]), \
				"method" : "put", \
				"filepath-local" : localpath, \
				"filepath-remote" : remotepath, \
			} \
		)

		return True

	def __putDirectory (self, result, localpath, dest, is_refresh=False) :
		if is_refresh :
			self.FILEBROWSER.listAllRemote()

		for i in os.listdir(localpath) :
			__localpath = os.path.join(localpath, i)
			__remotepath = os.path.join(dest, os.path.basename(__localpath))

			if __OS__.is_dir(__localpath) : # is directory
				d = self.FILEBROWSER.FILE_TRANSFER.cmd_MKDIR(__remotepath)
				d.addCallback(self.__putDirectory, __localpath, __remotepath)
				d.addErrback(self.FILEBROWSER.ignore)
			else : # is file
				self.__putFile(__localpath, __remotepath)

		return True

	def remove (self, remotepath, recursive=False) :
		# find out whether is file or directory.
		d = self.FILEBROWSER.FILE_TRANSFER.getAttrs(remotepath)
		d.addCallback(self.__remove, remotepath)
		#d.addErrback(self.FILEBROWSER.errBackRemote, "remove")

		return d

	def __remove (self, fileattr, remotepath) :
		if __OS__.is_directory_by_st_mode(fileattr["permissions"]) : # is directory
			d = self.__getFileList(remotepath)
		else : # is file
			d = self.__removeFile(remotepath)

		d.addCallback(self._cb_refreshPanel)

		return d

	def __removeFile (self, remotepath) :
		d = self.FILEBROWSER.FILE_TRANSFER.cmd_RM(remotepath)
		d.addCallback(self._cb_removeDone)
		d.addErrback(self.FILEBROWSER.errBackRemote, "remove")

		return d

	def __removeDirectory (self, remotepath) :
		d = self.FILEBROWSER.FILE_TRANSFER.cmd_RMDIR(remotepath)
		d.addCallback(self._cb_removeDone)

		return d

	def __getFileList (self, remotepath) :
		d = self.FILEBROWSER.FILE_TRANSFER.cmd_LS_RECURSIVE(remotepath)
		d.addCallback(self._cb_removeDirectory, remotepath)

		return d

	def _cb_removeDirectory (self, filelist, remotepath) :
		__list_directory = filelist.keys()
		__list_directory.sort()
		__list_directory.reverse()

		for i in __list_directory :
			# remove file first.
			for j in filelist[i] :
				self.__removeFile(j[0])

			# and remove this directory.
			self.__removeDirectory(i)

		return True

	def _cb_removeDone (self, result) :
		return True

	def _cb_refreshPanel (self, result) :
		self.FILEBROWSER.listAllRemote()

###################################################
# FUNCTIONS
def is_directory_by_fileattr (fileattr) :
	"""
	This functions check whether this file is directory or link to directory.
	"""
	if __OS__.is_directory_by_st_mode(fileattr[2]["permissions"]) :
		return True
	elif __OS__.is_link_by_st_mode(fileattr[2]["permissions"]) :
		if len(fileattr[2]["target"]) > 1 :
			if __OS__.is_directory_by_st_mode(fileattr[2]["target"][1]["permissions"]) :
				return True

	return False


"""
Description
-----------


ChangeLog
---------


Usage
-----


"""

__author__ =  "Spike^ekipS <spikeekips@gmail.com>"
__version__=  "0.1"
__nonsense__ = ""

__file__ = "FileTransfer.py"


