# -*- coding: utf-8 -*-
#	Copyright 2005 Spike^ekipS <spikeekips@gmail.com>
#
#	   This program is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; either version 2 of the License, or
#	(at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import gobject, gtk, exceptions, time, threading
from twisted.internet import defer
from twisted.python import failure
from twisted.conch.ssh import filetransfer
from twisted.internet import threads

from vee.child.filebrowser_ssh import Progress

class SSH :

	"""
	Scenario is,

		. transfer request is given.
		. append the request into 'QUEUE'

		. check whether there are the request in the QUEUE.
		. if QUEUE does not empty,
			. get the first reqeust in QUEUE
			. transfering the request
			. if it is failed,
				. alarm this request was failed.
				. go next reqeust
			. if it's done successfully,
				. go next request
		. if QUEUE is empty
			. sleep
	"""

	QUEUE = list()
	DATA = dict()
	TREESTORE = None
	FILE_TRANSFER = None
	HANDLER_CHECK_QUEUE = None

	def __init__ (self, filetransfer) :
		self.FILEBROWSER = filetransfer

	def append (self, uid, **kwargs) :
		if not kwargs.has_key("filepath-remote") or \
				not kwargs.has_key("filepath-local") or \
				not kwargs.has_key("method") or \
				not kwargs.has_key("iter") :
			raise RuntimeError, \
				"filepath-remote, filepath-local, iter must be given."

		self.DATA[uid] = kwargs
		self.QUEUE.append(uid)

	def startRunning (self) :
		self.checkQueue()

	def checkQueue (self) :
		self.HANDLER_CHECK_QUEUE = gobject.timeout_add(1000, self.__checkQueue)

	def __checkQueue (self) :
		if len(self.QUEUE) < 1 :
			return True

		self.transfer()
		return False

	def restartCheckQueue (self) :
		# stop the queue and start again.
		gobject.source_remove(self.HANDLER_CHECK_QUEUE)
		self.checkQueue()

	def transfer0 (self) :
		time.sleep(0.5)

		try :
			uid = self.QUEUE[0]
		except IndexError :
			self.QUEUE = list()
			self.restartCheckQueue()
			return

	def transfer (self) :
		gtk.gdk.threads_enter()
		commands = [(self.__transfer, [], {})]
		threads.callMultipleInThread(commands)
		gtk.gdk.threads_leave()

	def on_finished (self, result) :
		try :
			del(self.QUEUE[0])
		except :
			pass

		self.transfer()

	def on_error (self, f) :
		try :
			del self.QUEUE[0]
		except IndexError :
			pass

		self.FILEBROWSER.errBackRemote(f, "transfer")
		self.transfer()

	def stop (self, uid) :
		try :
			__idx = self.QUEUE.index(uid)
		except ValueError, e :
			pass
		else :
			del(self.QUEUE[__idx])
			if __idx == 0 :
				print "==========================> stop"
				self.FILEBROWSER.FILE_TRANSFER.stopTransfer()

		self.transfer()

	def __transfer (self) :
		time.sleep(0.5)

		try :
			uid = self.QUEUE[0]
		except IndexError :
			self.QUEUE = list()
			self.restartCheckQueue()
			return

		__progress = Progress.SSH(self.DATA[uid]["method"], \
			self.TREESTORE, self.DATA[uid]["iter"], self.FILEBROWSER)

		if self.DATA[uid]["method"] == "get" :
			__filepath_local = self.DATA[uid]["filepath-local"]
		else :
			__filepath_local = \
				self.FILEBROWSER.getRealPathLocal( \
					self.DATA[uid]["filepath-local"], "is-file" \
				)

		__filepath_remote = \
			self.FILEBROWSER.getRealPathRemote( \
				self.DATA[uid]["filepath-remote"], "is-file" \
			)

		if self.DATA[uid]["method"] == "get" :
			d = self.FILEBROWSER.FILE_TRANSFER.cmd_GET( \
				__filepath_remote, __filepath_local, __progress)
		elif self.DATA[uid]["method"] == "put" :
			d = self.FILEBROWSER.FILE_TRANSFER.cmd_PUT( \
				__filepath_local,__filepath_remote, __progress)

		d.addCallback(self.on_finished)
		d.addErrback(self.on_error)

		return d

"""
Description
-----------


ChangeLog
---------


Usage
-----


"""

__author__ =  "Spike^ekipS <spikeekips@gmail.com>"
__version__=  "0.1"
__nonsense__ = ""

__file__ = "FileTransferQueue.py"


