# -*- coding: utf-8 -*-
#	Copyright 2005 Spike^ekipS <spikeekips@gmail.com>
#
#	   This program is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; either version 2 of the License, or
#	(at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import sre, tempfile, time, os, struct, tty, fcntl, stat, shutil
import gtk, gobject, pango, gnomevfs
from twisted.internet import defer
from twisted.python import failure

from vee import Handler, WIDGET, func as func_global, __TREEVIEW__, \
	GNOME, __XML__, __OS__, i18n, Window, Widgets, __VEE__, Conf__
from vee.child.filebrowser_ssh import main, XMLHandler, FileTransfer, func
from vee.child.ssh import main as main_ssh

_ = i18n.gettext()

class TreeViewFileList \
		(__TREEVIEW__.TreeViewWithMultipleItemSelectionDND, \
			__TREEVIEW__.Core, Widgets.WithGNOME) :

	FILEBROWSER = None
	AVAILABLE_KEY = list()

	PROPERTIES = { \
		"enable-search" : True, \
		"fixed-height-mode" : False, \
		"headers-clickable" : True, \
		"headers-visible" : True, \
		"hover-expand" : False, \
		"hover-selection" : False, \
		"reorderable" : True, \
		"rules-hint" : True, \
	}

	def __init__ (self, type, xml, **kwargs) :
		if not kwargs.has_key("filebrowser") :
			raise RuntimeError, "filebrowser was missing."
		else :
			self.FILEBROWSER = kwargs["filebrowser"]

		Widgets.WithGNOME.__init__(self)
		self.generateIcon()

		__TREEVIEW__.Core.__init__(self, xml, "treeview-%s" % type)
		__TREEVIEW__.TreeViewWithMultipleItemSelectionDND.__init__(self)

		# parse keyboard shortcut for popup menu
		for i, j in main.MENU_ACCELERATOR_SSH_FILEBROWSER_FILELIST.items() :
			self.AVAILABLE_KEY.append(j)

	def addRows (self, result) :
		result = self.reorderDirectoryFirst(result)
		for i in result :
			self.addRow(i)

	def set_editable (self, iter) :
		__path = self.get_path(iter)

		__column = self.WIDGET_TREEVIEW.get_column(0)
		cell = __column.get_cell_renderers()[1]
		cell.set_property("editable", True)

		self.WIDGET_TREEVIEW.set_cursor(__path, __column, True)

	def getPermissions (self, iter) :
		if not iter : return None

		return self.WIDGET_TREESTORE.get_value(iter, \
			main.TREEVIEW_FILELIST_PANEL_COLUMN_PERMISSION_BIN)

	def getName (self, iter) :
		if not iter : return None

		return self.WIDGET_TREESTORE.get_value(iter, \
			main.TREEVIEW_FILELIST_PANEL_COLUMN_NAME)

	def create_frame (self) :
		# create a CellRenderers to render the data
		__cell1 = gtk.CellRendererPixbuf()
		__cell2 = gtk.CellRendererText()
		__cell3 = gtk.CellRendererText()
		__cell4 = gtk.CellRendererText()
		__cell5 = gtk.CellRendererText()

		__cell2.connect("edited", self.on_rename_edited)
		__cell2.connect("editing-canceled", self.on_editing_canceled)

		# set property.
		__cell1.set_property( \
			"width", self.ICONS["__default__"].get_property("width"))
		__cell1.set_property("xalign", 0.5)
		__cell2.set_property("xalign", 0.0)
		__cell3.set_property("xalign", 1.0)
		__cell4.set_property("xalign", 0.0)
		__cell5.set_property("xalign", 0.0)

		self.tvcolumn1 = gtk.TreeViewColumn(_("Name"))
		self.tvcolumn2 = gtk.TreeViewColumn(_("Size"))
		self.tvcolumn3 = gtk.TreeViewColumn(_("Permission"))
		self.tvcolumn4 = gtk.TreeViewColumn(_("Modified Time"))
		self.tvcolumn1.set_expand(True)

		self.WIDGET_TREESTORE = gtk.ListStore( \
			gobject.TYPE_OBJECT, gobject.TYPE_STRING, \
			gobject.TYPE_STRING, gobject.TYPE_STRING, \
			gobject.TYPE_STRING, gobject.TYPE_STRING, \
			gobject.TYPE_STRING, object, \
		)

		self.WIDGET_TREEVIEW.set_model(self.WIDGET_TREESTORE)

		# add columns to treeview
		self.WIDGET_TREEVIEW.append_column(self.tvcolumn1)
		self.WIDGET_TREEVIEW.append_column(self.tvcolumn2)
		self.WIDGET_TREEVIEW.append_column(self.tvcolumn3)
		self.WIDGET_TREEVIEW.append_column(self.tvcolumn4)

		self.tvcolumn1.pack_start(__cell1, False)
		self.tvcolumn1.pack_start(__cell2, False)
		self.tvcolumn2.pack_start(__cell3, False)
		self.tvcolumn3.pack_start(__cell4, False)
		self.tvcolumn4.pack_start(__cell5, False)

		self.tvcolumn1.set_attributes( \
			__cell1, pixbuf=main.TREEVIEW_FILELIST_PANEL_COLUMN_ICON)
		self.tvcolumn1.set_attributes( \
			__cell2, text=main.TREEVIEW_FILELIST_PANEL_COLUMN_NAME)
		self.tvcolumn2.set_attributes( \
			__cell3, text=main.TREEVIEW_FILELIST_PANEL_COLUMN_SIZE)
		self.tvcolumn3.set_attributes( \
			__cell4, text=main.TREEVIEW_FILELIST_PANEL_COLUMN_PERMISSION)
		self.tvcolumn4.set_attributes( \
			__cell5, text=main.TREEVIEW_FILELIST_PANEL_COLUMN_CREATED_TIME)

		self.tvcolumn1.set_resizable(True)
		self.tvcolumn2.set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)
		self.tvcolumn3.set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)
		self.tvcolumn4.set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)

	def insert_row (self) :
		pass

	def makeUriList (self, scheme="file") :
		__list = __OS__.make_uri_list_from_list( \
			["%s://%s" % \
				(scheme, self.get_value( \
					i, main.TREEVIEW_FILELIST_PANEL_COLUMN_FULL_PATH),) \
						for i in self.ROWS_CURRENT_SELECTED] \
		)

		return __list

	###################################################################
	# EVENT HANDLER
	def on_key_release_event (self, treeview, event) :
		__keyname = func_global.get_keyname(event)
		# remove
		if __keyname in self.AVAILABLE_KEY :
			if __keyname == "<Control>c" :
				self.on_copy(treeview)

			elif __keyname == "<Control>x" :
				self.on_cut(treeview)

			elif __keyname == "<Control>v" :
				self.on_paste(treeview)

			elif __keyname == "Delete" :
				self.on_remove(None)

			elif __keyname == "<Control>a" :
				self.on_select_all(treeview)

			elif __keyname == "F2" :
				self.on_rename(treeview)

			return True

		# go parent directory
		elif __keyname in ["<Alt>Up", "Backspace", ] :
			self.on_directory_up()
			return True

		# cursor moves.
		elif __keyname in ["<Shift>Down", "<Shift>Up", ] :
			__TREEVIEW__.TreeViewWithMultipleItemSelectionDND.on_key_release_event( \
				self, treeview, event)
			return False

		# refresh file list.
		elif __keyname in ["<Control>r", "F5", ] :
			self.on_refresh()
			return True
		else :
			return False

	def on_select_all (self, widget) :
		selection = self.WIDGET_TREEVIEW.get_selection()
		selection.select_all()

	def getInfoOfCopyAndPaste (self) :
		__path = self.get_selected()[1]

		__data = list()
		for i in __path :
			__full_path = self.WIDGET_TREESTORE.get_value( \
				self.get_iter(i), main.TREEVIEW_FILELIST_PANEL_COLUMN_FULL_PATH)
			__permission_bin = self.WIDGET_TREESTORE.get_value( \
				self.get_iter(i), main.TREEVIEW_FILELIST_PANEL_COLUMN_PERMISSION_BIN)

			__data.append((__permission_bin, __full_path, ))

		if isinstance(self, FileListRemote) :
			__type = "ssh-filebrowser-remote"
		elif isinstance(self, FileListLocal) :
			__type = "ssh-filebrowser-local"
		else :
			return None

		return (__type, __data)

	def on_copy (self, widget) :
		__type, __data = self.getInfoOfCopyAndPaste()
		self.FILEBROWSER.STATE_LOCAL["clipboard"].register( \
			"copy", __type, __data)

	def on_cut (self, widget) :
		__type, __data = self.getInfoOfCopyAndPaste()

		self.FILEBROWSER.STATE_LOCAL["clipboard"].register( \
			"cut", __type, __data)

	def on_paste (self, widget) :
		return False

		#__data = self.FILEBROWSER.STATE_LOCAL["clipboard"].getData()
		#self.FILEBROWSER.STATE_LOCAL["clipboard"].clean()

		"""
		IMPLEMENT BELOW
		"""

	def on_remove (self, widget) :
		__store, __paths = self.get_selected()
		if len(__paths) < 1 : return

		self.on_remove0(__paths)

	def on_rename (self, widget) :
		__path = self.get_selected()[1]
		if len(__path) > 1 :
			return
		else :
			iter = self.get_iter(__path[0])

		self.set_editable(iter)

	def on_directory_up (self) :
		"""
		NOT IMPLEMENTED
		"""
		pass

class FileListRemote (TreeViewFileList) :

	def __init__ (self, xml, **kwargs) :
		TreeViewFileList.__init__(self, "remote", xml, **kwargs)

		# add menu
		#self.MENU = Menu.SSHFileBrowser_FileList_Remote(obj_panel=self)

	def addRow (self, data) :
		__filetype = __OS__.get_filetype_by_st_mode(data[2]["permissions"])
		if FileTransfer.is_directory_by_fileattr(data) :
			__pixbuf = self.ICONS["directory"]
		else :
			if __filetype == stat.S_IFLNK :
				if not data[2]["target"] :
					__pixbuf = self.ICONS["none"]
				else :
					__pixbuf = self.ICONS["link"]
			else :
				__pixbuf = self.ICONS["file"]

		__permission = __OS__.get_permission_oct2str( \
			__OS__.get_permission_st_mode2oct(data[2]["permissions"]))

		if __filetype != stat.S_IFREG :
			__size = ""
		else :
			__size = __OS__.get_filesize_as_readable(int(data[2]["size"]))

		__iter = self.WIDGET_TREESTORE.append( \
			[ \
				__pixbuf, \
				data[0], \
				__size, \
				__permission, \
				time.strftime("%D %R", time.localtime(data[2]["mtime"])), \
				str(int(data[2]["permissions"])), \
				os.path.join(self.FILEBROWSER.FILE_TRANSFER.CURRENT_DIRECTORY, data[0]), \
				data, \
			] \
		)

		return __iter

	##################################################
	# EVENT METHOD
	def on_button_release_event (self, treeview, event) :
		__retval = TreeViewFileList.on_button_release_event(self, treeview, event)

		if event.button == 3 :
			self.MENU.popup(event)
			__retval = True

		return __retval

	def on_refresh (self) :
		self.FILEBROWSER.listAllRemote()

	def on_remove0 (self, paths) :
		__f = FileTransfer.TransferFile(self.FILEBROWSER)
		for i in paths :
			# get full path.
			__path = self.get_value(i, main.TREEVIEW_FILELIST_PANEL_COLUMN_FULL_PATH)
			__f.remove(__path)

		return

	def on_editing_canceled (self, cell) :
		cell.set_property("editable", False)

	def on_rename_edited (self, cell, path, new_text) :
		__name_old = self.get_value(path, main.TREEVIEW_FILELIST_PANEL_COLUMN_NAME)

		if __name_old == new_text :
			cell.set_property("editable", False)
			return

		__filepath = \
			self.get_value(path, main.TREEVIEW_FILELIST_PANEL_COLUMN_FULL_PATH)
		__new_path = os.path.join(self.FILEBROWSER.FILE_TRANSFER.CURRENT_DIRECTORY, new_text)

		d = self.FILEBROWSER.FILE_TRANSFER.cmd_RENAME(__filepath, __new_path)
		d.addCallback(self.__cb_on_rename_edited, cell, path, __new_path)
		d.addErrback(self.__eb_on_rename_edited, path)

	def __cb_on_rename_edited (self, result, cell, path, new_path) :
		self.set_value( \
			path, main.TREEVIEW_FILELIST_PANEL_COLUMN_FULL_PATH, new_path)
		self.set_value( \
			path, main.TREEVIEW_FILELIST_PANEL_COLUMN_NAME, \
			os.path.basename(new_path) \
		)

		cell.set_property("editable", False)

		# reload file list.
		self.FILEBROWSER.listAllRemote(self.FILEBROWSER.FILE_TRANSFER.CURRENT_DIRECTORY)

	def __eb_on_rename_edited (self, f, path) :
		self.FILEBROWSER.errBackRemote(failure.Failure())

		self.set_editable(self.get_iter(path))
		return

	def on_drag_data_get_data \
			(self, treeview, context, selection, target_id, etime) :
		__list = self.makeUriList("sftp")
		selection.set(selection.target, 8, __list)
		return True

	def on_drag_data_received_data \
			(self, treeview, context, x, y, selection, info, etime) :
		__f = FileTransfer.TransferFile(self.FILEBROWSER)
		for i in __OS__.get_list_from_uri_list(selection.data) :
			(__scheme, __path, ) = __OS__.get_pathinfo_from_uri(i)

			if __scheme == "file" :
				__f.put(__path, self.FILEBROWSER.FILE_TRANSFER.CURRENT_DIRECTORY)

	def on_directory_up (self) :
		self.FILEBROWSER.on_directory_up_remote(None)

	def on_row_activated (self, treeview, path, view_column, user_param=None) :
		__iter = self.WIDGET_TREESTORE.get_iter(path)
		__name = self.getName(__iter)
		__fileattr = \
			self.get_value(path, main.TREEVIEW_FILELIST_PANEL_COLUMN_FULL_FILEATTR)

		if FileTransfer.is_directory_by_fileattr(__fileattr) : # if this is directory
			self.FILEBROWSER.listAllRemote(__name)
		else : # if this is not a directory, view file.
			return

	def on_key_press_event (self, treeview, event) :
		__keyname = func_global.get_keyname(event)

		if __keyname == "Return" :
			__iters = self.get_selected()[1]
			if len(__iters) > 1 :
				return True

			if isinstance(__iters[0], gtk.TreeIter) :
				__iter = __iters[0]
			else :
				__iter = self.get_iter(__iters[0])

			__fileattr = \
				self.get_value(self.get_path(__iter), \
				main.TREEVIEW_FILELIST_PANEL_COLUMN_FULL_FILEATTR \
			)

			if FileTransfer.is_directory_by_fileattr(__fileattr) :
				return False
			else :
				return True

	##################################################
	# MISC METHOD
	def reorderDirectoryFirst (self, result, removeDotDirectory=True) :
		__directory = dict()
		__file = dict()

		for i in result :
			if FileTransfer.is_directory_by_fileattr(i) :
				__directory.update({ i[0] : i})
			else :
				__file.update({ i[0] : i})

		__dk = __directory.keys()
		__dk.sort(__OS__.cmpfunc_for_filesorting)
		__fk = __file.keys()
		__fk.sort(__OS__.cmpfunc_for_filesorting)

		__list = list()
		[__list.append(__directory[i]) for i in __dk]
		[__list.append(__file[i]) for i in __fk]

		return __list

class FileListLocal (TreeViewFileList) :

	MENU = None
	FOCUSED_IN = False

	def __init__ (self, xml, **kwargs) :
		TreeViewFileList.__init__(self, "local", xml, **kwargs)

		# add menu
		#self.MENU = Menu.SSHFileBrowser_FileList_Local(obj_panel=self)

	def addRow (self, data) :

		__filetype = __OS__.get_filetype_by_st_mode(data[1])

		if __filetype == stat.S_IFDIR :
			__pixbuf = self.ICONS["directory"]
		elif __filetype == stat.S_IFLNK :
			__pixbuf = self.ICONS["link"]
		else :
			__pixbuf = self.ICONS["file"]

		__permission = __OS__.get_permission_oct2str( \
			__OS__.get_permission_st_mode2oct(data[1]))

		if __filetype != stat.S_IFREG :
			__size = ""
		else :
			__size = __OS__.get_filesize_as_readable(int(data[7]))

		__iter = self.WIDGET_TREESTORE.append( \
			[ \
				__pixbuf, \
				os.path.basename(data[0]), \
				__size, \
				__permission, \
				time.strftime("%D %R", time.localtime(data[8])), \
				str(int(data[1])), \
				data[0], \
				data, \
			] \
		)

		return __iter

	##################################################
	# EVENT METHOD
	def on_button_release_event (self, treeview, event) :
		__retval = TreeViewFileList.on_button_release_event(self, treeview, event)

		if event.button == 3 :
			self.MENU.popup(event)
			__retval = True

		return __retval

	def on_paste (self, treeview) :
		__data = self.FILEBROWSER.STATE_LOCAL["clipboard"].getData()
		if len(__data.keys()) < 1 : return
		if len(__data["value"]) < 1 : return

		# if the file in clipboard is in current directory, does noting.
		__dir_path = os.path.abspath(os.path.dirname(__data["value"][0][1]))
		if __dir_path == \
				os.path.abspath(self.FILEBROWSER.CURRENT_DIRECTORY_LOCAL) :
			print "same directory, does nothing."
			return False

		for perm_bin, file_path in __data["value"] :
			if not __OS__.copy( \
					file_path, self.FILEBROWSER.CURRENT_DIRECTORY_LOCAL, perm_bin) :
				print "copy failed"
				break

		print "copy ok."
		#self.FILEBROWSER.STATE_LOCAL["clipboard"].clean()

	def on_refresh (self) :
		self.FILEBROWSER.listAllLocal()

	def on_remove0 (self, paths) :
		for i in paths :
			__filepath = \
				self.get_value(i, main.TREEVIEW_FILELIST_PANEL_COLUMN_FULL_PATH)

			if not __OS__.remove(__filepath) :
				self.FILEBROWSER.errBackLocal(__retval, "remove")
				break

		self.FILEBROWSER.listAllLocal()

	def on_editing_canceled (self, cell) :
		cell.set_property("editable", False)

	def on_rename_edited (self, cell, path, new_text) :

		__name_old = self.get_value(path, main.TREEVIEW_FILELIST_PANEL_COLUMN_NAME)
		if __name_old == new_text :
			cell.set_property("editable", False)
			return

		__filepath = \
			self.get_value(path, main.TREEVIEW_FILELIST_PANEL_COLUMN_FULL_PATH)
		__new_path = os.path.join(self.FILEBROWSER.CURRENT_DIRECTORY_LOCAL, new_text)

		try :
			os.rename(__filepath, __new_path)
		except OSError, message :
			self.FILEBROWSER.writeLog(message)
			self.set_editable(self.get_iter(path))
			return

		self.set_value( \
			path, main.TREEVIEW_FILELIST_PANEL_COLUMN_FULL_PATH, __new_path)
		self.set_value( \
			path, main.TREEVIEW_FILELIST_PANEL_COLUMN_NAME, new_text)

		cell.set_property("editable", False)

		# reload file list.
		self.FILEBROWSER.listAllLocal(self.FILEBROWSER.CURRENT_DIRECTORY_LOCAL)

	def on_drag_data_get_data (self, treeview, context, selection, target_id, etime) :
		__list = self.makeUriList("file")
		selection.set(selection.target, 8, __list)
		return True

	def on_drag_data_received_data \
			(self, treeview, context, x, y, selection, info, etime) :
		__f = FileTransfer.TransferFile(self.FILEBROWSER)
		for i in __OS__.get_list_from_uri_list(selection.data) :
			(__scheme, __path, ) = __OS__.get_pathinfo_from_uri(i)

			if __scheme == "file" :
				try :
					__path = gnomevfs.get_local_path_from_uri(i)
				except :
					continue

				__OS__.copy(__path, self.FILEBROWSER.CURRENT_DIRECTORY_LOCAL)
				self.FILEBROWSER.listAllLocal()
			elif __scheme == "sftp" :
				__f.get(__path, self.FILEBROWSER.CURRENT_DIRECTORY_LOCAL)

	def on_directory_up (self) :
		self.FILEBROWSER.on_directory_up_local(None)

	def on_row_activated (self, treeview, path, view_column, user_param=None) :
		__iter = self.WIDGET_TREESTORE.get_iter(path)
		__name = self.getName(__iter)
		__perms = self.getPermissions(__iter)

		if __OS__.is_directory_by_st_mode(int(__perms)) : # if this is directory
			self.FILEBROWSER.listAllLocal(__name)
			pass
		else : # if this is not a directory.
			__filepath = \
				self.get_value(path, main.TREEVIEW_FILELIST_PANEL_COLUMN_FULL_PATH)
			gnomevfs.url_show(gnomevfs.make_uri_canonical(__filepath))

	def on_key_press_event (self, treeview, event) :
		__keyname = func_global.get_keyname(event)

		if __keyname == "Return" :
			__iters = self.get_selected()[1]
			if len(__iters) > 1 :
				return True

			if isinstance(__iters[0], gtk.TreeIter) :
				__iter = __iters[0]
			else :
				__iter = self.get_iter(__iters[0])

			__name = self.getName(__iter)
			__perms = self.getPermissions(__iter)

			if __OS__.is_directory_by_st_mode(int(__perms)) : # if this is directory
				return False
			else : # if this is not a directory, view file.
				return True


	##################################################
	# MISC METHOD
	def reorderDirectoryFirst (self, result, removeDotDirectory=True) :

		__directory = dict()
		__file = dict()

		for i in result :
			if __OS__.is_directory_by_st_mode(int(i[1])) :
				__directory.update({ i[0] : i})
			else :
				__file.update({ i[0] : i})

		__dk = __directory.keys()
		__dk.sort(__OS__.cmpfunc_for_filesorting)
		__fk = __file.keys()
		__fk.sort(__OS__.cmpfunc_for_filesorting)

		__list = list()
		[__list.append(__directory[i]) for i in __dk]
		[__list.append(__file[i]) for i in __fk]

		return __list

class __SHORTCUT__ (__TREEVIEW__.TreeViewWithMultipleItemSelectionDND, \
	__VEE__.Core, __TREEVIEW__.Core, Widgets.WithGNOME) :


	SHORTCUT = None

	PROPERTIES = { \
		"enable-search" : True, \
		"fixed-height-mode" : False, \
		"headers-clickable" : False, \
		"headers-visible" : False, \
		"hover-expand" : False, \
		"hover-selection" : False, \
		"reorderable" : False, \
		"rules-hint" : False, \
	}

	def __init__ (self, type, xml, filebrowser, **kwargs) :
		"""
		kwargs is,
			state
		"""

		__VEE__.Core.__init__(self, **kwargs)
		Widgets.WithGNOME.__init__(self)

		self.FILEBROWSER = filebrowser

		__TREEVIEW__.Core.__init__(self, xml, "treeview-shortcut-%s" % type)
		__TREEVIEW__.TreeViewWithMultipleItemSelectionDND.__init__(self)

	def initialize (self) :
		__TREEVIEW__.Core.initialize(self)
		self.WIDGET_TREEVIEW.connect( \
			"key-release-event", self.on_key_release_event)

	def save_tree_to_xml (self) :

		self.DOM_TMP = __XML__.create_new_document("paths")

		self.WIDGET_TREESTORE.foreach(self.__save_tree_to_xml)

		fd = file(self.FILE_SHORTCUT, "w")
		fd.write(self.DOM_TMP.toprettyxml(indent='\t', newl='\n'))
		fd.close()

		return True

	def getUID (self, iter) :
		if not iter : return None

		return self.WIDGET_TREESTORE.get_value( \
			iter, main.TREEVIEW_SHORTCUT_PANEL_COLUMN_UID)

	def __save_tree_to_xml (self, model, path, iter, user_data=None) :
		__uid = self.getUID(iter)

		try :
			__node = self.SHORTCUT.getNodeByUID(__uid).cloneNode(False)
		except AttributeError :
			return

		self.NODELIST[path] = __node

		if len(path) == 1 :
			self.DOM_TMP.documentElement.appendChild(__node)
		else :
			__node_parent = self.NODELIST[path[:-1]]
			__node_parent.appendChild(__node)

	def create_frame (self) :

		# create a CellRenderers to render the data
		__cell0 = gtk.CellRendererPixbuf()
		__cell1 = gtk.CellRendererText()
		__cell1.connect("edited", self.on_rename_edited)
		__cell1.connect("editing-canceled", self.on_editing_canceled)

		# set property.
		__cell0.set_property( \
			"width", self.ICONS["up"].get_property("width"))
		__cell1.set_property("xalign", 0.0)

		self.tvcolumn0 = gtk.TreeViewColumn()
		self.tvcolumn0.set_expand(True)

		self.WIDGET_TREESTORE = gtk.ListStore( \
			gobject.TYPE_OBJECT, gobject.TYPE_STRING, \
			gobject.TYPE_STRING, gobject.TYPE_STRING, \
		)

		self.WIDGET_TREEVIEW.set_model(self.WIDGET_TREESTORE)

		# add columns to treeview
		self.WIDGET_TREEVIEW.append_column(self.tvcolumn0)

		self.tvcolumn0.pack_start(__cell0, False)
		self.tvcolumn0.pack_start(__cell1, True)

		self.tvcolumn0.set_attributes( \
			__cell0, pixbuf=main.TREEVIEW_SHORTCUT_PANEL_COLUMN_ICON)
		self.tvcolumn0.set_attributes( \
			__cell1, text=main.TREEVIEW_SHORTCUT_PANEL_COLUMN_NAME)
		self.tvcolumn0.set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)

	def insert_row (self) :
		self.WIDGET_TREESTORE.clear()
		if not self.SHORTCUT : return

		for i in self.SHORTCUT.getChild() :
			self.add_row(i)

	def add_row (self, node) :
		data = self.SHORTCUT.getValues(node)
		self.WIDGET_TREESTORE.append( \
			[self.ICONS["directory"], data["name"], data["value"], data["uid"], ] \
		)

	#####################################################################
	# EVENT HANDLER
	def on_remove_path (self, widget) :
		__store, __paths = self.get_selected()

		if len(__paths) < 1 : return

		__iters = list()
		for i in __paths :
			__iters.append(self.get_iter(i))

		for i in __iters :
			self.__remove_path(i)

	def __remove_path (self, iter) :
		if not iter : return

		__uid = self.getUID(iter)
		if self.SHORTCUT.remove(__uid) :
			self.remove(treeiter=iter)

			self.save_tree_to_xml()

	def set_editable (self, iter) :
		__path = self.get_path(iter)

		__column = self.WIDGET_TREEVIEW.get_column(0)
		cell = __column.get_cell_renderers()[1]
		cell.set_property("editable", True)

		self.WIDGET_TREEVIEW.set_cursor(__path, __column, True)

	def on_key_release_event (self, treeview, event) :
		__keyname = func_global.get_keyname(event)

		# remove
		if __keyname == "Delete" :
			self.on_remove_path(None)
			return True

		# rename
		elif __keyname == "F2" :
			__store, __iter = self.get_selected()
			if __iter is None : return

			self.set_editable(__iter)
			return True

		return False

	def on_editing_canceled (self, cell) :
		cell.set_property("editable", False)

	def on_rename_edited (self, cell, path, new_text) :
		cell.set_property("editable", False)

		__iter = self.get_iter(path)
		__uid = self.getUID(__iter)
		__node = self.SHORTCUT.getNodeByUID(__uid)

		self.SHORTCUT.setValue(__node, "name", new_text)
		self.set_value(path, main.TREEVIEW_SHORTCUT_PANEL_COLUMN_NAME, new_text)
		self.save_tree_to_xml()

	def on_drag_data_get_data (self, treeview, context, selection, target_id, etime) :
		"""
		NOT IMPLEMENTED
		"""
		pass


class ShortcutLocal (__SHORTCUT__) :

	NODELIST = dict()
	FILEBROWSER = None
	FILE_SHORTCUT = main.FILE_SHORTCUT_LOCAL

	def __init__ (self, xml, filebrowser, **kwargs) :
		__SHORTCUT__.__init__(self, "local", xml, filebrowser, **kwargs)

	def create_frame (self) :
		self.SHORTCUT = self.STATE_GLOBAL["shortcut-local"]
		__SHORTCUT__.create_frame(self)

	def initialize (self) :
		__SHORTCUT__.initialize(self)
		self.XML.get_widget("button34").connect("clicked", self.on_add_path)
		self.XML.get_widget("button35").connect("clicked", self.on_remove_path)

	def on_add_path (self, widget) :
		self.__add_path(self.FILEBROWSER.CURRENT_DIRECTORY_LOCAL)

	def __add_path (self, directory) :
		if self.SHORTCUT.getByValue(directory) :
			return

		__new_node = self.SHORTCUT.addNew()
		self.SHORTCUT.setValue(__new_node, "value", directory)
		self.SHORTCUT.setValue( \
			__new_node, "name", os.path.basename(os.path.normpath(directory)) \
		)

		self.add_row(__new_node)
		self.save_tree_to_xml()

	def on_row_activated (self, treeview, path, view_column, user_param=None) :
		self.FILEBROWSER.listAllLocal( \
			self.get_value(path, main.TREEVIEW_SHORTCUT_PANEL_COLUMN_VALUE) \
		)

	def on_drag_data_received_data (self, treeview, context, x, y, selection, info, etime) :
		for i in __OS__.get_list_from_uri_list(selection.data) :
			(__scheme, __path, ) = __OS__.get_pathinfo_from_uri(i)
			if __scheme != "file" :
				continue

			self.__add_path(__path)

class ShortcutRemote (__SHORTCUT__) :

	NODELIST = dict()
	FILEBROWSER = None

	def __init__ (self, xml, filebrowser, **kwargs) :
		__SHORTCUT__.__init__(self, "remote", xml, filebrowser, **kwargs)

	def initialize (self) :
		__SHORTCUT__.initialize(self)
		self.XML.get_widget("button34").connect("clicked", self.on_add_path)
		self.XML.get_widget("button35").connect("clicked", self.on_remove_path)

	def create_frame (self) :
		__labelname = main_ssh.get_label_name(self.FILEBROWSER.PROPERTIES)
		self.FILE_SHORTCUT = \
			os.path.join(os.getenv("HOME"), \
			Conf__.DIR_LOCAL, \
			"shortcut_remote_%s.xml" % __labelname \
		)

		# get local shortcut xmlhandler.
		try :
			self.SHORTCUT = XMLHandler.Shortcut(self.FILE_SHORTCUT)
		except IOError :
			self.SHORTCUT = XMLHandler.Shortcut()

		__SHORTCUT__.create_frame(self)

	def on_add_path (self, widget) :
		self.__add_path(self.FILEBROWSER.FILE_TRANSFER.CURRENT_DIRECTORY)

	def __add_path (self, directory) :
		if self.SHORTCUT.getByValue(directory) :
			return

		__new_node = self.SHORTCUT.addNew()
		self.SHORTCUT.setValue(__new_node, "value", directory)
		self.SHORTCUT.setValue( \
			__new_node, \
			"name", os.path.basename(os.path.normpath(directory)) \
		)

		self.add_row(__new_node)
		self.save_tree_to_xml()

	def on_row_activated (self, treeview, path, view_column, user_param=None) :
		self.FILEBROWSER.listAllRemote( \
			self.get_value(path, main.TREEVIEW_SHORTCUT_PANEL_COLUMN_VALUE) \
		)

	def on_drag_data_received_data (self, treeview, context, x, y, selection, info, etime) :
		for i in __OS__.get_list_from_uri_list(selection.data) :
			(__scheme, __path, ) = __OS__.get_pathinfo_from_uri(i)
			if __scheme != "sftp" :
				continue

			self.__add_path(__path)

class FileTransferStatus (__TREEVIEW__.Core, Widgets.WithGNOME) :

	FILE_TRANSFER = None
	PARENT = None

	HANDLER_ID_CURSOR_CHANGED = None
	FILETRANSFER = None
	FILETRANSFERQUEUE = None

	PROPERTIES = { \
		"enable-search" : True, \
		"fixed-height-mode" : False, \
		"headers-clickable" : False, \
		"headers-visible" : True, \
		"hover-expand" : False, \
		"hover-selection" : False, \
		"reorderable" : False, \
		"rules-hint" : True, \
	}


	def __init__ (self, xml, filetransfer, **kwargs) :
		self.FILETRANSFER = filetransfer

		Widgets.WithGNOME.__init__(self)
		self.generateIcon()

		__TREEVIEW__.Core.__init__(self, xml, "treeview-filetransfer-status")
		self.WIDGET_TREEVIEW.get_selection().set_mode(gtk.SELECTION_MULTIPLE)

		# initiate filetransferQueue
		self.FILETRANSFERQUEUE = self.FILETRANSFER.FILETRANSFERQUEUE
		self.FILETRANSFERQUEUE.TREESTORE = self.WIDGET_TREESTORE
		self.FILETRANSFERQUEUE.startRunning()

	def getCurrentSelectedUID (self) :
		__paths = self.get_selected()[1]
		__uids = list()
		for i in __paths :
			__uids.append(self.getUID(self.get_iter(i)))

		return __uids

	def getUID (self, iter) :
		if not iter : return None

		return self.WIDGET_TREESTORE.get_value(iter, \
			main.TREEVIEW_FILE_TRANSFER_STATUS_PANEL_COLUMN_UID)

	def create_frame (self) :
		# create a CellRenderers to render the data
		__cell0 = gtk.CellRendererPixbuf()
		__cell1 = gtk.CellRendererPixbuf()
		__cell2 = gtk.CellRendererText()
		__cell3 = gtk.CellRendererText()
		__cell4 = gtk.CellRendererText()
		__cell5 = gtk.CellRendererText()
		__cell6 = gtk.CellRendererText()
		__cell7 = gtk.CellRendererText()

		# set property.
		__cell0.set_property( \
			"width", self.ICONS["up"].get_property("width"))
		__cell1.set_property( \
			"width", self.ICONS["__default__"].get_property("width"))
		__cell1.set_property("xalign", 0.0)
		__cell2.set_property("xalign", 0.0)

		if func_global.get_pygtk_version() > (2,6, ):
			__cell2.set_property("ellipsize", pango.ELLIPSIZE_START)
			__cell2.set_property("ellipsize-set", True)

		__cell3.set_property("xalign", 0.0)
		__cell4.set_property("xalign", 1.0)
		__cell5.set_property("xalign", 1.0)
		__cell6.set_property("xalign", 1.0)
		__cell7.set_property("xalign", 0.0)

		self.tvcolumn0 = gtk.TreeViewColumn()
		self.tvcolumn1 = gtk.TreeViewColumn(_("Remote Filename"))
		self.tvcolumn2 = gtk.TreeViewColumn(_("Size"))
		self.tvcolumn3 = gtk.TreeViewColumn(_("TimeLeft"))
		self.tvcolumn4 = gtk.TreeViewColumn(_("%"))
		self.tvcolumn5 = gtk.TreeViewColumn(_("Speed"))
		self.tvcolumn1.set_expand(True)

		self.WIDGET_TREESTORE = gtk.ListStore( \
			gobject.TYPE_OBJECT, \
			gobject.TYPE_OBJECT, gobject.TYPE_STRING, \
			gobject.TYPE_STRING, gobject.TYPE_STRING, \
			gobject.TYPE_STRING, gobject.TYPE_STRING, \
			gobject.TYPE_STRING, gobject.TYPE_STRING, \
			gobject.TYPE_STRING, \
		)

		self.WIDGET_TREEVIEW.set_model(self.WIDGET_TREESTORE)

		# add columns to treeview
		self.WIDGET_TREEVIEW.append_column(self.tvcolumn0)
		self.WIDGET_TREEVIEW.append_column(self.tvcolumn1)
		self.WIDGET_TREEVIEW.append_column(self.tvcolumn2)
		self.WIDGET_TREEVIEW.append_column(self.tvcolumn3)
		self.WIDGET_TREEVIEW.append_column(self.tvcolumn4)
		self.WIDGET_TREEVIEW.append_column(self.tvcolumn5)

		self.tvcolumn0.pack_start(__cell0, False)
		self.tvcolumn1.pack_start(__cell1, False)
		self.tvcolumn1.pack_start(__cell2, True)
		self.tvcolumn2.pack_start(__cell7, True)
		self.tvcolumn3.pack_start(__cell4, False)
		self.tvcolumn4.pack_start(__cell5, False)
		self.tvcolumn5.pack_start(__cell6, False)

		self.tvcolumn0.set_attributes( \
			__cell0, pixbuf=main.TREEVIEW_FILE_TRANSFER_STATUS_PANEL_COLUMN_DIRECTION)
		self.tvcolumn1.set_attributes( \
			__cell1, pixbuf=main.TREEVIEW_FILE_TRANSFER_STATUS_PANEL_COLUMN_ICON)
		self.tvcolumn1.set_attributes( \
			__cell2, markup=main.TREEVIEW_FILE_TRANSFER_STATUS_PANEL_COLUMN_FILEPATH)
		self.tvcolumn2.set_attributes( \
			__cell7, markup=main.TREEVIEW_FILE_TRANSFER_STATUS_PANEL_COLUMN_FILE_SIZE)
		self.tvcolumn3.set_attributes( \
			__cell4, markup=main.TREEVIEW_FILE_TRANSFER_STATUS_PANEL_COLUMN_TIME_LEFT)
		self.tvcolumn4.set_attributes( \
			__cell5, markup=main.TREEVIEW_FILE_TRANSFER_STATUS_PANEL_COLUMN_PERCENT)
		self.tvcolumn5.set_attributes( \
			__cell6, markup=main.TREEVIEW_FILE_TRANSFER_STATUS_PANEL_COLUMN_SPEED)

		self.tvcolumn0.set_resizable(False)
		self.tvcolumn1.set_resizable(False)
		self.tvcolumn2.set_resizable(False)
		self.tvcolumn3.set_resizable(False)
		self.tvcolumn1.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
		self.tvcolumn3.set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)

	def insert_row (self) :
		pass

	def addRows (self, result) :
		result = self.reorderDirectoryFirst(result)
		for i in result :
			self.addRow(i)

	def addRow (self, **kwargs) :
		"""
		data is dict, which has the,
			key, value
			filepath
			filesize
		"""

		if kwargs["method"] == "put" :
			__direction = self.ICONS["up"]
		else :
			__direction = self.ICONS["down"]

		__uid = func_global.get_uid()
		__iter = self.WIDGET_TREESTORE.append( \
				[ \
					__direction.copy(), \
					self.ICONS["file"].copy(), \
					kwargs["filepath"], \
					__OS__.get_filesize_as_readable(int(kwargs["filesize"])), \
					"--:--", \
					"-", \
					"- Gbytes/s", \
					kwargs["filepath-local"], \
					__uid, \
					str(), \
				] \
		)

		kwargs["iter"] = __iter
		self.FILETRANSFERQUEUE.append(__uid, **kwargs)

		return __iter

	def clear (self) :
		self.WIDGET_TREESTORE.foreach(self.__clear)

	def __clear (self, model, path, iter, user_data=None) :
		__is_done = \
			self.get_value(path, main.TREEVIEW_FILE_TRANSFER_STATUS_PANEL_COLUMN_DONE)

		if __is_done == "done" : # remove this row.
			self.remove(path=path)

	##################################################
	# MISC METHOD
	SET_FOCUS = False
	def update (self, iter, timeLeft, rate, speed) :
		if not iter or (iter and not self.WIDGET_TREESTORE.iter_is_valid(iter)) :
			return True

		__path = self.get_path(iter)
		__row = self.WIDGET_TREESTORE[__path[0]]

		if not self.SET_FOCUS :
			self.set_cursor(__path)
			self.SET_FOCUS = True

		__row[main.TREEVIEW_FILE_TRANSFER_STATUS_PANEL_COLUMN_TIME_LEFT] = \
			func.abbrevTime(timeLeft)
		__row[main.TREEVIEW_FILE_TRANSFER_STATUS_PANEL_COLUMN_PERCENT] = \
			"%d%%" % rate
		__row[main.TREEVIEW_FILE_TRANSFER_STATUS_PANEL_COLUMN_SPEED] = \
			speed

		if rate == 100 :
			func_global.change_saturation( \
				__row[main.TREEVIEW_FILE_TRANSFER_STATUS_PANEL_COLUMN_DIRECTION], \
				0.2 \
			)

			func_global.change_saturation( \
				__row[main.TREEVIEW_FILE_TRANSFER_STATUS_PANEL_COLUMN_ICON], \
				0.2 \
			)

			__row[main.TREEVIEW_FILE_TRANSFER_STATUS_PANEL_COLUMN_FILE_SIZE] = \
				"<span foreground=\"grey\">%s</span>" % \
				__row[main.TREEVIEW_FILE_TRANSFER_STATUS_PANEL_COLUMN_FILE_SIZE]

			__row[main.TREEVIEW_FILE_TRANSFER_STATUS_PANEL_COLUMN_FILEPATH] = \
				"<span foreground=\"grey\">%s</span>" % \
				__row[main.TREEVIEW_FILE_TRANSFER_STATUS_PANEL_COLUMN_FILEPATH]

			__row[main.TREEVIEW_FILE_TRANSFER_STATUS_PANEL_COLUMN_SPEED] = \
				"<span foreground=\"grey\">0 G/s</span>"

			__row[main.TREEVIEW_FILE_TRANSFER_STATUS_PANEL_COLUMN_TIME_LEFT] = \
				"<span foreground=\"grey\">%s</span>" % func.abbrevTime(timeLeft)

			__row[main.TREEVIEW_FILE_TRANSFER_STATUS_PANEL_COLUMN_PERCENT] = \
				"<span foreground=\"grey\">%s</span>" % rate

			__row[main.TREEVIEW_FILE_TRANSFER_STATUS_PANEL_COLUMN_DONE] = \
				"done"

			return False

		return True


class __TREE__ (__TREEVIEW__.Core, Widgets.WithGNOME) :

	FILEBROWSER = None

	PROPERTIES = { \
		"enable-search" : True, \
		"fixed-height-mode" : False, \
		"headers-clickable" : False, \
		"headers-visible" : False, \
		"hover-expand" : False, \
		"hover-selection" : False, \
		"reorderable" : False, \
		"rules-hint" : False, \
	}

	def __init__ (self, type, xml, **kwargs) :

		self.FILEBROWSER = kwargs["filebrowser"]

		Widgets.WithGNOME.__init__(self)
		self.generateIcon()

		__TREEVIEW__.Core.__init__(self, xml, "treeview-tree-%s" % type)
		self.HANDLER_ID_CURSOR_CHANGED = \
			self.WIDGET_TREEVIEW.connect("cursor-changed", self.on_cursor_changed)

	def create_frame (self) :
		# create a CellRenderers to render the data
		__cell1 = gtk.CellRendererPixbuf()
		__cell2 = gtk.CellRendererText()

		# set property.
		__cell1.set_property( \
			"width", self.ICONS["__default__"].get_property("width"))
		__cell1.set_property("xalign", 0.5)
		__cell2.set_property("xalign", 0.0)

		self.tvcolumn1 = gtk.TreeViewColumn("Name")
		self.tvcolumn1.set_expand(True)

		self.WIDGET_TREESTORE = gtk.ListStore( \
			gobject.TYPE_OBJECT, gobject.TYPE_STRING, \
			gobject.TYPE_STRING, \
		)

		self.WIDGET_TREEVIEW.set_model(self.WIDGET_TREESTORE)

		# add columns to treeview
		self.WIDGET_TREEVIEW.append_column(self.tvcolumn1)

		self.tvcolumn1.pack_start(__cell1, False)
		self.tvcolumn1.pack_start(__cell2, True)

		self.tvcolumn1.set_attributes( \
			__cell1, pixbuf=main.TREEVIEW_TREE_PANEL_COLUMN_ICON)
		self.tvcolumn1.set_attributes( \
			__cell2, text=main.TREEVIEW_TREE_PANEL_COLUMN_NAME)

		self.tvcolumn1.set_resizable(True)
		#self.tvcolumn2.set_resizable(True)
		#self.tvcolumn3.set_resizable(True)
		#self.tvcolumn1.set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)
		self.tvcolumn1.set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)

	def getName (self, iter) :
		if not iter : return None

		return self.WIDGET_TREESTORE.get_value(iter, \
			main.TREEVIEW_TREE_PANEL_COLUMN_NAME)

	def insert_row (self) :
		pass

	def addRows (self, result) :
		self.clear()
		result = self.reorderDirectoryFirst(result)
		for i in result :
			self.addRow(i)


class TreeRemote (__TREE__) :

	FILEBROWSER = None
	PARENT = None

	HANDLER_ID_CURSOR_CHANGED = None

	def __init__ (self, xml, **kwargs) :
		__TREE__.__init__(self, "remote", xml, **kwargs)

	def addRow (self, data) :
		__iter = self.WIDGET_TREESTORE.append( \
			[ \
				self.ICONS["directory"], \
				data[0], \
				str(int(data[2]["permissions"])), \
			] \
		)

		return __iter

	##################################################
	# EVENT METHOD
	def on_cursor_changed (self, widget) :
		__store, __iter = self.get_selected()
		__name = self.getName(__iter)

		self.FILEBROWSER.listFileRemote( \
			os.path.join( \
				os.path.normpath( \
					os.path.join(
						self.FILEBROWSER.FILE_TRANSFER.CURRENT_DIRECTORY, "../"), \
					), \
				__name \
			)
		)

	##################################################
	# MISC METHOD
	def reorderDirectoryFirst (self, result, removeDotDirectory=True) :
		__directory = dict()
		__file = dict()

		for i in result :
			if not __OS__.is_directory_by_st_mode(i[2]["permissions"]) :
				continue

			__directory.update({ i[0] : i})

		__dk = __directory.keys()
		__dk.sort()

		__list = list()
		[__list.append(__directory[i]) for i in __dk]

		return __list

class TreeLocal (__TREE__) :

	PARENT = None

	HANDLER_ID_CURSOR_CHANGED = None

	def __init__ (self, xml, **kwargs) :
		__TREE__.__init__(self, "local", xml, **kwargs)

	def addRow (self, data) :
		__iter = self.WIDGET_TREESTORE.append( \
			[ \
				self.ICONS["directory"], \
				os.path.basename(data[0]), \
				str(int(data[1])), \
			] \
		)

		return __iter

	##################################################
	# EVENT METHOD
	def on_cursor_changed (self, widget) :
		__store, __iter = self.get_selected()

		__name = self.getName(__iter)

		__path = os.path.join( \
				os.path.normpath( \
					os.path.join(
						self.FILEBROWSER.CURRENT_DIRECTORY_LOCAL, "../"), \
					), \
				__name \
			)

		self.FILEBROWSER.listFileLocal(__path)

	##################################################
	# MISC METHOD
	def reorderDirectoryFirst (self, result, removeDotDirectory=True) :

		__directory = dict()
		__file = dict()

		for i in result :
			if not __OS__.is_directory_by_st_mode(i[1]) :
				continue

			__directory.update({ i[0] : i})

		__dk = __directory.keys()
		__dk.sort()

		__list = list()
		[__list.append(__directory[i]) for i in __dk]

		return __list

"""
Description
-----------


ChangeLog
---------


Usage
-----


"""

__author__ =  "Spike^ekipS <spikeekips@gmail.com>"
__version__=  "0.1"
__nonsense__ = ""

__file__ = "Panel.py"


