# -*- coding: utf-8 -*-
#    Copyright 2005 Spike^ekipS <spikeekips@gmail.com>
#
#       This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import xml
import xml.parsers.expat as Expat
from xml.dom import minidom
from vee import func, __XML__

VALUE_TYPE_SHORTCUT = { \
	"value" : "str", \
	"name" : "str", \
	"uid" : "str", \
}

class Shortcut (__XML__.__XML_PARSER__) :

	def __init__ (self, xml_filename=None) :
		if xml_filename :
			try :
				__XML__.__XML_PARSER__.__init__(self, xml_filename)
			except Expat.ExpatError :
				self.DOM = minidom.parseString("<?xml version=\"1.0\"?><paths></paths>")
			except IOError :
				self.DOM = minidom.parseString("<?xml version=\"1.0\"?><paths></paths>")
		else :
			self.DOM = minidom.parseString("<?xml version=\"1.0\"?><paths></paths>")

	def __makeXPath (self, path, uid=None) :
		"""
		Read gtk treestore style 'path' and make xpath.

		@param :
			path : tuple(), ex) (0, 1,)

		@return :
			xpath : str()
		"""
		if type(path) is not tuple : return None

		if not uid :
			__xpath = "/paths/%s" % "/".join(["path", ] * len(path))
		else :
			__xpath = "/paths/%s[@uid = '%s']" % ("/".join(["path", ] * len(path)), uid)

		return __xpath

	def getOnes (self, path, uid=None) :

		xpath = self.__makeXPath(path, uid)
		if not xpath : return list()
		nodes = __XML__.evaluate_path(self.DOM, xpath)

		return nodes

	def getNodeByUID (self, uid) :
		nodes = __XML__.evaluate_path(self.DOM, "//path[ @uid = '%s' ]" % uid)
		if not nodes : return None

		return nodes[0]

	def getValuesByUID (self, uid) :
		return self.getValues(self.getNodeByUID(uid))

	def getValues (self, node) :
		if not hasattr(node, "attributes") : return None

		return __XML__.get_attributes(node.attributes, VALUE_TYPE_SHORTCUT)

	def getParent (self, node) :
		return node.parentNode

	def getChild (self, node_parent=None) :
		"""
		If node_parent is None, start top root.
		"""
		if not node_parent :
			__child = list()
			for i in self.getOnes((0,)) :
				if i.nodeType in [i.TEXT_NODE, i.COMMENT_NODE, ] : continue
				__child.append(i)
		else :
			__child = list()
			for i in node_parent.childNodes :
				if i.nodeType in [i.TEXT_NODE, i.COMMENT_NODE, ] : continue
				__child.append(i)

		return __child

	def createNewOne (self) :
		new = self.DOM.createElement("path")
		new.setAttribute("uid", func.get_uid())

		return new

	def getValue (self, node, key) :
		if not hasattr(node, "attributes") : return None
		if not node.attributes.has_key(key) : return None

		return __XML__.casting( \
			VALUE_TYPE_SHORTCUT[key], node.attributes.get(key).nodeValue)

	def setValue (self, node, key, value) :
		if not hasattr(node, "attributes") : return False
		if not VALUE_TYPE_SHORTCUT.has_key(key) : return False

		node.setAttribute(key, str(value))

		return True

	def updateValues (self, node, values=dict()) :
		if not hasattr(node, "attributes") : return False

		for __key, __value in values.items() :
			if not VALUE_TYPE_SHORTCUT.has_key(__key) : continue
			node.setAttribute(__key, str(__value))

		return True

	def addNew (self) :

		__nodes = __XML__.evaluate_path(self.DOM, "/paths")
		if not __nodes :
			print "no nodes."
			return False
		if len(__nodes) < 1 :
			return False

		node_parent = __nodes[0]

		# create new node.
		__node_new = self.createNewOne()

		# add node to parent node
		return node_parent.appendChild(__node_new)

	def remove (self, uid) :
		__this_node = self.getNodeByUID(uid)
		__parent_node = self.getParent(__this_node)

		if not __parent_node : return False
		__parent_node.removeChild(__this_node)

		return True

	def getByValue (self, value) :
		nodes = __XML__.evaluate_path(self.DOM, "//path[ @value = '%s' ]" % value)
		return nodes

"""
Description
-----------


ChangeLog
---------


Usage
-----


"""

__author__ =  "Spike^ekipS <spikeekips@gmail.com>"
__version__=  "0.1"
__nonsense__ = ""

__file__ = "XMLHandler.py.py"


