# -*- coding: utf-8 -*-
#	Copyright 2005 Spike^ekipS <spikeekips@gmail.com>
#
#	   This program is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; either version 2 of the License, or
#	(at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from twisted.conch.error import ConchError
from twisted.conch.client import options, agent, default
from twisted.internet import stdio, defer, utils, protocol, reactor
from twisted.conch.ssh import \
	connection, connection, common, transport, userauth, channel

import sys, os, pickle, struct, base64
from vee import __OS__
from vee.child.ssh import func

class ClientOptions (options.ConchOptions) :

	synopsis = ""

	optParameters = [ \
		["escape", "e", "~"],
		["localforward", "L", None, "listen-port:host:port   Forward local port to remote address"],
		["remoteforward", "R", None, "listen-port:host:port   Forward remote port to local address"],
		['subsystem', 's', 'sftp', 'Subsystem/server program to connect to.'], \
		['buffersize', 'B', 2768, 'Size of the buffer to use for sending/receiving.'],
		['requests', 'R', 5, 'Number of requests to make before waiting for a reply.'],
	]

	optFlags = [["null", "n", "Redirect input from /dev/null."],
		["fork", "f", "Fork to background after authentication."],
		["tty", "t", "Tty; allocate a tty even if command is given."],
		["notty", "T", "Do not allocate a tty."],
		["noshell", "N", "Do not execute a shell or command."],
	]

	zsh_actionDescr = {"localforward":"listen-port:host:port",
					   "remoteforward":"listen-port:host:port"}
	zsh_extras = ["*:command: "]

	localForwards = []
	remoteForwards = []

	def opt_logininfo (self, **kwargs) :
		for i in ["user", "host", "port", "password", ] :
			if i not in kwargs.keys() :
				raise RuntimeError, "%s was missing." % i

		self["user"] = kwargs["user"]
		self["host"] = kwargs["host"]
		self["port"] = int(kwargs["port"])
		self["password"] = kwargs["password"]

	def opt_escape (self, esc) :
		"Set escape character; ``none"" = disable"
		if esc == "none":
			self["escape"] = None
		elif esc[0] == "^" and len(esc) == 2:
			self["escape"] = chr(ord(esc[1])-64)
		elif len(esc) == 1:
			self["escape"] = esc
		else:
			sys.exit("Bad escape character '%s'." % esc)

	def opt_localforward (self, f) :
		"Forward local port to remote address (lport:host:port)"
		localPort, remoteHost, remotePort = f.split(":") # doesn"t do v6 yet
		localPort = int(localPort)
		remotePort = int(remotePort)
		self.localForwards.append((localPort, (remoteHost, remotePort)))

	def opt_remoteforward (self, f) :
		"""Forward remote port to local address (rport:host:port)"""
		remotePort, connHost, connPort = f.split(":") # doesn"t do v6 yet
		remotePort = int(remotePort)
		connPort = int(connPort)
		self.remoteForwards.append((remotePort, (connHost, connPort)))

class SSHUserAuthClient(userauth.SSHUserAuthClient):

	WINDOW = None

	def __init__ (self, user, connectoin) :
		userauth.SSHUserAuthClient.__init__ (self, user, connectoin)

	def getPassword(self, prompt = None):
		return defer.succeed(self.transport.CONF["options"]["password"])

	def getGenericAnswers(self, name, instruction, prompts):
		responses = []
		for prompt, echo in prompts:
			if self.transport.CONF["options"].has_key("password") :
				responses.append(self.transport.CONF["options"]["password"])
			elif echo:
				responses.append(raw_input(prompt))
			else:
				responses.append(getpass.getpass(prompt))

		return defer.succeed(responses)

class SSHConnection (connection.SSHConnection) :
	CONF = None
	WINDOW = None

	def serviceStarted (self) :
		__session = SSHSession()
		__session.CONF = self.CONF
		__session.WINDOW = self.WINDOW
		self.openChannel(__session)

class SSHSession (channel.SSHChannel) :

	name = 'session'
	CONF = None
	WINDOW = None

	def channelOpen (self, foo):

		if type(self.conn.CONF["options"]['subsystem']) is str :
			if self.conn.CONF["options"]['subsystem'].startswith('/') :
				request = 'exec'
			else:
				request = 'subsystem'
		else :
			request = 'subsystem'

		d = self.conn.sendRequest(self, request, \
			common.NS(self.conn.CONF["options"]['subsystem']), \
			wantReply=1 \
		)

		d.addCallback(self._cbSubsystem)
		d.addErrback(self._ebExit)

	def _ebExit (self, f) :
		if hasattr(f.value, 'value'):
			s = f.value.value
		else:
			s = str(f)

		#exitStatus = "conch: exiting with error %s" % f
		try:
			self.conn.transport.loseConnection()
		except:
			pass

	def _cbSubsystem (self, result):
		self.client = self.CONF["filetransfer"].getClient()
		self.client.makeConnection(self)

		self.dataReceived = self.client.dataReceived
		self.stdio = sys.stdout

	def extReceived (self, t, data):
		if t==connection.EXTENDED_DATA_STDERR:
			sys.stderr.write(data)
			sys.stderr.flush()

	def eofReceived (self):
		self.stdio.closeStdin()

	def closeReceived (self):
		self.conn.sendClose(self)

	def closed (self):
		pass
		#try :
		#	reactor.stop()
		#except :
		#	pass

	def stopWriting (self):
		#self.stdio.stopReading()
		print "stop writing"

	def startWriting (self):
		print "start writing"
		#self.stdio.startReading()

class SSHClientTransport (transport.SSHClientTransport) :

	CONF = None
	WINDOW = None

	def __init__ (self, conf, window) :
		self.CONF = conf
		self.WINDOW = window

	def receiveError(self, code, desc):
		global exitStatus
		exitStatus = 'conch:\tRemote side disconnected with error code %i\nconch:\treason: %s' % (code, desc)
		self.WINDOW.writeLog("%s\n" % desc, None, "error")

	def verifyHostKey(self, pubKey, fingerprint):
		d = defer.Deferred()
		d.addCallback(lambda x:defer.succeed(1))
		d.callback(2)
		return d

	def connectionSecure(self):
		if self.CONF["options"]['user']:
			user = self.CONF["options"]['user']
		else:
			user = getpass.getuser()

		# cross reference.
		__connection = SSHConnection()
		__connection.CONF = self.CONF
		__connection.WINDOW = self.WINDOW
		self.WINDOW.CONNECTION = __connection

		self.requestService( \
			SSHUserAuthClient( \
				user, \
				__connection, \
			) \
		)

class SSHClientFactory (protocol.ClientFactory) :
	noisy = 1
	transport = None
	CONF = None
	WINDOW = None

	def __init__ (self, conf, window) :
		self.CONF = conf
		self.WINDOW = window

	def stopFactory(self):
		print "====================================================="
		print "connection stopped."
		self.WINDOW.writeLog("Connection stopped.\n", None, "info")
		self.WINDOW.toggleConnected()
		#reactor.stop()

	def buildProtocol(self, addr):
		self.transport = SSHClientTransport(self.CONF, self.WINDOW)
		return self.transport

	def clientConnectionFailed(self, connector, f):
		func.sendMessage( \
			self.CONF["state"]["local"]["uid"], \
			self.CONF["state"]["global"]["gconf-client"], \
			f, \
		)
		self.WINDOW.disconnect()
		self.WINDOW.toggleConnected()

	def clientConnectionLost(self, connector, f):
		return
		func.sendMessage( \
			self.CONF["state"]["local"]["uid"], \
			self.CONF["state"]["global"]["gconf-client"], \
			f, \
		)
		self.WINDOW.disconnect()
		self.WINDOW.toggleConnected()

"""
Description
-----------


ChangeLog
---------


Usage
-----


"""

__author__ =  "Spike^ekipS <spikeekips@gmail.com>"
__version__=  "0.1"
__nonsense__ = ""

__file__ = "__SSHClient__.py"


