# -*- coding: utf-8 -*-
#    Copyright 2005 Spike^ekipS <spikeekips@gmail.com>
#
#       This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import os, gtk, stat, pwd, grp
from vee import __OS__, __VEE__, Parser, GNOME, func
from vee.child import __WIDGET__
from vee.child.vim import Panel, Parser

NOTEBOOK_PAGE_INFORMATION = 0
NOTEBOOK_PAGE_METHOD_LIST = 1

class MethodList (__VEE__.Core, __WIDGET__.WIDGET) :

	FILENAME = str()
	PARSER = None
	PANEL = None

	WIDGET_METHOD_LIST = None
	IS_METHOD_LIST = True

	def __init__ (self, **kwargs) :
		__VEE__.Core.__init__(self, **kwargs)
		self.VTE = kwargs["vte"]

		self.CHILD_CONFIG = self.STATE_GLOBAL["children"]["vim"]
		__WIDGET__.WIDGET.__init__( \
			self, "vim", "box_info", self.CHILD_CONFIG)

		__kwargs = dict()
		__kwargs.update(kwargs)
		__kwargs.update({"callback_functions" : self.on_update})
		self.PANEL = Panel.MethodList(self.xml, **__kwargs)

		self.WIDGET_METHOD_LIST = self.getWidget("box_method_list")

	def initialize (self) :
		# to prevent the method list widget oversizing when expanding 'key help'.
		# FIXME : 200 is anonymous size.
		self.WIDGET_SELF.set_size_request(200, -1)

		self.NOTEBOOK = self.getWidget("notebook1")

	def setFilename (self, filename=None) :
		if filename : self.FILENAME = filename
		if not __OS__.is_file(self.FILENAME) :
			self.FILENAME = None
			return False

		return True

	def updateMethodList (self) :
		# update default information
		__tooltip = gtk.Tooltips()

		# update label as filename.
		self.getWidget("label_filename").set_markup( \
			os.path.basename(self.FILENAME))

		__tooltip.set_tip(self.getWidget("eventbox_filename"), self.FILENAME)
		__tooltip.enable()

		##################################################	
		# GET PARSER
		__parser = Parser.findParser(self.FILENAME)
		if not __parser :
			if self.IS_METHOD_LIST :
				self.NOTEBOOK.remove_page(NOTEBOOK_PAGE_METHOD_LIST)

			self.IS_METHOD_LIST = False
		else :
			if not self.IS_METHOD_LIST :
				self.NOTEBOOK.append_page( \
					self.WIDGET_METHOD_LIST, gtk.Label("Method List"))

			self.IS_METHOD_LIST = True

			self.PARSER = __parser()
			self.PARSER.setFromFile(self.FILENAME)
			self.PARSER.format()

			# insert members,
			self.PANEL.insertRow(self.PARSER)
			self.NOTEBOOK.set_current_page(NOTEBOOK_PAGE_METHOD_LIST)

	def updateInfo (self) :
		__stat = os.stat(self.FILENAME)
		__size = __OS__.get_filesize_as_readable(__stat[stat.ST_SIZE])
		__perm = __OS__.get_permission_oct2str( \
			__OS__.get_permission_st_mode2oct(__stat[stat.ST_MODE]))

		self.getWidget("label_filename0").set_label(os.path.basename(self.FILENAME))
		__tooltip = gtk.Tooltips()
		__tooltip.set_tip(self.getWidget("eventbox_filename0"), self.FILENAME)
		__tooltip.enable()

		self.getWidget("label_size").set_label(__size)
		self.getWidget("label_permission").set_label(__perm)

		self.getWidget("label_filetype").set_label( \
			GNOME.get_mime_description(self.FILENAME))

		__icon = GNOME.Icon("ICON_SIZE_DIALOG")
		__icon.setSize(100)
		__pixbuf = __icon.getPixbufByFilename(self.FILENAME)
		self.getWidget("image_filetype").set_from_pixbuf(__pixbuf)

		self.getWidget("label_user").set_label( \
			pwd.getpwuid(__stat[stat.ST_UID])[0])
		self.getWidget("label_group").set_label( \
			grp.getgrgid(__stat[stat.ST_GID])[0])

		self.getWidget("label_created").set_label( \
			func.get_formatted_time(__stat[stat.ST_CTIME]))
		self.getWidget("label_modified").set_label( \
			func.get_formatted_time(__stat[stat.ST_MTIME]))

	def update (self, filename=None) :
		if not self.setFilename(filename) :
			self.WIDGET_SELF.hide()
			return False

		if not self.WIDGET_SELF.get_property("visible") :
			self.WIDGET_SELF.show()

		self.updateMethodList()
		self.updateInfo()

		return True

	def setFocus (self) :
		if self.NOTEBOOK.get_current_page() == NOTEBOOK_PAGE_METHOD_LIST :
			self.getWidget("treeview_method_list").grab_focus()
		else :
			self.NOTEBOOK.grab_focus()

	###################################################################
	# EVENT HANDLER
	AVAILABLE_KEYNAME = \
		[ \
			"<Alt>i", \
			"<Alt>m", \
		]
	def on_key_press_event (self, __eventbox, event) :
		__keyname = func.get_keyname(event)

		__page = self.NOTEBOOK.get_current_page()

		if __keyname in self.AVAILABLE_KEYNAME :
			if __keyname == "<Alt>i" :
				self.NOTEBOOK.set_current_page(NOTEBOOK_PAGE_INFORMATION)
			elif __keyname == "<Alt>m" :
				self.NOTEBOOK.set_current_page(NOTEBOOK_PAGE_METHOD_LIST)
				self.PANEL.WIDGET_TREEVIEW.grab_focus()

			return True
		elif __page == NOTEBOOK_PAGE_METHOD_LIST and \
				__keyname in self.PANEL.AVAILABLE_KEYNAMES_IN_PANEL :
			self.PANEL.on_key_press_event(__eventbox, event)

			return True
		else :
			return False

	def on_update (self, button) :
		self.update()

"""
Description
-----------


ChangeLog
---------


Usage
-----


"""

__author__ = "Spike^ekipS <spikeekips@gmail.com>"
__version__=  "0.1"
__nonsense__ = ""

__file__ = "Etc.py"


