# -*- coding: utf-8 -*-
#    Copyright 2005 Spike^ekipS <spikeekips@gmail.com>
#
#       This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import os
import gtk, gobject
from vee import i18n, __OS__, func
from vee.child import __VTE__, vim
from vee.child.vim import Etc, main

_ = i18n.gettext()

GCONF_KEY_FILENAMES_CURRENT_OPENED = "/apps/vee/vim/filenames_current_opened"

COMMAND_INIT = [ \
	"\033:set title titlestring=%F;;;%t\n", \
	"\033:set mouse=nvi\n", \
	"\033:set confirm\n", \
]

COMMAND_OPEN_FILE  = "\033:e %s\n"

VIM_AUTOCMD_FOR_CURRENT_FILENAME = \
	"\033:autocmd BufWinEnter,BufNewFile,BufRead * :r! gconftool-2 --type str --set /apps/vee/vim/filenames_current_opened %s &>/dev/null\n:\n\n"
VIM_AUTOCMD_VALUE_CURRENT_FILENAME = "'%(uid)s;'`pwd`';'%%"

COMMAND_OPEN_FILE = "\033:e %s\n\n"

class egg (__VTE__.egg) :
	UID = None
	ID_GCONF_NOTIFICATION = None
	PATH = "~"

	def __del__ (self) :
		# remove gconf notification.
		self.GCONF_CLIENT.CLIENT.notify_remove(self.ID_GCONF_NOTIFICATION)

		__VTE__.egg.__del__(self)

	def normalizeProperty (self, kwargs=dict()) :
		__VTE__.egg.normalizeProperty(self, kwargs)

		# set base directory and the file(NOT FILES) to be read from vim.
		if not self.PROPERTIES.has_key("filename") :
			self.PROPERTIES["filename"] = str()

		self.PROPERTIES["enable_scrollbar"] = False

		self.LOGGING_ENABLED = False

	def make (self) :
		__VTE__.egg.make(self)

		# add notification to gconf key, 'GCONF_KEY_FILENAMES_CURRENT_OPENED'.
		self.ID_GCONF_NOTIFICATION = \
			self.GCONF_CLIENT.CLIENT.notify_add( \
				GCONF_KEY_FILENAMES_CURRENT_OPENED, self.nf_changed_current_filename)

		if self.GCONF_CLIENT.getValue("vim/use_fileinfo") :
			self.attachMethodList()

	def fork_command (self) :
		__VTE__.egg.fork_command(self)

		# run initial command inside vim.
		if self.GCONF_CLIENT.getValue("vim/use_fileinfo") :
			__COMMAND_INIT = COMMAND_INIT
		else :
			__value = VIM_AUTOCMD_VALUE_CURRENT_FILENAME % {"uid" : self.UID, }
			__COMMAND_INIT = \
				[VIM_AUTOCMD_FOR_CURRENT_FILENAME % main.escape_path(__value), ] + \
					COMMAND_INIT

		for i in __COMMAND_INIT :
			self.WIDGET.feed_child(i)

	def attachMethodList (self) :
		self.METHOD_LIST = Etc.MethodList( \
			vte=self.WIDGET, state=self.STATE)

		self.METHOD_LIST.update(self.PROPERTIES["filename"])

		# attach information panel.
		__box = gtk.HPaned()
		__box.pack1(self.getBox(), True)
		__box.pack2(self.METHOD_LIST.WIDGET_SELF, True)
		__box.show()

		# XXX: weird HAPPENINGS.
		# When the vim was added as notebook page, sometimes it's terminal was
		# broken. I did not find out the exact reason, but i guess except a little
		# dirty hacked code like this.
		gobject.idle_add(self.__resizeHPaned, __box)

		# attach eventbox
		self.BOX = gtk.EventBox()
		self.BOX.add(__box)
		self.BOX.show()

		# connect signal
		self.BOX.connect("event", self.on_box_event)

	def __resizeHPaned (self, box) :
		# XXX: HIDING.
		# If you want to view the information panel, comment the below line.
		box.set_position( \
			self.STATE_LOCAL["obj-vee"].WIDGET_SELF.allocation.width - 300)

	def parse_command (self) :
		(__command, __argument, __environment, __directory, ) = \
			__VTE__.egg.parse_command(self)

		# set base directory and the file(NOT FILES) to be read from vim.
		if self.PROPERTIES.has_key("filename") and \
				self.PROPERTIES["filename"].strip() :

			__directory = os.path.dirname(self.PROPERTIES["filename"])
			if not os.path.isdir(__directory) :
				__directory = os.getenv("HOME")

			# set file
			__argument += (self.PROPERTIES["filename"], )
		else :
			self.PROPERTIES["filename"] = str()

		return (__command, __argument, __environment, __directory, )

	def getTitle (self) :
		__a = __VTE__.egg.getTitle(self)
		if not __a : return str()

		__b = __a.split(";;;")

		if len(__b) < 2 :
			return self.PROPERTIES["filename"]
		else :
			__dirname = os.path.dirname(__OS__.normalize_path(__b[0]))
			if not os.path.isdir(__dirname) :
				__title = "".join(__b[1:])
			else :
				self.PROPERTIES["filename"] = __b[1]
				__title = __b[0]

		return __title

	def getFilenameFromTitle (self) :
		__a = __VTE__.egg.getTitle(self)
		if not __a : return str()

		__b = __a.split(";;;")
		if len(__b) > 1 :
			__filename = __OS__.normalize_path(__b[0])
			if __OS__.is_file(__filename) :
				return __filename

		return str()

	def attached (self, obj_tab) :
		__VTE__.egg.attached(self, obj_tab)

	def getLabelName (self) :
		if self.PROPERTIES["filename"] :
			return self.PROPERTIES["filename"]
		else :
			return _("VIM")

	###################################################################################
	# EVENT HANDLER
	def nf_changed_current_filename (self, client, parm1, entry, parm3) :
		__data = self.GCONF_CLIENT.getValueFromValue(entry.get_value())

		__a = [i.strip() for i in __data.split(";")]
		if len(__a) != 3 :
			return

		if self.UID != __a[0] :
			return

		__filepath = __OS__.normalize_path(__a[2])
		if not __OS__.is_file(__filepath) :
			return

		self.PROPERTIES["filename"] = __filepath

		# update information panel.
		if self.GCONF_CLIENT.getValue("vim/use_fileinfo") :
			self.METHOD_LIST.update(__filepath)

	def on_changed_title (self) :
		# if window title changed in vim, it represent that vim read another files
		# or directory, os when the window title was changed,
		# self.PROPERTIES["filename"] must be altered.
		__filename = self.getFilenameFromTitle()
		if __filename and self.PROPERTIES["filename"] != __filename :
			self.PROPERTIES["filename"] = __filename

	IS_FOCUSED_METHOD_LIST = False
	def on_box_event (self, __eventbox, event) :
		if event.type in [gtk.gdk.KEY_RELEASE, ] :
			__keyname = func.get_keyname(event)
			if __keyname in ["<Alt>bracketright" , ] :
				# To escape weird keystroke event, send '\033' sequence code to vte.
				if self.IS_FOCUSED_METHOD_LIST :
					self.WIDGET.grab_focus()
					self.WIDGET.feed_child("\033")
					self.IS_FOCUSED_METHOD_LIST = False
				else :
					self.WIDGET.feed_child("\033")
					self.METHOD_LIST.setFocus()
					self.IS_FOCUSED_METHOD_LIST = True

				return True

	def on_drag_received_data \
			(self, widget, context, x, y, selection, targetType, time) :

		__filelist = list()
		for i in __OS__.get_list_from_uri_list(selection.data) :
			(__scheme, __filepath, ) = __OS__.get_pathinfo_from_uri(i)
			if __scheme != "file" :
				continue
			if not __OS__.exists(__filepath) :
				continue

			__filelist.append(__filepath)

		for i in __filelist :
			self.WIDGET.feed_child(COMMAND_OPEN_FILE % main.escape_path(i))


"""
Description
-----------


ChangeLog
---------


Usage
-----


"""

__author__ =  "Spike^ekipS <spikeekips@gmail.com>"
__version__=  "0.1"
__nonsense__ = ""

__file__ = "egg.py"


