#!/usr/bin/env python
# -*- coding: utf-8 -*-
#    Copyright 2005 Spike^ekipS <spikeekips@gmail.com>
#
#       This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
import sys, os, pprint, sre

def usage () :
	print \
"""Usage : %s <glade filename> [py filename]
""" % sys.argv[0]

if len(sys.argv) < 2 :
	print "(WW) No glade filename."
	usage()
	sys.exit(1)

if not os.path.exists(sys.argv[1]) :
	print "(WW) No glade filename."
	usage()
	sys.exit(1)

import xml.dom.ext as ext
from xml.dom import minidom
from xml.xpath import Evaluate

def evaluate_path (dom, xpath) :
	if not hasattr(dom, "documentElement") : return list()

	nodes = Evaluate(xpath, dom.documentElement)

	if len(nodes) < 1 : return list()

	return nodes

def get_element_info_signal (element) :
	widget_name = element.parentNode.getAttribute("id")
	widget_class = element.parentNode.getAttribute("class")
	type = element.getAttribute("name")
	handler = element.getAttribute("handler")

	return \
		(str(widget_class), str(widget_name), str(type), str(handler), )

def get_element_info_label (element) :
	widget_name = element.parentNode.getAttribute("id")
	widget_class = element.parentNode.getAttribute("class")
	type = element.getAttribute("name")
	try :
		__string = element.childNodes[0].nodeValue
	except IndexError :
		__string = ""

	return (str(widget_class), str(widget_name), str(__string), str(type))

############################################################################
SRE_STRING = sre.compile("[a-z][a-z]*", sre.IGNORECASE)

TEMPLATE_PY = \
"""# -*- coding: utf-8 -*-
#    Copyright 2005 Spike^ekipS <spikeekips@gmail.com>
#
#       This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

\"\"\"
Description
-----------
This file was generated by createTranslationForGlade.py.

\"\"\"

"""

# make filename
FILENAME_GLADE = sys.argv[1]
__filename = os.path.splitext(FILENAME_GLADE)[0]

__create_py = False
if len(sys.argv) > 2 :
	FILENAME_PY = sys.argv[2]
	if not os.path.exists(FILENAME_PY) :
		__create_py = True

	FD_PY = file(FILENAME_PY, "a")
else :
	FILENAME_PY = "%s.py" % __filename
	if not os.path.exists(FILENAME_PY) :
		__create_py = True

	FD_PY = file(FILENAME_PY, "w")

if __create_py :
	FD_PY.write(TEMPLATE_PY)

####################################################################################
## OLD CONFIG.
####################################################################################
DOM = minidom.parse(FILENAME_GLADE)

# generate for TRANSLATION.
__xpath_window = "/glade-interface/widget"
__xpath_translate = "/glade-interface/widget[@id = '%s']//property[@translatable = 'yes']"
nodes = evaluate_path(DOM, __xpath_window)

for i in nodes :
	__window_name = str(i.getAttribute("id"))
	__a = dict()
	__widget_list = dict()
	for j in Evaluate(__xpath_translate % __window_name, i) :
		(__c, __n, __s, __t, ) = get_element_info_label(j)
		if not __s.strip() : continue
		if not SRE_STRING.findall(__s) : continue

		if not __widget_list.has_key(__n) :
			__widget_list[__n] = list()

		__widget_list[__n].append((__c, __n, __s, __t, ))

	FD_PY.write("LABEL_%s = \\\n" % __window_name.upper())
	pp = pprint.PrettyPrinter(indent=4, stream=FD_PY)
	pp.pprint(__widget_list)
	FD_PY.write("\n")

# generate for SIGNAL HANDLER CONNECTION.
__xpath_window = "/glade-interface/widget"
__xpath_translate = "/glade-interface/widget[@id = '%s']//signal"
nodes = evaluate_path(DOM, __xpath_window)

for i in nodes :
	__window_name = str(i.getAttribute("id"))
	__a = dict()
	__widget_list = dict()
	for j in Evaluate(__xpath_translate % __window_name, i) :
		(__c, __n, __t, __h, ) = get_element_info_signal(j)

		if not __widget_list.has_key(__n) :
			__widget_list[__n] = list()

		__widget_list[__n].append((__c, __n, __t, __h))

	FD_PY.write("SIGNAL_%s = \\\n" % __window_name.upper())
	pp = pprint.PrettyPrinter(indent=4, stream=FD_PY)
	pp.pprint(__widget_list)
	FD_PY.write("\n")

FD_PY.close()

sys.exit(0)

"""
Description
-----------
This script make i18n python script for glade xml file.


ChangeLog
---------


Usage
-----
python createTranslationForGlade.py <glade filename>

"""

__author__ =  "Spike^ekipS <spikeekips@gmail.com>"
__version__=  "0.1"
__nonsense__ = ""

__file__ = "createTranslationForGlade.py"


