#!/usr/bin/env python
# -*- coding: utf-8 -*-
#    Copyright (C) 2005 Spike^ekipS <spikeekips@gmail.com>
#
#    http://vee.kldp.net
#
#       This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import sys, os
import pygtk
pygtk.require("2.0")

from vee import i18n, __OS__
_ = i18n.gettext()

# for scp client.
USE_TWISTED = False
try :
	from twisted.internet import gtk2reactor
	gtk2reactor.install()
	from twisted.internet import reactor

	from twisted.python import threadable
	threadable.init(1)
except ImportError, msg :
	USE_TWISTED = False
else :
	USE_TWISTED = True

try :
	import gtk, gnome
except :
	print _("You need to install pygtk or GTKv2 and PyGnome.")
	sys.exit(1)

from vee import Conf, Conf__, XMLHandler, Window, Options, GConf, func, State, \
	GNOME, WIDGET, child, __VEE__

if __name__ == "__main__" :

	# parse command-line argument.
	__parser = Options.VEE()
	(__OPTIONS, __ARGS) = __parser.parse_args()

	# check options and print error.
	if __OPTIONS.ARG_GEOMETRY :
		if not WIDGET.parse_geometry(__OPTIONS.ARG_GEOMETRY) :
			__parser.error(_("Invalid '--geometry' value, %s") % __OPTIONS.ARG_GEOMETRY)

	# check environment
	__OS__.check_environment( \
		log_directory=os.path.join(Conf.DIR_HOME, "vee"))

	state_global = State.VEE()
	state_global.set_value("list-vee", list())
	state_global.set_value("use_twisted", USE_TWISTED)

	if __OPTIONS.ARG_INSTALL_SCHEMA :
		GConf.verify(True)

	if not GConf.verify() :
		print "(WW) invalid gconf schema. Check gconf."
		__parser.print_help()
		sys.exit(1)

	try :
		gconf_client = GConf.Client(Conf.GCONF_APP_DIR)
	except RuntimeError, msg :
		print "(WW) %s" % msg
		sys.exit(1)
	else :
		state_global.set_value("gconf-client", gconf_client)

	# get locale info.
	(__localedir, __lang) = i18n.getLocale()
	state_global.set_value("locale-dir", __localedir)
	state_global.set_value("lang", __lang)

	# get children
	__config_children = child.get_children()
	__global_config = \
		XMLHandler.GlobalConfig( \
			os.path.join(Conf.DIR_LIB, "child/global.xml") \
		)
	__config_global = __global_config.getAll()

	state_global.set_value("children", __config_children)
	state_global.set_value("children_global", __config_global)

	# add bookmark to state_global
	__bookmark = __VEE__.readBookmark(__config_children)

	# external command
	try :
		__command = XMLHandler.Command(Conf.FILE_COMMAND)
	except IOError :
		__command = XMLHandler.Command()

	state_global.set_value("bookmark", __bookmark)
	state_global.set_value("command", __command)

	GNOME.program_init( \
		PARAM_APP_DATADIR=Conf__.DIR_SHARE, \
		app_name=Conf__.INFO["name"], \
		version=Conf__.INFO["version"], \
	)

	app = Window.VEE( \
		state_global, args_options=__OPTIONS, args_args=__ARGS)

	gtk.gdk.threads_init()
	try :
		if USE_TWISTED :
			reactor.run()
		else :
			gtk.main()
	except :
		sys.exit(1)
	else :
		sys.exit(0)

"""
Description
-----------


ChangeLog
---------


Usage
-----


"""

__author__ =  "Spike^ekipS <spikeekips@gmail.com>"
__version__=  "0.1"
__nonsense__ = ""

__file__ = "vee.py"


